package Abstract;

import java.lang.*;
import java.util.*;
import java.io.*;

/**
* Abstract AddressTable class for store and retain Address.
* The static addresses can be stored to the AddressTable from file reading and
* dynamic addresses can be stored by addAddress method.
* You can extend this class to use 'description' field of Address more structured
* way. The key field is the agent name and the value field is address object.
*/

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public abstract class AddressTable extends Hashtable   {

	/**
	* Default constructor
	*/	
	public AddressTable()  {
	}
	
	/**
	* Same agent id address will be replaced with new one. Assume that agent address
	* can be dynamically changed.
	* @param addr : Address object for the agent
	* @return None
	*/
	public abstract void addAddress(Address addr);
	
	/**
	* Remove an address from the table
	* @param id agent name
	*/
	public abstract void removeAddress(String id);
	
	/**
	* Get an Address object using agent name
	* @param id Agent name
	* @return Address object, the agent has. Null if the address is not defined
	* @exception None
	*/
	public abstract Address getAddress(String id);
	
	/**
	* Get an AddressTable of the same Type address
	* @param type Type definition(e.g. class name) to retrieve
	* @return AddressTable
	* @exception None
	*/
	public abstract AddressTable getAddressTableWithType(String type);
}	
					
		
			