package Abstract;

import java.io.*;
import java.util.*;
import java.lang.*;

/**
* BaseLayer.Connection.ConnectionTable
* A Table with key as an agent name and field value as a ReceiverThread.
* It is always assumed that there's only one ReceiverThread for each
* agent to comminicate with. If you want more than one connection for
* the same agent, you should specify it with a different name.
* When you add a ReceiverThread with the same name, the pre-existing
* thread will be stop and new thread will be added to the table.
* @see ReceiverThread
*/

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
*
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/

public abstract class ConnectionTable extends Hashtable  {

	/**
	* Default constructor
	*/
	public ConnectionTable()  {
	}

	/**
	* Get Connection by agent name. If the name is not exist, null will be returned
	* @param agentName Agent name referenced as key
	* @return ReceiverThread. 'null' if not exist
	*/
	public abstract ReceiverThread getConnection(String agentName);

	/**
	* Add ReceiverThread to the table.
	* If the same name ReceiverThread is exist, and the override is true,the ReceiverThread will be
	* stop and replaced by a new ReceiverThread
	* @param agentName Agent name as a key
	* @param ReceiverThread ReceiverThread as a field
	* @param override Override current ReceiverThread with the same name if true
	*/
	public abstract /*synchronized*/ void addConnection(String agentName,Thread thread,boolean override);

	/**
	* Remove ReceiverThread from the table
	* If the ReceiverThread associated with agent name does not exist, do nothing
	* @param agentName Agent name to be referenced
	*/
	public abstract void removeConnection(String agentName);

	/**
	* Find out whether the receiver is alive
	* @param name receiver thread name
	*/
	public abstract boolean isLiving(String name);
}
