package Abstract;

import java.lang.*;

/**
* The receiver abstract class. Check socket input stream and if a message string 
* is received, construct a message object and invoke Act(Object) method of 
* AgentAction. From beta version, writeMsg(String) will throw Connection exception 
* if IO error or some other error conditions.
* @see BaseLayer.BRecvThread
*/

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 
public abstract class ReceiverThread extends Connection  {
	
  /**
   * Message Queue object. JATLite always assumes that the receiver thread
   * has a message queue. Received messages will be saved to the queue. AgentAction
   * will check the message queue and retrieve if it is not empty.
   * @see Abstract.MessageBuffer
   */
  protected MessageBuffer _queue;
  
  /**
   * Default constructor
   */	
  public ReceiverThread()  {
    _queue = null;
  }
  
  /**
   * Set thread name
   * @param name ReceiverThread name
   */
  public ReceiverThread(String name)  {
    super(name);
    _queue = null;
  }
  
  /**
   * Constructor with thread group, id
   * @param tg Thread group 
   * @param id Thread(agent) name
   */		
  public ReceiverThread(ThreadGroup tg, String name)  {
    super(tg,name);
  }
  
  /**
   * Set message queue
   * @param queue Message Buffer
   */
  public void setMessageQueue(MessageBuffer queue)  {
    _queue = queue;
  }
  
  /**
   * Get message queue
   * @return Message Buffer
   */
  public MessageBuffer getMessageQueue()   {
    return _queue;
  }
  
  /**
   * Get one message from the receiver thread. In general, this method is
   * less efficient than to read several messages at once and menage them.
   */
  public abstract String getOneMessage();
	
  /**
   * Send one message
   * @param msg String to be sent
   * @exception Abstract.ConnectionException will be thrown if IO error occurs.
   */
  public abstract void writeMsg(String msg) throws ConnectionException;
  
  /**
   * Clean up the data members initialized by this thread
   */
  protected abstract void cleanUp();
  
  /**
   * Close all the connections and stop thread
   */
  protected abstract void closeSocket();
  
}