package BaseLayer.Example;

import java.io.*;
import java.util.*;
import java.lang.*;

import BaseLayer.*;

/**
* Extension of BaseLayer.BConnectionTable. It is extended since this example needs
* for the connction table to inform the added/removed receiver thread to GUI.
* A Table with key as an agent name and field as a ReceiverThread
* This always assume that there's only one ReceiverThread for each
* agent to comminicate with. If you want more than one connection for
* the same agent, you should specify it with a different name.
* When you add a ReceiverThread with the same name, the pre-existing
* thread will be stop and new thread will be added to the table.
* @see ReceiverThread
* @version Java(tm) 1.1 beta 12.24.96
*/
/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class ChatterConnectionTable extends BConnectionTable  {

	ChatterServer _action;
	
	/**
	* Default constructor
	*/
	public ChatterConnectionTable(ChatterServer action)  {
		_action = action;
	}
	

	/**
	* Override Add ReceiverThread to the table
	* If the same name ReceiverThread is exist, the ReceiverThread will be
	* stop and replaced by a new ReceiverThread
	* @param agentName Agent name as a key
	* @param ReceiverThread ReceiverThread as a field
	*/
	public synchronized void addConnection(String agentName,Thread thread,boolean override)  {
		super.addConnection(agentName,thread,override);
		if(_action != null)  {
			String args[] = new String[1];
			args[0] = agentName;
			_action.processMessage("AddConnection",args);		
		}	
	}	

	/**
	* Override Remove ReceiverThread from the table
	* If the ReceiverThread associated with agent name does not exist, do nothing
	* @param agentName Agent name to be referenced
	*/
	public void removeConnection(String agentName)  {
		super.removeConnection(agentName);
		if(_action != null)   {
			String args[] = new String[1];
			args[0] = agentName;
			_action.processMessage("RemoveConnection",args);		
		}	
 }	
 
}