package KQMLLayer;

import Abstract.*;
import BaseLayer.*;

import java.net.*;

/**
  *
  * To support complete KQML parsing
  */
/*  
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/  
public class KQMLRecvThread extends BRecvThread   {

	public KQMLRecvThread()  {
		super();
	}
		
	/**
	* Create ReceiverThread with specific thread name
	* @param name Agent name
	*/	
	public KQMLRecvThread(String name)  {
		super(name);
	}
		
	/**
	* Create ReceiverThread with specific thread name and Thread group.
	* Passing thread group is essential when you will create the receiver thread for an applet
	* because of security issue.
	* @param tg Thread group to which this thread will be owned to
	* @param name Agent name
	*/	
	public KQMLRecvThread(ThreadGroup tg,String name)  {
		super(tg,name);
	}
		
		
  /**
  *  Constructor, to be called from the serverthread after accepting
  *  a connection. Note default endWith is '\004'. You can change endWith
  *  character by setEndWith(char) method. Note that the timer will not
  *  be set. To set timer, use setDurationTime(int minutes) method.
  *  <I>Do not use this thread when you want to initiate(conect) to
  *  other server. If you use this constructor the role will be set
  *  as a server.</I> If there is a special reason you should use
  *  this constructor even if your agent role is a server, use setRole(false)
  *  method to set the role as a client.
	* @param client_scoket The socket for the remote client;
	* @param priority the priority of the thread.
	* @param address My address.
	* @param connections the connection table is passed as an argument.
	* @param queue the Message Queue.
	* @param security Security for checking this connection security
	* @exception ConnectionException if error occurs
  */
  public KQMLRecvThread(Socket client_socket,
		     int priority, 
		     Address address,
		     ConnectionTable connections,
		     MessageBuffer queue,
		     Security security)
	throws ConnectionException    {
		super(client_socket,priority,address,connections,queue,security);
	}


/**
*  This constructor is called when an agent want to initialize a connection
*  to the other agents, using host name and port. This constructor is
*  will be useful if you do not want to use any security check._security
*  data member will be set as null. Do not use this constructor if you
*  want security check.
*  @param serverid Agent name to shich the this receiver thread will connect
*  @param host the name of the remote host.
*  @param priority the priority of the thread.
*  @param connections the connection table is passed as an argument.
*  @param queue the Message Queue.
*  @param port the port of the remote host.
*  @param tg Thread group to which this thread will be owned to. Necessary to evade applet security.
* @exception ConnectionException if error occurs
*/
  public KQMLRecvThread(String serverid,
		     String host, 
		     int priority, 
		     Address myaddress,
		     ConnectionTable connections,
		     MessageBuffer queue,
		     int port,
		     ThreadGroup tg)   
  throws ConnectionException   {
  	super(serverid,host,priority,myaddress,connections,queue,port,tg);
  }

  /**
   *  This constructor is called when an agent want to initialize a connection
   *  to the other agents, using Address. Use this constructor if you want
   *  security check.
   *  @param serveraddress The address of the remote host.
   *  @param priority The priority of the thread.
   *  @param myaddress My addres
   *  @param connections the connection table is passed as an argument.
   *  @param queue the Message Queue.
   *  @param security The security object 
   * @exception ConnectionException if error occurs
   */
  public KQMLRecvThread(Address serveraddress,
		     int priority, 
		     Address myaddress,
		     ConnectionTable connections,
		     MessageBuffer queue,
		     Security security,
		     ThreadGroup tg)
    throws ConnectionException   {
    super(serveraddress,priority,myaddress,connections,queue,security,tg);
  }
  
	/**
	 *  This is the only difference between BRecvThread and KQMLRecvThread.
	 *  Since KQML may contains arbitrary ascii with double quotes or #digit,
	 *  _endWith character may be contained. Skip fixed number of #digit and
	 *  double quotes.
	 */
  protected int findEndLocation(char[] buffer,
																int startPosition,
																int maxLen)  {
    
    int unitLen = 0;

    while(startPosition < maxLen)  {
    
    	char c = buffer[startPosition++];
    	if(c == '#')  {
    		unitLen++;
    		
    		String binaryno = "";
    		while(startPosition < maxLen) {
    			c = buffer[startPosition++];
    			unitLen++;
    			
    			if((c >= '0') && (c <= '9'))  {
    				binaryno = binaryno + c;
    			}
    			else
		 			if(c == '\"')  {
		 				if(!binaryno.equals(""))  {
    					int nofbin = (Integer.valueOf(binaryno)).intValue();					
    					startPosition += nofbin;
    					unitLen += nofbin;
    					// out of bound
    					if(startPosition > maxLen)  {
    						return -1;
    					}	
    					break;
    				}
    				// wrong format
    				else  {
    					return -1;
    				}		
    			}
    			// not a string
    			else  {
    				break;
    			}		
    		}		
    	}
    	else
    	if(c == '\"')  {
    		unitLen++;
    		
    		while(true)  {
    			if(startPosition > maxLen)  {
    				return -1;
    			}	
    		
    			c = buffer[startPosition++];
    			unitLen++;
    			
    			if(c == '\"')  {
    				break;
    			}
    			else
    			if(c == '\\')  {
    				startPosition++;
    				unitLen++;
    			}
    		}
    	}				
    	else
    	if(c == _endWith)  {
				return unitLen;
    	}
    	else  {	
				unitLen++;
      }
    }
    
    return -1;
  }

}  