package ProtocolLayer;

import java.lang.Thread;
import Abstract.*;

/**
* Connector timer thread. Check communication between an applet and a internet service
* server and if time limit meets, stop ConnectorThread and stop itself.
*/

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class ConnectorTimerThread extends Thread  {

	/**
	* Time to be set
	*/
  int _waitTime;
  
  /**
  * Receiver thread to be controlled by the timer
  */
  ConnectorThread _thread;
  
  /**
  * Record starting time
  */
  long start;
  
  /**
  * Check the current time is in time or over
  */
  public boolean inTime;

	/**
	* Constructor
	* @param t ReceiverThread to be controlled by the timer
	* @param waitingTimeinMinutes Time to be set. Unit is minutes.
	*/
  public ConnectorTimerThread(ConnectorThread t,int waitingTimeinMinutes)  {
    _waitTime = waitingTimeinMinutes;
    _thread = t;
    resetTime();
  }

	/**
	* The timer will check the time for every minute. 
	*/
  public void run()  {
    while(true)  {
    	try  {
	   		long currentTime = System.currentTimeMillis();
        long difference = (currentTime - start) / 60000;
        if(difference >= _waitTime)  {
        	if(_thread.isAlive())  {
           	_thread.disconnectCon(); 
          }
          stop();
          join();
        }
        else  {
        	sleep(60000);
        }	
      } catch (Exception e)  {
      	System.out.println(e.toString());
      }	  
    }
  }
  
  /**
    * Reset timer
    */
  public void resetTime()  {
  	start = System.currentTimeMillis();
  }	

}