package ProtocolLayer.Example.CAD;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class AxisSymCanvas extends ExtrusionCanvas implements MouseListener {
	
	public AxisSymCanvas()  {
		super();
	}
	
	protected Point findNearestPoint(Point p)  {
		Point op = super.findNearestPoint(p);
		Point np = new Point(op.x,op.y);
		
		if(op.x < _res*_unitPixel / 2)  {
			np.x = _res*_unitPixel;
		}
		return np;
	}
	
	public void setPoints(Point2d[] edge,float scale)  {
		_point.removeAllElements();
		int tsize = _res*_unitPixel/2;
		
		for(int i=0; i < edge.length; i++)  {
			int x = (int)((edge[i]._x / scale) + tsize);
			int y = (int)((edge[i]._y / scale) + tsize);
			_point.addElement(findNearestPoint(new Point(x,y)));
		}		
		
		repaint();
	}

	public void paint(Graphics g)  {
		Color old = g.getColor();
		
		g.setColor(Color.green);
		
		
		int tsize = _res*_unitPixel;
		
		int startx = tsize/2;
		int starty = 0;
		
		
		for(int i=0; i < _res; i++)  {
			if((i % 5) == 0)  {
				g.setColor(Color.gray);	
				g.drawLine(startx,starty,tsize,starty);
				g.setColor(Color.green);
			}	
			else  {
				g.drawLine(startx,starty,tsize,starty);
			}
				
			starty += _unitPixel;
		}
		
		startx = tsize/2;
		starty = 0;
		for(int i=_res/2; i < _res; i++)  {
			if((i % 5) == 0)  {
				g.setColor(Color.gray);	
				g.drawLine(startx,starty,startx,tsize);
				g.setColor(Color.green);
			}
			else  {
				g.drawLine(startx,starty,startx,tsize);
			}	
			startx += _unitPixel;
		}
		
		g.setColor(Color.red);
		g.drawLine(tsize/2,0,tsize/2, tsize);
		g.drawLine(tsize/2,tsize/2,tsize,tsize/2);
		g.drawString("0",tsize/2 - 15,tsize/2+15);
		g.drawString("Y",tsize - 30, tsize/2+15);
		g.drawString("Z",tsize/2+15,30);
		
		int size = _point.size();
		for(int i=0; i < size; i++)  {
			Point s = (Point)_point.elementAt(i);
			g.drawRect(s.x-2,s.y-2,4,4);
		}	
		
		if(size > 1)  {
			g.setColor(Color.black);
			for(int i=0; i < size -1; i++)  {
				Point from = (Point)_point.elementAt(i);
				Point to = (Point)_point.elementAt(i+1);
				g.drawLine(from.x,from.y,to.x,to.y);
			}
		}
			
		g.setColor(old);	
	}
}	