package ProtocolLayer.Example.CAD;

import java.awt.*;
import java.awt.event.*;

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class AxisSymFrame extends Frame implements ActionListener {

	ViewCanvas _parent = null;
	AxisSymCanvas _canvas;
	TextField _scale;
	MatrixPanel _matPanel;
	CADObject _obj = null;
	CADObject _clone = null;
	
	public AxisSymFrame(ViewCanvas parent)  {
		super("JATLite AxisSymmerty Model");
		_parent = parent;
		init();
		setSize(500,600);
		setVisible(true);
	}	

	public AxisSymFrame(ViewCanvas parent,AxisSym ext)  {
		this(parent);
		if(ext != null)  {
			_obj = ext;
			_clone = new AxisSym();
			ext.copyTo(_clone);;
			_clone.reset();
//			ext.reset();
			Rectangle rect = ext.getBoundingBox();
			int width = rect.width;
			int height = rect.height;
			
			int canvaswidth = _canvas.size().width * 2 /3;
			int canvasheight = _canvas.size().height * 2 /3;
			float scale = canvaswidth/width;
			float scaley = canvasheight/height;
			if(scaley >= scale)  {
				scale = scaley;
			}	
				
			_scale.setText(Float.toString(1/scale));
			Point3d p = ext.getOrigin();
			_matPanel.setOrigin(p);
			p = ext.getOrientation();
			_matPanel.setOrientation(p);
		
			_canvas.setPoints(ext.getEdge(),1/scale);
				
		}	
	}		
		

	public AxisSymFrame()  {
		super("JATLite AxisSymmerty Model");
		init();
		setSize(500,600);
		setVisible(true);
	}	

	protected void init()  {
		_matPanel = new MatrixPanel();
		add(_matPanel,"North");
		
		_canvas = new AxisSymCanvas();
		add(_canvas,"Center");
		
		Panel bpanel = new Panel();
		
		Label l = new Label("Scale");
		bpanel.add(l);
		_scale = new TextField("1.0",8);
		bpanel.add(_scale);
		Button clear = new Button("Clear");
		clear.addActionListener(this);
		bpanel.add(clear);
		Button ok = new Button("OK");
		ok.addActionListener(this);
		bpanel.add(ok);
		Button cancel = new Button("Cancel");
		cancel.addActionListener(this);
		bpanel.add(cancel);
		
		add(bpanel,"South");
	}
	
	
	public void actionPerformed(ActionEvent evt)  {
		String cmd = evt.getActionCommand();
			
		if("Clear".equals(cmd))  {
			_canvas.clearPoints();
		}			
		else
		if("OK".equals(cmd))  {
			if(_parent == null)  {
				return;
			}
				
			String scalestr = _scale.getText();
			float scale = 0;
			try  {
				scale = Float.valueOf(scalestr).floatValue();
			} catch (Exception se)  {
				System.out.println(se.toString());
				return;
			}		
				
			if(scale <= 0)  {
				System.out.println("Depth should be greater than 0.0");
				return;
			}	
			
			Point2d[] p = _canvas.getPoints();
			for(int i=0; i < p.length; i++)  {
				p[i]._x *= scale;
				p[i]._y *= scale;
			}
			
			Point3d origin = _matPanel.getOrigin();
			Point3d orientation = _matPanel.getOrientation();
			
			AxisSym ext = new AxisSym(p,origin,orientation);
			ext.setSelected(true);
			if(_obj != null)  {
				_parent.removeCADObject(_obj);
			}
				
			_parent.reset();
			_parent.addCADObject(ext);
			dispose();		
		}	
		else 
		if("Cancel".equals(cmd)) {
			dispose();
		}	
	}
}