package ProtocolLayer.Example.CAD;

import java.awt.*;
import java.io.*;
import java.util.*;

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
*
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/

public class Box extends CADObject  {

	public Box()  {
		super();
		_type = CADObject.BOX;
	}

	// default origin(0,0,0) and orientation
	public Box(float width, float height, float depth)  {
		super(width,height,depth);
		_type = CADObject.BOX;
	}


	public Box(float width, float height, float depth, Point3d origin, Point3d rotation)  {
		super(width,height,depth,origin,rotation);
		_type = CADObject.BOX;
	}

	public Box(float width, float height, float depth, float ox,float oy,float oz, float thetax,float thetay, float thetaz)  {
		super(width,height,depth,ox,oy,oz,thetax,thetay,thetaz);
		_type = CADObject.BOX;
	}

	public Box(float[] matelm, float width, float height, float depth)  {
		super(matelm,width,height,depth);
		_type = CADObject.BOX;
	}

	public Box(Matrix3D mat,float width, float height, float depth)  {
		super(mat,width,height,depth);
		_type = CADObject.BOX;
	}

	public void setVertex()  {
		_totalNo = 8;

		if(_vertex == null)   {
			// vertex are fixed and points are changed
			_vertex = new Point3d[_totalNo];
			_point = new IPoint3d[_totalNo];
			for(int i=0; i < _totalNo; i++)  {
				_vertex[i] = new Point3d();
				_point[i] = new IPoint3d();
			}
		}
System.out.println("Vertex changed");
		_vertex[0]._x = _dimX / 2;
		_vertex[0]._y = _dimY / 2;
		_vertex[0]._z = - (_dimZ /2 );

		_vertex[1]._x = - _dimX / 2;
		_vertex[1]._y =  _dimY / 2;
		_vertex[1]._z = - (_dimZ /2 );

		_vertex[2]._x = - (_dimX / 2);
		_vertex[2]._y =  - _dimY / 2;
		_vertex[2]._z = - (_dimZ /2 );

		_vertex[3]._x =  _dimX / 2;
		_vertex[3]._y =  - (_dimY / 2);
		_vertex[3]._z = - (_dimZ /2 );

		_vertex[4]._x = _dimX / 2;
		_vertex[4]._y = _dimY / 2;
		_vertex[4]._z = (_dimZ /2 );

		_vertex[5]._x = - _dimX / 2;
		_vertex[5]._y =  _dimY / 2;
		_vertex[5]._z = (_dimZ /2 );

		_vertex[6]._x = - (_dimX / 2);
		_vertex[6]._y =  - _dimY / 2;
		_vertex[6]._z = (_dimZ /2 );

		_vertex[7]._x =  _dimX / 2;
		_vertex[7]._y =  - (_dimY / 2);
		_vertex[7]._z = (_dimZ /2 );
	}

	public void draw(Graphics g)  {
		if((_dimX != 0) && (_dimY != 0) && (_dimZ != 0))  {
			_mat.transform(_vertex,_point,_totalNo);
			Color oldcolor = g.getColor();
			g.setColor(_color);

			g.drawLine(_point[0]._x,_point[0]._y,_point[1]._x,_point[1]._y);
			g.drawLine(_point[1]._x,_point[1]._y,_point[2]._x,_point[2]._y);
			g.drawLine(_point[2]._x,_point[2]._y,_point[3]._x,_point[3]._y);
			g.drawLine(_point[3]._x,_point[3]._y,_point[0]._x,_point[0]._y);
			g.drawLine(_point[4]._x,_point[4]._y,_point[5]._x,_point[5]._y);
			g.drawLine(_point[5]._x,_point[5]._y,_point[6]._x,_point[6]._y);
			g.drawLine(_point[6]._x,_point[6]._y,_point[7]._x,_point[7]._y);
			g.drawLine(_point[7]._x,_point[7]._y,_point[4]._x,_point[4]._y);
			g.drawLine(_point[0]._x,_point[0]._y,_point[4]._x,_point[4]._y);
			g.drawLine(_point[1]._x,_point[1]._y,_point[5]._x,_point[5]._y);
			g.drawLine(_point[2]._x,_point[2]._y,_point[6]._x,_point[6]._y);
			g.drawLine(_point[3]._x,_point[3]._y,_point[7]._x,_point[7]._y);

			if(_isSelected)  {
				g.setColor(Color.red);
				for(int i=0; i < 8; i++)  {
					g.drawRect(_point[i]._x -2, _point[i]._y -2, 4,4);
				}
			}

			g.setColor(oldcolor);

		}
	}

	public String toString()  {
		String myself = "Box\n";
		myself += super.toString();
		return myself;
	}

	// print out current orientation, origin,width, height, depth, deliminated with ','
	public void writeToFile(DataOutputStream out) throws IOException {
		out.writeChars("Box\n");
		//super.writeToFile(out);
 	}
}
