package ProtocolLayer.Example.CAD;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class ButtonBarOld extends Canvas  {

	ButtonBarAppletOld _parent;
	Vector _buttons;
	int _insetX = 2,_insetY = 5;
	ImgButtonOld _currentButton = null;
	ImgButtonOld _eraseButton = null;
	
	Font _tipFont = new Font("Dialog",Font.PLAIN,10); 
	int _currentX = 4;int _currentY = 4;
	boolean _mouseEntered = false;
	
//	Graphics _dbuffer = null;
	
	// Image button coordination
	public ButtonBarOld(ButtonBarAppletOld parent,int width, int height)  {
		_parent = parent;
		_buttons = new Vector();
		
		_currentX += _insetX;
		_currentY += _insetY;
		setBackground(Color.white);
		
		resize(width + _insetX * 2,height + _insetY*2);
	}

	
	public void addButton(String buttonId,Image imgd, Image imga,Image imgc,Image imgi)  {
		ImgButtonOld b = new ImgButtonOld(buttonId,imgd,imga,imgc,imgi,this);
		int imgWidth = imgd.getWidth(this);
		b.setBounds(new Rectangle(_currentX,_currentY,imgWidth,imgd.getHeight(this)));
		_currentX = _currentX + _insetX + imgWidth;
		_buttons.addElement(b);
		_currentButton = b;
		
		repaint();
	}	

	public void addButton(String buttonId,Image[] img)  {
		ImgButtonOld b = new ImgButtonOld(buttonId,img,this);
		int imgWidth = img[0].getWidth(this);
		b.setBounds(new Rectangle(_currentX,_currentY,imgWidth,img[0].getHeight(this)));
		_currentX = _currentX + _insetX + imgWidth;
		_buttons.addElement(b);
		_currentButton = b;
		
		repaint();
	}	
		
	public void addSpace()  {
		_currentX += _insetX;
	}
				
	public ImgButtonOld getButtonByName(String name)  {
		ImgButtonOld retbtn = null;
		Enumeration e = _buttons.elements();
		while(e.hasMoreElements())  {
			ImgButtonOld b = (ImgButtonOld)e.nextElement();
			if(b.getName().equals(name))  {
				return b;
			}
		}
		return retbtn;		
	}
			
	public void setPersistentButton(String buttonName,int mode)  {
		ImgButtonOld b = getButtonByName(buttonName);
		if(b != null)  {
			b.setPersistentMode(mode);
		}
	}
							
	public void getButtonByPosition(int x, int y)  {
		if(!_mouseEntered)  {
			return;
		}
		
		Enumeration e = _buttons.elements();
		while(e.hasMoreElements())  {
			ImgButtonOld b = (ImgButtonOld)e.nextElement();
			if(b.contains(x,y))  {
				_currentButton = b;
				return;
			}
		}	
		
		_currentButton = null;
	}
			
	
 	public boolean mouseMove(Event e,int x,int y) {
 	  // move on the same button 
		if(!inactivateButton(x,y)) return true;
 		// return to default
 			
 		getButtonByPosition(x,y);
 		if(_currentButton != null)  {
			_currentButton.setState(1);			
			repaint();	
		}	
		return true;
	}
	
				
 	public boolean mouseUp(Event e,int x,int y) {
 	  // move on the same button 
		inactivateButton(x,y); 	
 		getButtonByPosition(x,y);
 		if(_currentButton != null)  {
			_currentButton.setState(1);			
			repaint();
			
			// process with name and event
			_parent.processButtonReleased(_currentButton.getName(),x,y);
		}	
		return true;
	}
	
 	public boolean mouseDown(Event e,int x, int y) {
 	  // move on the same button 
		inactivateButton(x,y); 	
 		getButtonByPosition(x,y);
 		if((_currentButton != null) && (!_currentButton.isDisabled())) {
			_currentButton.setState(2);			
			repaint();
			
			// process with name and event
			_parent.processButtonPressed(_currentButton.getName(),x,y);
		}	

 		return true;
 	}

 	public boolean mouseEnter(Event e,int x,int y) {
 		_mouseEntered = true;
 		getButtonByPosition(x,y);
 		if(_currentButton != null)  {
			_currentButton.setState(1);			
			repaint();
		}	
		
		return true;
 	}

 	public boolean mouseExit(Event e,int x,int y) {
 		_mouseEntered = false;
 	  // move on the same button 
 		
		if(_currentButton != null)  {
			_currentButton.setState(0);
			_eraseButton = _currentButton;
		}
		
		repaint();
		
		return true;
 	}

	// if act on the same button, return false. Otherwise return true
	protected boolean inactivateButton(int x,int y)   {
 		if(_currentButton != null) {
 			if(_currentButton.contains(x,y))  {
 				return false;
 			}
 			else  {
 				_eraseButton = _currentButton;
	 			_eraseButton.setState(0);
 			}
 		}
 		
 		return true;
	}
	
	public void paint(Graphics g)  {	
		int x = size().width;
		int y = size().height;
		
		g.setColor(Color.black);
		g.drawLine(3,3,x-6,3);
		g.drawLine(0,y-1,x,y-1);
		g.drawLine(3,3,3,y - 6);
		g.drawLine(x -1,0,x-1,y);

		g.setColor(Color.gray);
		g.drawLine(2,2,x-4,2);
		g.drawLine(1,y - 2,x-2,y -2);
		g.drawLine(2,2,2,y - 4);
		g.drawLine(x-2,1,x-2,y - 2);
		
		g.setColor(Color.lightGray);
		g.drawLine(0,0,x,0);
		g.drawLine(3,y-4,x -6,y-4);
		g.drawLine(0,0,0,y);
		g.drawLine(x-4,3,x-4,y-6);
		
		g.setColor(Color.white);
		g.drawLine(1,1,x-2,1);
		g.drawLine(2,y -3,x-4,y-3);
		g.drawLine(1,1,1,y - 2);
		g.drawLine(x-3,2,x-3,y - 4);
		
		Enumeration ee = _buttons.elements();
		while(ee.hasMoreElements())  {
			ImgButtonOld b = (ImgButtonOld)ee.nextElement();
			g.drawImage(b.getImage(),b.getX(),b.getY(),this);
		}	
	}


	public void update(Graphics g)  {	

		FontMetrics _fMetrics = getFontMetrics(_tipFont);
		int height = _fMetrics.getHeight() + 2;
	
		if(_eraseButton != null)  {
			int width = _fMetrics.stringWidth(_eraseButton.getName()) + 4;
		
			int startx = _eraseButton.getX() + 2 * _eraseButton.getWidth() / 3 ;
			int starty = _eraseButton.getY() + 3 * _eraseButton.getHeight() / 4;
		
			g.clearRect(startx,starty,width + 1,height + 1);
			
			Enumeration ee = _buttons.elements();
			while(ee.hasMoreElements())  {
				ImgButtonOld b = (ImgButtonOld)ee.nextElement();
				int x = b.getX();
				if((x <= (startx + width +1)) && (x >= startx))  {
					g.drawImage(b.getImage(),b.getX(),b.getY(),this);
				}	
			}	
		
//			int x = _eraseButton.getX();
//			int y = _eraseButton.getY();
//			Rectangle r = _eraseButton.getTipRectangle();
//			g.copyArea(r.x + 1200,r.y,r.width + 1,r.height + 1,-1200,0);
			g.drawImage(_eraseButton.getImage(),_eraseButton.getX(),_eraseButton.getY(),this);
			_eraseButton = null;
		}
	
		
		if(_currentButton != null)  {
		
			int x = _currentButton.getX();
			int y = _currentButton.getY();
			g.drawImage(_currentButton.getImage(),x,y,this);
System.out.println("Image updated");			
			String id = _currentButton.getName();
			if(((_currentButton.getState() == 1) || 
			    (_currentButton.getState() == 2)) && 
			    (id != null)) {
			
				int width = _fMetrics.stringWidth(id) + 4;
			
				int startx = _currentButton.getX() + 2 * _currentButton.getWidth() / 3;
				int starty = _currentButton.getY() + 3 * _currentButton.getHeight() /4;
			
				Color old = g.getColor();

				// 1200 will be enough?
//				g.copyArea(startx,starty,width + 1,height + 1,1200,0);
//				_dbuffer.copyArea(startx,starty,width + 1, height + 1,startx,starty);
				
				g.setColor(Color.yellow);
				_currentButton.setTipRectangle(startx,starty,width,height);
				g.fillRect(startx,starty,width,height);
				g.setColor(Color.black);
				g.drawRect(startx,starty,width,height);
				g.drawString(id,startx + 2,starty + height - 4);
System.out.println("Draw Tip");			
				
				g.setColor(old);
			}
		}
	}	
}