package ProtocolLayer.Example.CAD;

import java.awt.*;
import java.io.*;
import java.util.*;


/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
*
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/

public class Cone extends CADObject  {

	static final int _discNo = 12;

	public Cone()  {
		super();
		_type = CADObject.CONE;

	}

	// default origin(0,0,0) and orientation
	public Cone(float xr, float yr, float depth)  {
		super(xr,yr,depth);
		_type = CADObject.CONE;
	}


	public Cone(float xr, float yr, float depth, Point3d origin, Point3d rotation)  {
		super(xr,yr,depth,origin,rotation);
		_type = CADObject.CONE;
	}

	public Cone(float xr, float yr, float depth, float ox,float oy,float oz, float thetax,float thetay, float thetaz)  {
		super(xr,yr,depth,ox,oy,oz,thetax,thetay,thetaz);
		_type = CADObject.CONE;

	}

	public Cone(float[] matelm, float xr, float yr, float depth)  {
		super(matelm,xr,yr,depth);
		_type = CADObject.CONE;
	}

	public Cone(Matrix3D mat,float width, float height, float depth)  {
		super(mat,width,height,depth);
		_type = CADObject.CONE;

	}

	public void setVertex()  {
		if(_vertex == null)  {
			_totalNo = _discNo + 1;

			_vertex = new Point3d[_totalNo];
			_point = new IPoint3d[_totalNo];
			for(int i=0; i < (_totalNo); i++)  {
				_vertex[i] = new Point3d();
				_point[i] = new IPoint3d();
			}
		}

		double unit = (2 * Math.PI / _discNo);
		int i=0;
		for(; i < _discNo; i++)  {
			double theta = i * unit;
			double r = Math.pow((Math.cos(theta)/_dimX),2) + Math.pow((Math.sin(theta)/_dimY),2);
			r = 1/r;
			r = Math.pow(r,0.5);
			_vertex[i]._x = (float)(r * Math.cos(theta));
			_vertex[i]._y = (float)(r * Math.sin(theta));
			_vertex[i]._z = - _dimZ /2 ;
		}

		_vertex[i]._x = 0;
		_vertex[i]._y = 0;
		_vertex[i]._z = _dimZ /2 ;
	}

	public void draw(Graphics g)  {
		if((_dimX != 0) && (_dimY != 0) && (_dimZ != 0))  {
			Color oldcolor = g.getColor();
			g.setColor(_color);

			_mat.transform(_vertex,_point,_totalNo);
			for(int i=0; i < (_discNo-1); i++)  {
				g.drawLine(_point[i]._x,_point[i]._y,_point[i+1]._x,_point[i+1]._y);
				g.drawLine(_point[i]._x,_point[i]._y,_point[_discNo]._x,_point[_discNo]._y);
			}
			g.drawLine(_point[0]._x,_point[0]._y,_point[_discNo-1]._x,_point[_discNo-1]._y);
			g.drawLine(_point[_discNo]._x,_point[_discNo]._y,_point[_discNo-1]._x,_point[_discNo-1]._y);

			if(_isSelected)  {
				g.setColor(Color.red);
				for(int i=0; i < _totalNo; i++)  {
					g.drawRect(_point[i]._x -2, _point[i]._y -2, 4,4);
				}
			}


			g.setColor(oldcolor);

		}
	}


	public String toString()  {
		String myself = "Cone\n";
		myself += super.toString();
		return myself;
	}

	// print out current orientation, origin,width, height, depth, deliminated with ','
	public void writeToFile(DataOutputStream out) throws IOException {
		out.writeChars("Cone\n");
//		super.writeToFile(out);
 	}

}
