package ProtocolLayer.Example.CAD;

import java.awt.*;
import java.io.*;
import java.util.*;

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class Extrusion extends CADObject  {
	Point2d[] _edge = null;
	
	public Extrusion()  {
		super();
		_type = CADObject.EXTRUSION;
		// vertex are fixed and points are changed

	}		
	
	// default origin(0,0,0) and orientation
	public Extrusion(Point2d[] edge, float depth)  {
		this();
		int size = edge.length;
		_totalNo = size * 2;
		_edge = new Point2d[size];
		for(int i=0; i < size; i++)  {
			_edge[i] = new Point2d();
			_edge[i]._x = edge[i]._x;
			_edge[i]._y = edge[i]._y;
		}
			
		_vertex = new Point3d[_totalNo];
		_point = new IPoint3d[_totalNo];
		for(int i=0; i < _totalNo; i++)  {
			_vertex[i] = new Point3d();
			_point[i] = new IPoint3d();
		}	
		
		_dimZ = depth;
		setVertex();
	}
	
	public Point2d[] getEdge()  {
		return _edge;
	}
	
	public Extrusion(Point2d[] edge, float depth, Point3d origin, Point3d rotation)  {
		this(edge,depth);
		rotate(rotation);
		translate(origin);
		_mat.copyTo(_initConfig);
		
	}
	
	public Extrusion(Point2d[] edge, float depth, float ox,float oy,float oz, float theatx,float thetay, float thetaz)  {
		this(edge,depth);
		rotate(theatx,thetay,thetaz);
		translate(ox,oy,oz);
		_mat.copyTo(_initConfig);
		
	}
		
	public Extrusion(float[] matelm, Point2d[] edge, float depth)  {
		this(edge,depth);
		_mat.setElement(matelm);
		_mat.copyTo(_initConfig);
	}
	
	public Extrusion(Matrix3D mat,Point2d[] edge, float depth)  {
		this(edge,depth);
		mat.copyTo(_mat);
		mat.copyTo(_initConfig);
	}
	
	// returns original configuration. Width,depth,height is current one		
	public Object clone()  {
		Extrusion e = new Extrusion(_initConfig,_edge, _dimZ);
		e.setColor(_color);
		e.setType(_type);
		return e;
	}
	
	public void copyTo(Extrusion e)  {
		super.copyTo(e);
		e._dimZ = _dimZ;
		for(int i=0; i < _edge.length; i++)  {
			e._edge[i]._x = _edge[i]._x;
			e._edge[i]._y = _edge[i]._y;
		}
	}			
			
	public float getDX()  {
		return 0;
	}
	
	public float getDY()  {
		return 0;
	}
	
	public void setVertex()  {	
		int size = _totalNo/2;
		for(int i=0; i < size; i++)  {
			_vertex[i]._x = _edge[i]._x;
			_vertex[i]._y = _edge[i]._y;
			_vertex[i]._z = - (_dimZ /2 );

			_vertex[i+size]._x = _edge[i]._x;
			_vertex[i+size]._y = _edge[i]._y;
			_vertex[i+size]._z = _dimZ /2;
		}
	}

	public void draw(Graphics g)  {
		if((_edge != null) && (_dimZ != 0))  {
//System.out.println(_mat.toString());		
			Color oldcolor = g.getColor();
			g.setColor(_color);
			_mat.transform(_vertex,_point,_totalNo);
			int size = _totalNo/2;
			
			// draw poliline
			for(int i=0; i < (size-1); i++)  {
				g.drawLine(_point[i]._x,_point[i]._y,_point[i+1]._x,_point[i+1]._y);
				g.drawLine(_point[i+size]._x,_point[i+size]._y,_point[i+size +1]._x,_point[i+size+1]._y);
				g.drawLine(_point[i]._x,_point[i]._y,_point[i+size]._x,_point[i+size]._y);
			}
			// closing	
			g.drawLine(_point[0]._x,_point[0]._y,_point[size-1]._x,_point[size-1]._y);
			g.drawLine(_point[size-1]._x,_point[size-1]._y,_point[_totalNo-1]._x,_point[_totalNo-1]._y);
			g.drawLine(_point[size]._x,_point[size]._y,_point[_totalNo-1]._x,_point[_totalNo-1]._y);
			
			if(_isSelected)  {
				g.setColor(Color.red);
				for(int i=0; i < _totalNo; i++)  {
					g.drawRect(_point[i]._x -2, _point[i]._y -2, 4,4);
				}
			}	
			
			
			g.setColor(oldcolor);
		}			
	}
	
	public String toString()  {
		String myself = "Extrusion\n";
		myself = myself + _color.getRed() + "," + _color.getGreen() + "," + _color.getBlue() + "\n";
		myself = myself + _initConfig.toString() + "\n";
		myself = myself + _edge.length + "\n";
		int i = 0;
		for(; i < _totalNo/2 -1; i++)  {
			myself = myself + _edge[i]._x + "," + _edge[i]._y + ",";
		}
		myself = myself + _edge[i]._x + "," + _edge[i]._y + "\n";
		myself = myself + _dimZ + "\n"; 
		return myself;
	}
		
	// print out current orientation, origin,width, height, depth, deliminated with ','
	public void writeToFile(DataOutputStream out) throws IOException {
		out.writeChars("Extrusion\n");
		// color property
		out.writeInt(_color.getRed()); out.writeInt(_color.getGreen()); out.writeInt(_color.getBlue());
		out.writeFloat(_initConfig.xo);	out.writeFloat(_initConfig.xx);	out.writeFloat(_initConfig.xy);	out.writeFloat(_initConfig.xz);
		out.writeFloat(_initConfig.yo);	out.writeFloat(_initConfig.yx);	out.writeFloat(_initConfig.yy);	out.writeFloat(_initConfig.yz);
		out.writeFloat(_initConfig.zo);	out.writeFloat(_initConfig.zx);	out.writeFloat(_initConfig.zy);	out.writeFloat(_initConfig.zz);
		out.writeInt(_edge.length);
		for(int i=0; i < _edge.length; i++)  {
			out.writeFloat(_edge[i]._x);out.writeFloat(_edge[i]._y); 
		}	
		out.writeFloat(_dimZ);
 	}
	
	public void writeToTxtFile(PrintWriter out)   {
		out.print(toString());
	}
	
	// Box is already read from the main process
	public void readFromFile(DataInputStream in) throws IOException {
		_color = new Color(in.readInt(),in.readInt(),in.readInt());	
		
		float[] matelm = new float[12];
		for(int i=0; i < 12; i++)  {
			matelm[i] = in.readFloat();
		}

		if(_mat != null)  {
			_mat.setElement(matelm);
		}			
		else  {
			_mat = new Matrix3D(matelm);
		}	
		
		if(_initConfig != null)  {
			_initConfig.setElement(matelm);
		}			
		else  {
			_initConfig = new Matrix3D(matelm);
		}	
		
		int size = in.readInt();
		_totalNo = size * 2;
		_edge = new Point2d[size];
		
		for(int i=0; i < size; i++)  {
			_edge[i] = new Point2d();
			_edge[i]._x = in.readFloat();
			_edge[i]._y = in.readFloat();
		}
			
		_vertex = new Point3d[_totalNo];
		_point = new IPoint3d[_totalNo];
		for(int i=0; i < _totalNo; i++)  {
			_vertex[i] = new Point3d();
			_point[i] = new IPoint3d();
		}	
			
		_dimZ = in.readFloat();	
		setVertex();
		
	}
	
	public void readFromTxtFile(ObjectInputStream in) throws IOException {
		float[] matelm = new float[12];
	
		String line = in.readLine();
		StringTokenizer token = new StringTokenizer(line,",");
		if(token.countTokens() != 3)  {
			return;
		}
		int r,g,b;
		r = Integer.valueOf(token.nextToken()).intValue();
		g = Integer.valueOf(token.nextToken()).intValue();
		b = Integer.valueOf(token.nextToken()).intValue();
		
		_color = new Color(r,g,b);
		
		token = new StringTokenizer(line,",");
		if(token.countTokens() != 12)  {
			return;
		}
		
		for(int i=0; i < 12; i++)  {
			matelm[i] = Float.valueOf(token.nextToken()).floatValue();
		}
		
		if(_mat != null)  {
			_mat.setElement(matelm);
		}			
		else  {
			_mat = new Matrix3D(matelm);
		}	
		
		if(_initConfig != null)  {
			_initConfig.setElement(matelm);
		}			
		else  {
			_initConfig = new Matrix3D(matelm);
		}	
		
		
		line = in.readLine();
		
		int size = Integer.valueOf(line).intValue();
		_totalNo = size * 2;
		_edge = new Point2d[size];
		
		for(int i=0; i < size; i++)  {
			_edge[i] = new Point2d();
			_edge[i]._x = Float.valueOf(token.nextToken()).floatValue();
			_edge[i]._y = Float.valueOf(token.nextToken()).floatValue();
		}
		
		_vertex = new Point3d[_totalNo];
		_point = new IPoint3d[_totalNo];
		for(int i=0; i < _totalNo; i++)  {
			_vertex[i] = new Point3d();
			_point[i] = new IPoint3d();
		}	
			
		setVertex();
		line = in.readLine();
		_dimZ = Float.valueOf(line).floatValue();	
	}
	
}