package ProtocolLayer.Example.CAD;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class ExtrusionCanvas extends Canvas implements MouseListener {
	Vector _point;
	
	// 100 x 100 resolution
	protected int _res = 50;
	protected int _unitPixel = 10;
	
	public ExtrusionCanvas()  {
		_point = new Vector();
		addMouseListener((MouseListener)this);
	}
	
  public void mouseReleased(MouseEvent e) {
		Point p = findNearestPoint(e.getPoint());
		
		// check the same point
		for(int i=1; i < _point.size(); i++)  {	
			Point tmp = (Point)_point.elementAt(i);
			if((p.x	== tmp.x) && (tmp.y == p.y))  {
				return;
			}
		}		
		_point.addElement(p);
				
		repaint();
	}
	
  public void mousePressed(MouseEvent e) {
  }

  public void mouseClicked(MouseEvent e) {
  }

  public void mouseEntered(MouseEvent e) {
  }

  public void mouseExited(MouseEvent e) {
  }
	
	protected Point findNearestPoint(Point p)  {
		int x = p.x / _unitPixel;
		int y = p.y / _unitPixel;
		
		if((p.x - x*_unitPixel) < (_unitPixel/2))  {
			x = x * _unitPixel;
		}	
		else  {
			x = (x + 1) * _unitPixel;
		}
		
		if((p.y - y*_unitPixel) < (_unitPixel/2))  {
			y = y * _unitPixel;
		}	
		else  {
			y = (y + 1) * _unitPixel;
		}
		return new Point(x,y);
	}
	
	public void paint(Graphics g)  {
		Color old = g.getColor();
		
		g.setColor(Color.green);
		
		int startx = 0;
		int starty = 0;
		
		int tsize = _res*_unitPixel;
		
		for(int i=0; i < _res; i++)  {
			if((i % 5) == 0)  {
				g.setColor(Color.gray);	
				g.drawLine(startx,starty,tsize,starty);
				g.setColor(Color.green);
			}	
			else  {
				g.drawLine(startx,starty,tsize,starty);
			}
				
			starty += _unitPixel;
		}
		
		startx = 0;
		starty = 0;
		for(int i=0; i < _res; i++)  {
			if((i % 5) == 0)  {
				g.setColor(Color.gray);	
				g.drawLine(startx,starty,startx,tsize);
				g.setColor(Color.green);
			}
			else  {
				g.drawLine(startx,starty,startx,tsize);
			}	
			startx += _unitPixel;
		}
		
		g.setColor(Color.red);
		g.drawLine(tsize/2,0,tsize/2, tsize);
		g.drawLine(0,tsize/2,tsize,tsize/2);
		g.drawString("0",tsize/2 - 15,tsize/2+15);
		g.drawString("X",tsize - 30, tsize/2+15);
		g.drawString("Y",tsize/2+15,30);
		
		int size = _point.size();
		for(int i=0; i < size; i++)  {
			Point s = (Point)_point.elementAt(i);
			g.drawRect(s.x-2,s.y-2,4,4);
		}	
		
		if(size > 1)  {
			g.setColor(Color.black);
			for(int i=0; i < size -1; i++)  {
				Point from = (Point)_point.elementAt(i);
				Point to = (Point)_point.elementAt(i+1);
				g.drawLine(from.x,from.y,to.x,to.y);
			}
		}
			
		g.setColor(old);	
	}
	
	public void clearPoints()  {
		_point.removeAllElements();
		repaint();
	}
			
	public void setPoints(Point2d[] edge,float scale)  {
		_point.removeAllElements();
		int tsize = _res*_unitPixel/2;
		
		for(int i=0; i < edge.length; i++)  {
			int x = (int)((edge[i]._x / scale) + tsize);
			int y = (int)((edge[i]._y / scale) + tsize);
			_point.addElement(findNearestPoint(new Point(x,y)));
		}		
		
		_point.addElement(_point.elementAt(0));	
					
		repaint();
	}
				
	public Point2d[] getPoints()  {
		int size = _point.size();
		
		if(size == 0)  {
			return new Point2d[0];
		}
			
		Point end = (Point)_point.elementAt(size-1);
		Point start = (Point)_point.elementAt(0);
		if((start.x == end.x) && (start.y == end.y))  {
			size--;
		}
			
		Point2d[] p = new Point2d[size];
		int tsize = _res*_unitPixel/2;
		
		for(int i=0; i < size; i++)  {
			p[i] = new Point2d();
			Point point = (Point)_point.elementAt(i);
			p[i]._x = point.x - tsize;
			p[i]._y = point.y - tsize;
		}
		
		return p;	
	}
}	