package ProtocolLayer.Example.CAD;

import java.awt.*;
import java.awt.event.*;

import ProtocolLayer.Example.IPRCApplet.MessageBox;

public class ExtrusionFrame extends Frame implements ActionListener {

	ViewCanvas _parent = null;
	ExtrusionCanvas _canvas;
	TextField _scale;
	TextField _depth;
	MatrixPanel _matPanel;
	CADObject _obj = null;
	CADObject _clone = null;
	
	public ExtrusionFrame(ViewCanvas parent)  {
		super("JATLite Extrusion Model");
		_parent = parent;
		init();
		setSize(500,600);
		setVisible(true);
	}	

	public ExtrusionFrame(ViewCanvas parent,Extrusion ext)  {
		this(parent);
		if(ext != null)  {
			_obj = ext;
			_clone = new Extrusion();
			ext.copyTo(_clone);
			_clone.reset();
			Rectangle rect = ext.getBoundingBox();
			int width = rect.width;
			int height = rect.height;
			
			int canvaswidth = _canvas.size().width * 2 /3;
			int canvasheight = _canvas.size().height * 2 /3;
			float scale = canvaswidth/width;
			float scaley = canvasheight/height;
			if(scaley >= scale)  {
				scale = scaley;
			}	
				
			_scale.setText(Float.toString(1/scale));
			Point3d p = ext.getOrigin();
			_matPanel.setOrigin(p);
			p = ext.getOrientation();
			_matPanel.setOrientation(p);
			float depth = ext.getDZ();
			_depth.setText(Float.toString(depth));
			
			_canvas.setPoints(ext.getEdge(),1/scale);
				
		}	
	}		
		

	public ExtrusionFrame()  {
		super("JATLite Extrusion Model");
		init();
		setSize(500,600);
		setVisible(true);
	}	

	protected void init()  {
		_matPanel = new MatrixPanel();
		add(_matPanel,"North");
		
		_canvas = new ExtrusionCanvas();
		add(_canvas,"Center");
		
		Panel bpanel = new Panel();
		Label dl = new Label("Depth");
		bpanel.add(dl);
		_depth = new TextField("",8);
		bpanel.add(_depth);
		
		Label l = new Label("Scale");
		bpanel.add(l);
		_scale = new TextField("1.0",8);
		bpanel.add(_scale);
		Button clear = new Button("Clear");
		clear.addActionListener(this);
		bpanel.add(clear);
		Button ok = new Button("OK");
		ok.addActionListener(this);
		bpanel.add(ok);
		Button cancel = new Button("Cancel");
		cancel.addActionListener(this);
		bpanel.add(cancel);
		
		add(bpanel,"South");
	}
	
	
	public void actionPerformed(ActionEvent evt)  {
		String cmd = evt.getActionCommand();
			
		if("Clear".equals(cmd))  {
			_canvas.clearPoints();
		}			
		else
		if("OK".equals(cmd))  {
			if(_parent == null)  {
				return;
			}
				
			String depthstr = _depth.getText();
			float depth = 0;
			try {	
				if(depthstr.equals(""))  {
					MessageBox b = new MessageBox("Depth should be specified",this);
					return;
				}				
				depth = Float.valueOf(depthstr).floatValue();
			} catch (Exception de)  {
				System.out.println(de.toString());
			}		
			
			if(depth <= 0)  {
				MessageBox b = new MessageBox("Depth should be greater than 0.0",this);
				return;
			}	
			
			String scalestr = _scale.getText();
			float scale = 0;
			try  {
				if(scalestr.equals(""))  {
					MessageBox b = new MessageBox("Scale should be specified",this);
					return;
				}				
			
				scale = Float.valueOf(scalestr).floatValue();
			} catch (Exception se)  {
				System.out.println(se.toString());
				return;
			}		
				
			if(scale <= 0)  {
				MessageBox b = new MessageBox("Scale should be greater than 0.0",this);
				return;
			}	
			
			Point2d[] p = _canvas.getPoints();
			for(int i=0; i < p.length; i++)  {
				p[i]._x *= scale;
				p[i]._y *= scale;
			}
			
			Point3d origin = _matPanel.getOrigin();
			Point3d orientation = _matPanel.getOrientation();
			
			Extrusion ext = new Extrusion(p,depth,origin,orientation);
			ext.setSelected(true);
			if(_obj != null)  {
				_parent.removeCADObject(_obj);
			}
				
			_parent.reset();
			_parent.addCADObject(ext);
			dispose();		
		}	
		else 
		if("Cancel".equals(cmd)) {
			dispose();
		}	
	}
	
	public static void main(String args[])  {
		ExtrusionFrame f = new ExtrusionFrame();
	}	
				
}