package ProtocolLayer.Example.CAD;

import java.awt.*;
import java.awt.event.*;

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class ObjectFrame extends Frame implements ActionListener {
	
	CADObject _obj = null;
	ViewCanvas _parent = null;
	
	TextField _originX,_originY,_originZ;
	TextField _orienX,_orienY,_orienZ;
	TextField _dX,_dY,_dZ;
	
	ColorPicker _cc;
	int _objType = -1;  // valid only for creation
	
	public ObjectFrame(ViewCanvas parent,int objType)  {
		super("Object Modeler");
		_parent = parent;
		_objType = objType;
		init();
	}
	
	public ObjectFrame(ViewCanvas parent,CADObject obj)  {
		super("Object Modeler");
		_parent = parent;
		_obj = obj;
		init();
		setObject();
	}

	protected void setObject()  {
		Point3d p = _obj.getOrigin();
		_originX.setText(Float.toString(p._x));
		_originY.setText(Float.toString(p._y));
		_originZ.setText(Float.toString(p._z));

		_dX.setText(Float.toString(_obj.getDX()));
		_dY.setText(Float.toString(_obj.getDY()));
		_dZ.setText(Float.toString(_obj.getDZ()));
		
		p = _obj.getOrientation();
		
		_orienX.setText(Float.toString(p._x));
		_orienY.setText(Float.toString(p._y));
		_orienZ.setText(Float.toString(p._z));
		
	}
	
	protected void init()  {
		Panel txtpanel = new Panel();
		txtpanel.setLayout(new GridLayout(4,4));
		txtpanel.add(new Label("           "));
		txtpanel.add(new Label("     X   "));
		txtpanel.add(new Label("     Y   "));
		txtpanel.add(new Label("     Z   "));
		txtpanel.add(new Label("Origin"));
		_originX = new TextField("0.0",8);
		txtpanel.add(_originX);
		_originY = new TextField("0.0",8);
		txtpanel.add(_originY);
		_originZ = new TextField("0.0",8);
		txtpanel.add(_originZ);
		txtpanel.add(new Label("Orientation"));
		_orienX = new TextField("0.0",8);
		txtpanel.add(_orienX);
		_orienY = new TextField("0.0",8);
		txtpanel.add(_orienY);
		_orienZ = new TextField("0.0",8);
		txtpanel.add(_orienZ);
		txtpanel.add(new Label("Dimension"));
		_dX = new TextField("0.0",8);
		txtpanel.add(_dX);
		_dY = new TextField("0.0",8);
		txtpanel.add(_dY);
		_dZ = new TextField("0.0",8);
		txtpanel.add(_dZ);
			
		add(txtpanel,"North");
		
		_cc = new ColorPicker();
		add(_cc,"Center");
		
		Panel bp = new Panel();
		Button clear = new Button("Clear");
		clear.addActionListener(this);
		Button cancel = new Button("Cancel");
		cancel.addActionListener(this);
		Button ok = new Button("OK");
		ok.addActionListener(this);
		
		bp.add(clear);
		bp.add(cancel);
		bp.add(ok);
		
		add(bp,"South");
		setSize(300,250);
		setVisible(true);
	}		
	
	public void actionPerformed(ActionEvent evt)  {
		String cmd = evt.getActionCommand();
			
		if("Clear".equals(cmd))  {
			_originX.setText("0.0");
			_originY.setText("0.0");
			_originZ.setText("0.0");

			_orienX.setText("0.0");
			_orienY.setText("0.0");
			_orienZ.setText("0.0");

			_dX.setText("0.0");
			_dY.setText("0.0");
			_dZ.setText("0.0");
		}			
		else
		if("OK".equals(cmd))  {
			float ox,oy,oz,rx,ry,rz,dx,dy,dz;
			
			try  {
				// copy tmp copy to original object to update
				String originX = _originX.getText(); ox = Float.valueOf(originX).floatValue();
				String originY = _originY.getText(); oy = Float.valueOf(originY).floatValue();
				String originZ = _originZ.getText(); oz = Float.valueOf(originZ).floatValue();
			
				String orienX = _orienX.getText(); rx = Float.valueOf(orienX).floatValue();
				String orienY = _orienY.getText(); ry = Float.valueOf(orienY).floatValue();
				String orienZ = _orienZ.getText(); rz = Float.valueOf(orienZ).floatValue();
			
				String dX = _dX.getText(); dx = Float.valueOf(dX).floatValue();
				String dY = _dY.getText(); dy = Float.valueOf(dY).floatValue();
				String dZ = _dZ.getText(); dz = Float.valueOf(dZ).floatValue();
			
				if(_objType == -1)  {
					_obj.setOrientation(rx,ry,rz);
					_obj.setOrigin(ox,oy,oz);
					_obj.setDX(dx);
					_obj.setDY(dy);
					_obj.setDZ(dz);
					_obj.setColor(_cc.getPickedColor());
				}
				else  {
					switch(_objType)  {
						case CADObject.BOX: _obj =new Box(dx,dy,dz,ox,oy,oz,rx,ry,rz); break;
						case CADObject.CYLINDER:_obj =new Cylinder(dx,dy,dz,ox,oy,oz,rx,ry,rz); break;
						case CADObject.CONE: _obj =new Cone(dx,dy,dz,ox,oy,oz,rx,ry,rz); break;
						case CADObject.SPHERE: _obj =new Sphere(dx,dy,dz,ox,oy,oz,rx,ry,rz); break;
					}
					
					if(_obj != null)  {
						_obj.setColor(_cc.getPickedColor());
						_parent.addCADObject(_obj);
					}	
				}	
					
				_parent.reset();
				
			}	catch (Exception e)  {
				System.out.println(e.toString() + " in CADModeler");
			}			
			dispose();		
		}	
		else 
		if("Cancel".equals(cmd)) {
			dispose();
		}	
	}
		
}