package ProtocolLayer.Example.CAD;

import java.awt.*;
import java.io.*;
import java.util.*;


/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
*
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/

public class Sphere extends CADObject  {

	// radius direction discretion number
	static final int _discNo = 12;
	// depth direction discretion number. Should be even number and greater than 4
	static final int _dDiscNo = 6;


	public Sphere()  {
		super();
		_type = CADObject.SPHERE;
	}

	// default origin(0,0,0) and orientation
	public Sphere(float xr, float yr, float depth)  {
		super(xr,yr,depth);
		_type = CADObject.SPHERE;
	}


	public Sphere(float xr, float yr, float depth, Point3d origin, Point3d rotation)  {
		super(xr,yr,depth,origin,rotation);
		_type = CADObject.SPHERE;
	}

	public Sphere(float xr, float yr, float depth, float ox,float oy,float oz, float thetax,float thetay, float thetaz)  {
		super(xr,yr,depth,ox,oy,oz,thetax,thetay,thetaz);
		_type = CADObject.SPHERE;
	}

	public Sphere(float[] matelm, float xr, float yr, float depth)  {
		super(matelm,xr,yr,depth);
		_type = CADObject.SPHERE;
	}

	public Sphere(Matrix3D mat,float width, float height, float depth)  {
		super(mat,width,height,depth);
		_type = CADObject.SPHERE;
	}

	public void setVertex()  {
		if(_vertex == null)  {
			_totalNo = _discNo * (_dDiscNo -1) + 2;

			// vertex are fixed and points are changed
			_vertex = new Point3d[_totalNo];
			_point = new IPoint3d[_totalNo];
			for(int i=0; i < _totalNo; i++)  {
				_vertex[i] = new Point3d();
				_point[i] = new IPoint3d();
			}
		}

		double unit = (2 * Math.PI / _discNo);

		_vertex[0]._x = 0; _vertex[0]._y = 0; _vertex[0]._z = _dimZ;
		_vertex[_totalNo-1]._x = 0; _vertex[_totalNo-1]._y = 0; _vertex[_totalNo-1]._z = -_dimZ;

		// for zero latitude
		for(int i=0; i < _discNo; i++)  {
			double theta = i * unit;
			double r = Math.pow((Math.cos(theta)/_dimX),2) + Math.pow((Math.sin(theta)/_dimY),2);
			r = 1/r;
			r = Math.pow(r,0.5);
			int curno = _discNo * (_dDiscNo / 2 -1) + i + 1;
			_vertex[curno]._x = (float)(r * Math.cos(theta));
			_vertex[curno]._y = (float)(r * Math.sin(theta));
			_vertex[curno]._z = 0.0f ;
		}

		double unitIta = Math.PI / 2 / (_dDiscNo / 2);  // ita discretion
		double currentIta = Math.PI / 2 - unitIta;
		for(int j=1; j < (_dDiscNo/2); j++)  {
			// get Xr for current ita
			double Xr = Math.pow((Math.cos(currentIta)/_dimX),2) + Math.pow((Math.sin(currentIta)/_dimZ),2);
			Xr = 1/Xr;
			Xr = Math.pow(Xr,0.5);
			Xr = Xr*Math.cos(currentIta);

			// get Yr for current ita
			double Yr = Math.pow((Math.cos(currentIta)/_dimY),2) + Math.pow((Math.sin(currentIta)/_dimZ),2);
			Yr = 1/Yr;
			Yr = Math.pow(Yr,0.5);

			// for depth
			double mr = Yr;

			Yr = Yr*Math.cos(currentIta);

			for(int i=0; i < _discNo; i++)  {
				double theta = i * unit;
				double r = Math.pow((Math.cos(theta)/Xr),2) + Math.pow((Math.sin(theta)/Yr),2);
				r = 1/r;
				r = Math.pow(r,0.5);
				int curno = _discNo * (j - 1) + i + 1;
				_vertex[curno]._x = (float)(r * Math.cos(theta));
				_vertex[curno]._y = (float)(r * Math.sin(theta));
				_vertex[curno]._z = (float)(mr*Math.sin(currentIta)) ;
				int inverseno = _discNo * (_dDiscNo - j - 1) + i + 1;
				_vertex[inverseno]._x = _vertex[curno]._x;
				_vertex[inverseno]._y = _vertex[curno]._y;
				_vertex[inverseno]._z = - _vertex[curno]._z;
			}
			currentIta -= unitIta;

		}

	}

	public void draw(Graphics g)  {
		if((_dimX != 0) && (_dimY != 0) && (_dimZ != 0))  {
			Color oldcolor = g.getColor();
			g.setColor(_color);

			_mat.transform(_vertex,_point,_totalNo);
			// draw verticals
			for(int i=0; i < (_discNo); i++)  {
				g.drawLine(_point[0]._x,_point[0]._y,_point[i+1]._x,_point[i+1]._y);

				int j = 1;
				for(; j < (_dDiscNo - 1); j++)  {
					g.drawLine(_point[_discNo * (j-1) + i+1]._x,_point[_discNo * (j-1) +i+1]._y,_point[_discNo * (j) +i + 1]._x,_point[_discNo * (j) +i + 1]._y);
				}
				g.drawLine(_point[_discNo * (j-1) + i+1]._x,_point[_discNo * (j-1) + i+1]._y,_point[_totalNo-1]._x,_point[_totalNo-1]._y);
			}

			// draw center lines
			int curno = 0;
			for(int i=0; i < (_discNo-1); i++)  {
				curno = _discNo * (_dDiscNo / 2) + 1 +i;
				g.drawLine(_point[curno]._x,_point[curno]._y,_point[curno + 1]._x,_point[curno + 1]._y);
			}

			// draw intermediate lines
			for(int j=0; j < (_dDiscNo / 2); j++)   {
				for(int i=0; i < (_discNo -1); i++)  {
					curno = j * _discNo + i + 1;
					// upper part
					g.drawLine(_point[curno]._x,_point[curno]._y,_point[curno + 1]._x,_point[curno + 1]._y);
					// lower part
					curno = _discNo * (_dDiscNo -j -2) + i + 1;
					g.drawLine(_point[curno]._x,_point[curno]._y,_point[curno + 1]._x,_point[curno + 1]._y);
				}

				// closing
				curno = j * _discNo + 1;
				g.drawLine(_point[curno]._x,_point[curno]._y,_point[curno + _discNo -1]._x,_point[curno + _discNo -1]._y);
				curno = _discNo * (_dDiscNo -j -2) + 1;
				g.drawLine(_point[curno]._x,_point[curno]._y,_point[curno + _discNo -1]._x,_point[curno + _discNo -1]._y);
			}

			if(_isSelected)  {
				g.setColor(Color.red);
				for(int i=0; i < _totalNo; i++)  {
					g.drawRect(_point[i]._x -2, _point[i]._y -2, 4,4);
				}
			}

			g.setColor(oldcolor);

		}
	}

	public String toString()  {
		String myself = "Sphere\n";
		myself = myself + super.toString();
		return myself;
	}

	// print out current orientation, origin,width, height, depth, deliminated with ','
	public void writeToFile(DataOutputStream out) throws IOException {
		out.writeChars("Sphere\n");
		//super.writeToFile(out);
 	}


}
