package ProtocolLayer.Example.CAD;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import Abstract.*;

import features.*;
import webcad_01_0_1.*;

/*
 * Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
 * All rights reserved.<p>
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You can obtain the GNU General Public License at
 * http://www.gnu.org/copyleft/gpl.html or by writing to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * This software is bound by the terms and conditions listed in the
 * attached <a href="../LICENSE">LICENSE file</A>.
 *  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
 *  @version Java(tm) 1.1
 */

public class ViewCanvas
    extends Canvas
    implements MouseListener, MouseMotionListener {
  Vector _CADObjList = new Vector();

  boolean painted = true;

  int prevx, prevy;

  boolean _isSelectMode = false;
  CADObject _currentSelection = null;
  FramePrincipal framePrincipal;

  public ViewCanvas(/*ArmazenadorDeFeatures armazenador*/FramePrincipal framePrincipal1) {

//    this.armazenador = armazenador;
this.framePrincipal = framePrincipal1;
    addMouseListener( (MouseListener)this);
    addMouseMotionListener( (MouseMotionListener)this);

    init();
    setBackground(Color.gray.brighter());
  }

  public void init() {
    setSize(650, 500);
  }

  public void setSelectMode(boolean isSelectMode) {
    _isSelectMode = isSelectMode;
  }

  public boolean getSelectMode() {
    return _isSelectMode;
  }

  public void mousePressed(MouseEvent e) {
    if (!_isSelectMode) {
      prevx = e.getX();
      prevy = e.getY();
    }
  }

  public void mouseReleased(MouseEvent e) {
  }

  public void mouseClicked(MouseEvent e) {
    if (!_isSelectMode) {
      return;
    }

    Point p = e.getPoint();

    // click on the same selected object to see the object info.
    if ( (_currentSelection != null) && (_currentSelection.contains(p))) {
      showObjectInfo();
      return;
    }

    _currentSelection = null;

    // clear selection
    Enumeration ee = _CADObjList.elements();
    while (ee.hasMoreElements()) {
      CADObject obj = (CADObject) ee.nextElement();
      obj.setSelected(false);
      if (obj.contains(p)) {
        _currentSelection = obj;
      }
    }

    if (_currentSelection != null) {
      _currentSelection.setSelected(true);
      /*
          // IE does not support double click.
          if(e.getClickCount() == 2)  {
           // double click. Show the object information
           showObjectInfo();
          }
       */
    }

    repaint();
  }

  public void mouseEntered(MouseEvent e) {
  }

  public void mouseExited(MouseEvent e) {
  }

  public void mouseMoved(MouseEvent e) {
  }

  public void mouseDragged(MouseEvent e) {
    if (!_isSelectMode) {
      int x = e.getX();
      int y = e.getY();

      float xtheta = (prevy - y) * 360.0f / size().width;
      float ytheta = (x - prevx) * 360.0f / size().height;
      Enumeration ee = _CADObjList.elements();
      while (ee.hasMoreElements()) {
        CADObject obj = (CADObject) ee.nextElement();
        obj.rotate(xtheta, ytheta, 0.0f);
      }

      if (painted) {
        painted = false;
        repaint();
      }
      prevx = x;
      prevy = y;
    }
  }

  public void addCADObject(CADObject o) {
    // for drawing
    _CADObjList.addElement(o);
    repaint();
  }

  public void removeCADObject() {
    if (_currentSelection != null) {
      int index = _CADObjList.indexOf(_currentSelection);
      _CADObjList.removeElement(_currentSelection);
      _currentSelection = null;
    }
    repaint();
  }

  public void removeCADObject(CADObject obj) {
    _CADObjList.removeElement(obj);
    repaint();
  }

  public void removeAllObjects() {
    _CADObjList.removeAllElements();
  }

  public void showObjectInfo() {
    if (_currentSelection != null) {
      if (_currentSelection.getType() == CADObject.EXTRUSION) {
        ExtrusionFrame f = new ExtrusionFrame(this,
                                              (Extrusion) _currentSelection);
      }
      else
      if (_currentSelection.getType() == CADObject.AXISSYM) {
        AxisSymFrame f = new AxisSymFrame(this, (AxisSym) _currentSelection);
      }
      else {
        ObjectFrame of = new ObjectFrame(this, _currentSelection);
      }
    }
  }

  // for modeling support
  public Vector getInitialModel() {
    Vector v = new Vector();
    Enumeration e = _CADObjList.elements();
    while (e.hasMoreElements()) {
      CADObject obj = (CADObject) e.nextElement();
      v.addElement(obj.clone());
    }

    return v;
  }

  public void paint(Graphics g) {
    Enumeration e = _CADObjList.elements();
    int x = size().width / 2;
    int y = size().height / 2;
    while (e.hasMoreElements()) {
      CADObject obj = (CADObject) e.nextElement();
      obj.translate(x, y, 0.0f);
      obj.draw(g);
      obj.translate( -x, -y, 0.0f);
    }

    setPainted();

  }

  public void translate(int x, int y) {
    Enumeration e = _CADObjList.elements();
    while (e.hasMoreElements()) {
      CADObject obj = (CADObject) e.nextElement();
      obj.translate(x, y, 0.0f);
    }

    repaint();
  }

  public void scale(float scale) {
    Enumeration e = _CADObjList.elements();
    while (e.hasMoreElements()) {
      CADObject obj = (CADObject) e.nextElement();
      obj.scale(scale);
    }

    repaint();
  }

  public void zoom(float scale) {
    Enumeration e = _CADObjList.elements();
    while (e.hasMoreElements()) {
      CADObject obj = (CADObject) e.nextElement();
      obj.scale(scale);
//				Matrix3D mat = obj.getMatrix();
//				obj.translate(mat.xo*(scale - 1),mat.yo*(scale-1),/*mat.zo*(scale-1)*/0);
    }

    repaint();
  }

  private synchronized void setPainted() {
    painted = true;
    notifyAll();
  }

  public synchronized void reset() {
    Enumeration e = _CADObjList.elements();
    while (e.hasMoreElements()) {
      CADObject obj = (CADObject) e.nextElement();
      obj.reset();
    }

    repaint();
  }

  public void writeObjects(String fileName, /*DataOutputStream out*/ ObjectOutputStream out) throws
      IOException {
   //System.out.println(_CADObjList.size());
    System.out.println("Size do armazenador: "+ this.framePrincipal.dadosDoProjeto.armazenadorDeFeatures.getThis().size());
    System.out.println("ARMAZENADOR DE FEATURES - ViewCanvas - Linha 274");
    //Enumeration e = _CADObjList.elements();
//    Enumeration e = armazenador.getThis().elements();
    /*while (e.hasMoreElements()) {
      CADObject obj = (CADObject) e.nextElement();
      obj.writeToFile(out);
      out.flush();
    }*/
  out.writeObject(this.framePrincipal.dadosDoProjeto);
  out.flush();out.close();
  }

  // No difference between append and put
  public void appendObjects(String fileName, /*DataOutputStream out*/ ObjectOutputStream out) throws
      IOException {
    writeObjects(fileName, out);
  }

  public void readObjects(String fileName, /*DataInputStream in*/ObjectInputStream in) throws
      IOException {
    System.out.println("READ OBJECTS - ViewCanvas");
    System.out.println("IN.GETCLASS: "+in.getClass());
    //ObjectInputStream in1 = new ObjectInputStream(in);
    try {
      this.framePrincipal.dadosDoProjeto = (DadosDoProjeto) in.readObject();
//      this.framePrincipal.repaint();

      in.close();

    }
    catch (ClassNotFoundException ex) {
    }
    catch (IOException ex) {
    }

//    removeAllObjects();
//    System.out.println("IN.GETCLASS :::"+in.getClass());
//    char c;

/*    try {
      c = in.readChar();
    }
    catch (EOFException e) {
      return;
    }

    String objName = "";
*/
    /*while (true) {
      while (c != '\n') {
        objName = objName + c;
//System.out.println(objName);
        try {
          c = in.readChar();
        }
        catch (EOFException ee) {
          throw new IOException(ee.toString());
        }
      }

      if (objName.equalsIgnoreCase("Box")) {
        Box b = new Box();
        b.readFromFile(in);
        addCADObject(b);
      }
      else
      if (objName.equalsIgnoreCase("Cylinder")) {
        Cylinder cyl = new Cylinder();
        cyl.readFromFile(in);
        addCADObject(cyl);
      }
      else
      if (objName.equalsIgnoreCase("Cone")) {
        Cone con = new Cone();
        con.readFromFile(in);
        addCADObject(con);
      }
      else
      if (objName.equalsIgnoreCase("Sphere")) {
        Sphere sph = new Sphere();
        sph.readFromFile(in);
        addCADObject(sph);
      }
      else
      if (objName.equalsIgnoreCase("Extrusion")) {
        Extrusion ext = new Extrusion();
        ext.readFromFile(in);
        addCADObject(ext);
      }
      else
      if (objName.equalsIgnoreCase("AxisSym")) {
        AxisSym a = new AxisSym();
        a.readFromFile(in);
        addCADObject(a);
      }
      else {
        return;
      }

      try {
        c = in.readChar();
      }
      catch (EOFException eee) {
        return;
      }
      objName = "";
    }*/
  }
}
