package ProtocolLayer;

import java.net.*;
import java.io.*;

import BaseLayer.*;
import Abstract.ConnectionException;

import webcad_01_0_1.FramePrincipal;

/**
 * Data transfer thread for FTP. After all necessary procedures are setup(security,
 * change directory, connect to port socket), this thread will be created to
 * perform read/write from file/socket.
 */
/*
 * Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
 * All rights reserved.<p>
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You can obtain the GNU General Public License at
 * http://www.gnu.org/copyleft/gpl.html or by writing to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * This software is bound by the terms and conditions listed in the
 * attached <a href="../LICENSE">LICENSE file</A>.
 *  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
 *  @version Java(tm) 1.1
 */
public class FTPPortThread
    extends BRecvThread {

  /**
   * Port server socket, if the agent is stand alone application. For applet agents
   * this data member is meaningless
   */
  ServerSocket _portServerSocket = null;

  /**
   * Parent FTPThread
   */
  FTPThread _parent;

  /**
   * FTPCon mode
   */
  int _mode;

  /**
   * For applet file name
   */
  String _appletFileName = null;

  /**
   * For general purpose. Do not construct FTPPortThread directly using]
   * this constructor.
   * @param parent FTPThread object
   * @param mode FTP mode
   * @exception Abstract.ConnectionException
   */
  FTPPortThread(FTPThread parent, int mode) throws ConnectionException {
    if (parent == null) {
      throw new ConnectionException("FTPThread not specified for FTPPortThread");
    }
    _parent = parent;
    _mode = mode;
  }

  /**
   * For stand alone GET operation
   * @param parent FTPThread object
   * @param ss Server socket to accept socket connection from FTP server
   * @param out File output stream
   * @param mode FTP mode
   * @exception Abstract.ConnectionException
   */
  FTPPortThread(FTPThread parent,
                ServerSocket ss,
                BufferedWriter out,
                int mode) throws ConnectionException {
    this(parent, mode);
    _portServerSocket = ss;
    _out = out;
  }

  /**
   * For stand alone GET operation
   * @param parent FTPThread object
   * @param ss Server socket to accept socket connection from FTP server
   * @param in File input stream
   * @param mode FTP mode
   * @exception Abstract.ConnectionException
   */
  FTPPortThread(FTPThread parent,
                ServerSocket ss,
                BufferedReader in,
                int mode) throws ConnectionException {
    this(parent, mode);
    _portServerSocket = ss;
    _in = in;
  }

  /**
   * For applet data transfer operation
   * @param parent FTPThread object
   * @param socket Socket, connected to a router proxy server socket
   * @param mode FTP mode
   * @param filename File name, specified for data transfer
   */
  FTPPortThread(FTPThread parent,
                Socket socket,
                int mode,
                String filename) throws ConnectionException {
    this(parent, mode);

    // for applet, agentAction is essential for call back routine
    if (parent.getAgentAction() == null) {
      throw new ConnectionException(
          "IPRouterAction not specified for FTPThread");
    }

    _client = socket;
    _appletFileName = filename;
    System.out.println("STRING _appletFileName: "+_appletFileName);
  }

  /**
   * Read/write operation. After finished, it will resume parent FTPThread if it
   * was suspended.
   */
  public void run() {
    try {
      System.out.println("Running FTPPortThread");
      // for stand alone case,
      if ( (_mode & FTPCon.ISAPPLET) == 0) {
        System.out.println("Stand alone");
        char[] buffer = new char[1024];
        int readlen;
        int readSize = 0;

        _client = _portServerSocket.accept();
        _portServerSocket.close();
        _portServerSocket = null;

        // get mode. Input from socket, output to file
        if ( (_mode & FTPCon.ISGETMODE) != 0) {
          ObjectInputStream obj4 = new ObjectInputStream(_client.getInputStream());
          _in = new BufferedReader(new InputStreamReader(/*_client.getInputStream()*/obj4),
                                   1024);
          while ( (readlen = _in.read(buffer, 0, 1024)) > 0) {
            _out.write(buffer, 0, readlen);
            readSize += readlen;
          }
        }
        else {
          ObjectOutputStream obj5 = new ObjectOutputStream(_client.getOutputStream());
          _out = new BufferedWriter(new OutputStreamWriter(/*_client.
              getOutputStream()*/obj5), 1024);
          if ( (_mode & FTPCon.ASCIITYPE) != 0) {
            // because of unicode, it is less efficient
            while ( (readlen = _in.read(buffer, 0, 1024)) > 0) {
              for (int i = 0; i < readlen; i++) {
                if (buffer[i] <= 0x7f) {
                  _out.write(buffer[i]);
                }
              }
              _out.flush();
            }
          }
          else {
            while ( (readlen = _in.read(buffer, 0, 1024)) > 0) {
              _out.write(buffer, 0, readlen);
              _out.flush();
            }
          }
        }
      }
      // For applet data transfer
      else {
        System.out.println("Applet FTPPortThread");

        IPRouterClientAction agentAction = _parent.getAgentAction();
//Passou
        if ( (_mode & FTPCon.ISGETMODE) != 0) {
          System.out.println("CLASSE FTPPorThread");
          System.out.println("Getmode start");
          ObjectInputStream obj2 = new ObjectInputStream(_client.getInputStream());
          System.out.println("TOSTRING - ObjectInputStream :::"+ obj2.readObject().toString());
          //System.out.println("Classe carregada:"+ obj2.readObject().getClass());
          agentAction.FTPRetrieve(_appletFileName, /*_client.getInputStream()*/obj2);
          System.out.println("Getmode end");
        }
        else {
          if ( (_mode & FTPCon.APPEND) != 0) {
            System.out.println("Appendmode start");
            ObjectOutputStream obj3 = new ObjectOutputStream(_client.getOutputStream());
            agentAction.FTPAppend(_appletFileName, /*_client.getOutputStream()*/obj3);
            System.out.println("Appendmode end");
          }
          else {
            System.out.println("Storemode start");
            ObjectOutputStream obj1 = new ObjectOutputStream(_client.getOutputStream());

            agentAction.FTPStore(_appletFileName, /*_client.getOutputStream()*/obj1);
            System.out.println("Storemode end");
          }
        }
        System.out.println("End transmission");

      }
    }
    catch (Exception e) {
      System.out.println(e.toString());

      _parent.notifyError("FTPPort throws error : " + e.toString()+" esse  o erro!!");//Esse erro est ocorrendo.
    }
    finally {
      if (_parent != null) {
        _parent.resume();
      }

      endConn();
    }
  }

  /**
   * Close connection and stop thread
   */
  public void endConn() {
    try {
      if (_portServerSocket != null) {
        _portServerSocket.close();
        _portServerSocket = null;
      }
    }
    catch (Exception e) {}

    super.endConn();
  }

}
