package ProtocolLayer;

import Abstract.*;
import BaseLayer.*;
import KQMLLayer.*;
import RouterLayer.Router.*;
import RouterLayer.AgentClient.*;
import RouterLayer.util.*;


import java.net.*;
import java.io.*;
import java.util.*;

/**
* Template to support protocol layer RouterClientAction. <BR>
* As the same as RouterClientAction, you need to implement Act(Object) method.
* IPRouterClientAction implements processMessage(String,Object) for managing SMTP/FTP.
* Therefore, if you want to extend the method, you should call
* processMessage(String,Object) of IPRouterClientAction like :<P>
* public void processMessage(String cmd,Object obj)  {<BR>
* super.processMessage(cmd,obj);<BR>
* ///// Add your implementation here<BR>
* .....<BR>
* }<P>
* Furthermore, you need to implement three FTP related callback functions :<P>
* FTPStore(String finename,OutputStream), FTPAppend(String filename,OutputStream),
* FTPRetrieve(String filename,InputStream), where filename is the file name you have specified
* when you call createDataTransferConnection(FTPCon ftp) method and InputStream/OutputStream is
* iostream from socket. Those three functions are callback functions - it will be invoked by
* processMessage(String,Object) method from received 'port-info' message, replied from router.<P>
* For implementation of FTPXXX callbacks, just assume that you are given file name and iostream.
* Do whatever you want to the iostream - serialize objects, write informations, retrieve information,etc.
* You need to wrap the InputStream/OutputStream with appropriate java IO objects such as PrintWriter,
* DataInputStream,etc.<P>
* <B>For stand-alone application</B> :<BR>
* <UL>
* <LI>Since stand alone applications is free to create ServerSocket and to connect
* any other machine, SMTP/FTP will be performed directly like numerous
* SMTP/FTP applications.<P>
* <LI>SMTP : <B>To use SMTP, you should specify your agent email address within agent
* Address(_myAddress) object in IPRouterClientAction, which is normally passed to
* constructor.</B><BR>
* Call sendEmailMessage(Address sender,Address receiver,String msg) or
* sendEmailMessage(String kqmlstr) for sending KQML message as an email. If your agent
* email address is defined in the 'Address sender' parameter and your agent
* knows receiver email address which is included in formal parameter receiver 'Address',
* you can use the first one. If your agent does not know receiver agent's email address
* use the second one. The second method will ask receiver address to router and router
* will reply. If the receiver email is known to router, email will be sent to the
* receiver but if it is not defined in registry of router, ConnectionException will be
* thrown.<P>
* <LI>FTP : The simplest way of FTP is using FTPCon object. Define FTPCon object and
* pass it to FTP(FTPCon) method. For error conditions, FTPCon(FTPCon) method will
* throw ConnectionException.<P>
* </UL>
* <B>For applets</B> :<P>
* <UL>
* <LI>Since applet has some restrictions for socket connection, you can not directly
* connect to SMTP/FTP server if those servers are different from HTTP server where
* router is running. For applets, router will be take a role as a proxy for processing
* SMTP/FTP but no data will be saved/cached to router side. Router will create a
* ConnectorThread for each transaction and the ConnectorThread redirects input/output
* from the applet side to SMTP/FTP server side and vice versa.<P>
* <LI>SMTP : <B>To use SMTP, you should specify your agent email address within agent
* Address(_myAddress) object in IPRouterClientAction, which is normally passed to
* constructor.</B><BR>
* Call createEmailConnection(String kqmlmsg) or createEmailConnection(KQMLmessage kqml)
* for sending KQML message as an email. Router will create a ServerSocket where the
* applet will connect to and reply 'port-info' message to the applet. For error
* conditions,such receiver email is not specified or SMTP server connection fails,
* etc, router will send an error message instead of 'port-info' reply.
* The 'port-info' message includes ServerSocket number for the applet to connect.
* After the applet connect to the ServerSocket, IPRouterClientAction will automatically
* start to transact SMTP(this will be performed by sendEmailToRouter(KQMLmessage)
* so that you <B>do not need to call sendEmailToRouter(KQMLmessage) directly</B>.
* For error conditions, sendEmailToRouter method
* will throw ConnectionException.<P>
* <LI>FTP : Note that an applet can not access to local file system where it is running
* but it can transfer data to remote file system using FTP function JATLite is providing.
* Call createDataTransferConnection(FTPCon con) or
* createDataTransferConnection(String conid, String remotePath, String host,
* int port, String userName,String password, String localPath, String[] files, int mode). As the same as SMTP,
* IPRouterClientAction will request ServerSocket to router and router will reply
* a 'port-info' message. Then IPRouterClientAction will start to transfer data after
* connect to the ServerSocket. 'port-info' message will be processed automatically by
* processRouterMessage(KQMLmessage).<P>
* </UL>
* @see RouterLayer.AgentClient.RouterClientAction
* @see ProtocolLayer.FTPCon
*/

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
*
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/

public abstract class IPRouterClientAction extends RouterClientAction {

	/**
	  * Static numbering for reply message
	  */
	protected int _replyNo;

	/**
	* Reply queue is a depository to save information for asynchronous internet protocol
	* transaction. When an applet agent want to use router as a proxy, the agent will
	* send appropriate message to the router with reply-with slot and save data to
	* the hashtable. Therefore, the key field is string like 'smtp0' or 'ftp3' and
	* the value field can be anything.
	*/
  protected Hashtable _replyQueue;

  /**
   * Default constructor
   */
  public IPRouterClientAction()  {
    super();
    initReplyQueue();
  }


  /**
   * Constructor. Note that your address is not specified yet. You should use setMyAddress()
   * method before invoke connect() method.
   * @param routeraddress The Router address to connect
   * @param router registrar address
   * @param durationTime Maximum idle time for the connection
   */
  public IPRouterClientAction(Address routeraddress,
			    Address registraraddress,
			    int durationTime)  {
		super(routeraddress,registraraddress,durationTime);
    initReplyQueue();
  }

  /**
   * Router AgentAction constructor with given my address,rotuer address and router registrar
   * address. You can use this constructor if your agent is already registered to the Router
   * @param myAddress My address
   * @param routerAddress The Router address
   * @param routerAddress The Router address
   * @param durationTime Maximum idle time
   */
  public IPRouterClientAction(Address myAddress,
			    Address routerAddress,
			    Address registrarAddress,
			    int durationTime)  {
    super(myAddress,routerAddress,registrarAddress,durationTime);
    initReplyQueue();
  }

  /**
   * Constructor using Address strings
   * @param routeraddressstr The Router address string to connect
   * @param registraraddressstr The Router registrar address string
   * @param durationTime Maximum idle time for the connection
   */
  public IPRouterClientAction(String routeraddressstr,
			    String registraraddressstr,
			    int durationTime)  {
		super(routeraddressstr,registraraddressstr,durationTime);
    initReplyQueue();

  }

  /**
   * Router AgentAction constructor with given my address,rotuer address and router registrar
   * address. You can use this constructor if your agent is already registered to the Router
   * @param myAddressstr My address string
   * @param routerAddressstr The Router address string
   * @param durationTime Maximum idle time
   */
  public IPRouterClientAction(String myAddressstr,
			    String routerAddressstr,
			    String registrarAddressstr,
			    int durationTime)  {
    super(myAddressstr,routerAddressstr,registrarAddressstr,durationTime);
    initReplyQueue();

  }

	/**
	  * Initialize reply queue
	  */
	protected void initReplyQueue()  {
		_replyNo = 0;
		_replyQueue = new Hashtable();
	}

  /**
  * Send KQML message through email. Address sender and receiver should contains email
  * address within its description field. For example, sender address may be :<BR>
  * AgentName,cdr.stanford.edu,2345,"MessageRouter","(agent-info :password xxx :email jhc@cdr.stanford.edu)"
  * @param sender Sender(usually agent itself) address.
  * @param receiver Receiver address.
  * @param msg KQML message to be sent
  * @exception Abstract.ConnectionException Note that KQML parse exception will not be thrown
  */
	public void sendEmailMessage(Address sender,Address receiver,String msg)
		throws ConnectionException  {
		SMTPThread t = null;
		try  {
			String email = getEmailAddress(receiver);
			if(email== null)  {
				throw new ConnectionException("SMTP error : Receiver <" + receiver.getID() + "> has no email address");
			}

			t = new SMTPThread(sender,receiver,msg,getThreadGroup());

			// want to get reply from SMTP Thread
			t.setAgentAction(this);
			t.connect();
			t.start();
		} catch (Exception e)  {
			throw new ConnectionException(e.toString());
		}
	}

	/**
	* Send KQML message using email protocol. The receiver address should be cached.
	* My address and receiver address should contain email address in description field
	* @param kqmlstr KQML message to be sent.
  * @exception Abstract.ConnectionException Note that KQML parse exception will not be thrown
	*/
	public void sendEmailMessage(String kqmlstr)  throws ConnectionException {
		try  {
			KQMLmessage kqml = new KQMLmessage(kqmlstr);
			String receiver = kqml.getValue("receiver");
			if(receiver != "")  {
				Address recaddr = _addresses.getAddress(receiver);
				if(recaddr != null)  {
					sendEmailMessage(_myAddress,recaddr,kqmlstr);
				}
				else  {
					String replyWith = "address-" + Integer.toString(_replyNo);
					requestAddress(receiver,replyWith);
					_replyNo++;
					_replyQueue.put(replyWith,kqmlstr);
					System.out.println("Receiver address is not cached <" + receiver + ">. Request Address to Router");
				}
			}
			else  {
				throw new ConnectionException("Receiver missing <" + kqmlstr + ">");
			}
		} catch (Exception e)  {
			throw new ConnectionException(e.toString());
		}
	}

	/**
	* Asynchronous email sending routine. Send email server information to router. The router
	* will provide a connector socket number.
	* @param replyWith Reply-with slot of KQML message. After creating a connector, router will
	* reply connector socket number with in-reply-to.
  * @exception Abstract.ConnectionException Note that KQML parse exception will not be thrown
	*/
	public void createEmailConnection(String kqmlmsg)  throws ConnectionException  {
		try  {
			this.createEmailConnection(new KQMLmessage(kqmlmsg));
		}  catch (Exception e)  {
			throw new ConnectionException(e.toString());
		}
	}

	/**
	* Asynchronous email sending routine. Send email server information to router. The router
	* will provide a connector socket number.
	* @param replyWith Reply-with slot of KQML message. After creating a connector, router will
	* reply connector socket number with in-reply-to.
  * @exception Abstract.ConnectionException Note that KQML parse exception will not be thrown
	*/
	public void createEmailConnection(KQMLmessage kqml) throws ConnectionException {

		if(_myAddress == null)  {
				throw new ConnectionException("My address not specified");
		}

		String receiver = kqml.getValue("receiver");
		if(receiver == null)  {
			throw new ConnectionException("Receiver not specified to send email message");
		}

		Address addr = _addresses.getAddress(receiver);
		String remail = getEmailAddress(addr);

		String email = getEmailAddress(_myAddress);
		if(email == null)  {
			throw new ConnectionException("My email address is not specified");
		}

		int atloc = email.indexOf("@");
		if(atloc == -1)  {
			throw new ConnectionException("Wrong my email address");
		}

		String server = email.substring(atloc + 1);

		String replyWith = "smtp" + _replyNo;
		_replyNo++;


		String sendmsg = "(data-transfer :sender ";
		sendmsg = sendmsg + _myAddress.getID() + " :receiver " + _routerName;
		sendmsg = sendmsg + " :reply-with " + replyWith + " :protocol smtp :content (";
		sendmsg = sendmsg + "send-mail :server " + server + " :receiver " + receiver + "))";

		// server address and receiver email will be given from the router
		// use setServerAddress and setReceiverAddress before construct SMTPThread
		String[] recs = null;
		if((remail != null) && (!remail.equals("")))  {
			recs = new String[1];
			recs[0] = remail;
		}

		SMTPCon con = new SMTPCon(null,email,recs,kqml.getSendString());

		_replyQueue.put(replyWith,con);

		try  {
			sendMessage(sendmsg);
		} catch (ParseException p)  {
			throw new ConnectionException("Parse Error : " + p.toString());
		}
	}

	/**
	  * Email sending to router -> receiver. If an agent want to send a KQML message
	  * through email, the applet will create email connection using createEmailConnection
	  * method. Router will create a email port and will reply a message with the email
	  * port. The applet will connect to the port and start to send the KQML message
	  * to router and router will deliver the email to the receiver. This method is for
	  * actual data transfer(sending KQML message throuth email) after receiving
	  * the port information from router.
	  * @param kqml KQML message received from router which contains a proxy socket number
	  * @exception Abstract.ConnectionException
	  */
	protected void sendEmailToRouter(KQMLmessage kqml) throws ConnectionException  {

		String replyWith = kqml.getValue("in-reply-to");
		if(replyWith.equals(""))  {
			throw new ConnectionException("Reply with slot missing");
		}

		String content = kqml.getValue("content");
		String receiveremail,port;
		try  {
			KQMLmessage ckqml = new KQMLmessage(content);
System.out.println(kqml.getReadString());
			port = ckqml.getValue("port");
			receiveremail = ckqml.getValue("receiver-email");
		}  catch (Exception e)  {
			throw new ConnectionException("Email error " + e.toString());
		}

		SMTPCon con = (SMTPCon)_replyQueue.get(replyWith);
		if(con == null)  {
			System.out.println("Uncashed reply-with for " + kqml);
		}
		else  {
			con.setServerAddress(new Address("SMTP",
																		   (_addresses.getAddress(_routerName)).getHost(),
																		   (Integer.valueOf(port)).intValue(),
																		   "SMTP",
																		   ""));
			String[] receaddr = con.getReceiverAddresses();

			if(receaddr == null) {
				String[] recs = new String[1];
				recs[0] = receiveremail;
				con.setReceiverAddresses(recs);
			}

			_replyQueue.remove(replyWith);

			try  {
				SMTPThread t = new SMTPThread(con,Thread.NORM_PRIORITY,getThreadGroup());

				// want to get reply from SMTPThread
				t.setAgentAction(this);
				t.connect();
				t.start();
			}  catch (Exception e)  {
				throw new ConnectionException("SMTP Connection fail::" + e.toString());
			}
		}
	}

	/**
	* Cache the receiver address from received KQML message. The message will be sent from
	* router and its performative is 'agent-address'. The address will be added to _addressTable.
	* @param addressKQML KQML message, whose performative is 'agent-address'.
	*/
	public Address cacheAddress(KQMLmessage addressKQML)  {
		if(addressKQML == null)  {
			System.out.println("Null string is passed to cacheAddress method");
		}

		String perf = addressKQML.getValue("performative");
		if(!perf.equals("agent-address"))  {
			System.out.println("Wrong message passed to cacheAddress method");
			return null;
		}

		String name = addressKQML.getValue("name");
		String host = addressKQML.getValue("host");
		String port = addressKQML.getValue("port");
		String mm = addressKQML.getValue("message-method");
		String desc = addressKQML.getValue("description");

		// if the same name agent address, it will be replaced.
		Address addr = new Address(name,host,(Integer.valueOf(port)).intValue(),mm,desc);
		_addresses.addAddress(addr);
		return addr;
	}

	/**
	  * Get email address from Address object
	  * @param addr Address
	  * @return null if no email is specified. Email address(e.g. jhc@cdr.stanford.edu)
	  * if addr contains email address information.
	  */
	public String getEmailAddress(Address addr)   {
		if(addr == null)  {
			return null;
		}

		String descript = addr.getDescription();
		if(descript == null)  {
			return null;
		}

		try  {
			KQMLmessage kqml = new KQMLmessage(descript);
			return kqml.getValue("email");
		}  catch (Exception e)  {
		}

		return null;
	}

	/**
	  * Data transfer using FTP. Only for stand alone application
	  * @param con FTPCon object
	  * @see ProtocolLayer.FTPCon
	  * @exception Abstract.ConnectionException
	  */
	public void FTP(FTPCon con)  throws ConnectionException {
		FTPThread t = null;
		try  {
			t = new FTPThread(con,Thread.NORM_PRIORITY,getThreadGroup());

			// want to get result
			t.setAgentAction(this);
			t.connect();
			t.start();
		} catch (Exception e)  {
			throw new ConnectionException("FTP Connection error " + e.toString());
		}

  }

	/**
	  * Data transfer using FTP. Only for stand alone application
	  * @param conid FTP connection name
	  * @param remotePath FTP site initial directory
	  * @param host FTP server host name
	  * @param port 21. Otherwiser, you need to set
	  * @param userName FTP server login user name
	  * @param password FTP server login password
	  * @param localPath Local machine initial directory
	  * @param files Files to put or get
	  * @param mode Mode definition
	  * @see ProtocolLayer.FTPCon
	  * @exception Abstract.ConnectionException
	  */
	public void FTP(String conid,
								  String remotePath,
								  String host,
								  int port,
								  String userName,
								  String password,
								  String localPath,
								  String[] files,
								  int mode)  throws ConnectionException {
		if((host.equals("")) ||
			 (port == -1) ||
			 (userName.equals("")) ||
			 (password.equals("")) ||
			 (files.length == 0))  {
			 throw new ConnectionException("Insufficient information for FTP put operation");
		}

		// for put mode, local path will be ignored
		FTP(new FTPCon(conid,remotePath,host,port,userName,password,localPath,files,mode));
	}

	/**
	  * Data transfer using FTP. Only for applet agents. Similar to SMTP, an applet will
	  * invoke this method first to transfer data using FTP. This method will send a
	  * KQML message like :<P>
	  * (data-transfer<BR>
	  * :sender me<BR>
	  * :receiver Router<BR>
	  * :reply-with ftp32<BR>
	  * :protocol ftp<BR>
	  * :content (get :server FTPserverInternetAddress :port 21))<P>
	  * to router. Router will create a connector between the applet and FTP server
	  * and reply a message which contains a proxy socket number. The applet will
	  * connect to the proxy socket and start to data transfer.
	  * @param con FTPCon object
	  * @exception Abstract.ConnectionException
		*/
	public void createDataTransferConnection(FTPCon con) throws ConnectionException {
		if(_myAddress == null)  {
			throw new ConnectionException("My address not specified");
		}

		String replyWith = "ftp" + _replyNo;
		_replyNo++;

		String sendmsg = "(data-transfer :sender ";
		sendmsg = sendmsg + _myAddress.getID() + " :receiver " + _routerName;
		sendmsg = sendmsg + " :reply-with " + replyWith + " :protocol ftp :content (";
		// get file
		if((con.getMode() & FTPCon.ISGETMODE) != 0)  {
			sendmsg = sendmsg + "get ";
		}
		else  	{
			sendmsg = sendmsg + "put ";
		}

		sendmsg = sendmsg + " :server " + con.getHost() + " :port " + Integer.toString(con.getPort()) + "))";

		_replyQueue.put(replyWith,con);
		try  {
			sendMessage(sendmsg);
		} catch (ParseException p)  {
			throw new ConnectionException("Parse Error : " + p.toString());
		}
  }

	/**
	  * Data transfer using FTP. Only for applet agents. Similar to SMTP, an applet will
	  * invoke this method first to transfer data using FTP. This method will send a
	  * KQML message like :<P>
	  * (data-transfer<BR>
	  * :sender me<BR>
	  * :receiver Router<BR>
	  * :reply-with ftp32<BR>
	  * :protocol ftp<BR>
	  * :content (get :server FTPserverInternetAddress :port 21))<P>
	  * to router. Router will create a connector between the applet and FTP server
	  * and reply a message which contains a proxy socket number. The applet will
	  * connect to the proxy socket and start to data transfer.
	  * @param conid FTP connection name
	  * @param remotePath FTP site initial directory
	  * @param host FTP server host name
	  * @param port 21. Otherwiser, you need to set
	  * @param userName FTP server login user name
	  * @param password FTP server login password
	  * @param localPath Local machine initial directory
	  * @param files Files to put or get
	  * @param mode Mode definition
	  * @see ProtocolLayer.FTPCon
	  * @exception Abstract.ConnectionException
		*/
	public void createDataTransferConnection(String conid,
                                                 String remotePath,
                                                 String host,
                                                 int port,
                                                 String userName,
                                                 String password,
                                                 String localPath,
                                                 String[] files,
                                                 int mode) throws ConnectionException {
          if((host.equals("")) ||
			 (port == -1) ||
			 (userName.equals("")) ||
			 (password.equals("")) ||
			 (files.length == 0))  {
			 throw new ConnectionException("Insufficient information for FTP put operation");
		}

		// for put mode, local path will be ignored
		createDataTransferConnection(new FTPCon(conid,remotePath,host,port,userName,password,localPath,files,mode));
	}


	/**
	  * Data transfer through router <-> receiver. If an agent want to trasfer data
	  * through router, the applet will create data transfer connection using
	  * createDataTransferConnection
	  * method. Router will create a proxy port and will reply a message with the
	  * port. The applet will connect to the port and start to transfer data
	  * to router and router will deliver/retrieve data and transfer to the applet.
	  * This method is for actual data transfer after receiving
	  * the port information from router.
	  * @param kqml KQML message received from router which contains a proxy socket number
	  * @exception Abstract.ConnectionException
	  */
	protected void dataTransferToRouter(KQMLmessage kqml) throws ConnectionException {
		String replyWith = kqml.getValue("in-reply-to");
		if(replyWith.equals(""))  {
			throw new ConnectionException("Reply with slot missing");
		}
System.out.println(kqml.getReadString());
		String content = kqml.getValue("content");
		String receiveremail,port;
		try  {
			KQMLmessage ckqml = new KQMLmessage(content);
			port = ckqml.getValue("port");
		}  catch (Exception e)  {
			throw new ConnectionException(e.toString());
		}

		FTPCon con = (FTPCon)_replyQueue.get(replyWith);
		if(con == null)  {
			System.out.println("No cached reply-with for " + kqml);
		}
		else  {
			Address addr = new Address("FTP",
																	(_addresses.getAddress(_routerName)).getHost(),
																	(Integer.valueOf(port)).intValue(),
																	"FTP",
																	"");

			_replyQueue.remove(replyWith);

			try  {
				FTPThread t = new FTPThread(this,addr,con,Thread.NORM_PRIORITY,getThreadGroup());
				t.setAgentAction(this);
				t.connect();
				t.start();
			}  catch (Exception e)  {
				throw new ConnectionException("FTP Connection fail::" + e.toString());
			}
		}
	}

  /**
   * Since message format is different(reading tag and time stamp), this
   * method should be overriden. At least, you need to add the received
   * KQMLmail to the _mailQueue. IPRouterClientAction should perform
   * 'processRouterMessage method first.
   * @param o KQMLmail object.
   */
  public boolean Act(Object o)  {
    String stampmsg = (String)o;

    //KQMLmail wrap the KQMLmessage
    try  {
      KQMLmail mail = new KQMLmail(stampmsg,_mailQueue.size());
      _mailQueue.addElement(mail);

      processRouterMessage(mail.getKQMLmessage());
      ////////////////////////////////////////
      //
      // Fill in what you need to do here
      //
      ///////////////////////////////////////
    } catch (Exception e)  {
      System.out.println(e.toString());
    }
    return true;
  }

  /**
  * Default behavior for the messages from router. React only the sender is router
  * @param kqml KMQLmessage, received
  */
  public void processRouterMessage(KQMLmessage kqml)
  	throws ConnectionException {
  	try {
  		String sender = kqml.getValue("sender");
  		if(sender.equals(_routerName))  {
  			String perf = kqml.getValue("performative");
  			if(perf.equals("port-info"))  {
  				String protocol = kqml.getValue("protocol");
  				if(protocol.equals("smtp"))  {
  					sendEmailToRouter(kqml);
  				}
  				else
  				if(protocol.equals("ftp"))  {
  					dataTransferToRouter(kqml);
  				}
  			}
  			else
  			if(perf.equals("agent-address"))  {
	  			Address addr = cacheAddress(kqml);
System.out.println(kqml.getReadString());
  				String inReplyTo = kqml.getValue("in-reply-to");
  				String receiver = kqml.getValue("name");
  				if((inReplyTo != null) && (!inReplyTo.equals("")))  {
	  				String kqmlmsg = (String)_replyQueue.get(inReplyTo);
  					if(kqmlmsg != null)  {
	  					_replyQueue.remove(inReplyTo);
  						if(getEmailAddress(addr) != null)  {
	  						sendEmailMessage(kqmlmsg);
  						}
  						else  {
  							SMTPCon con = new SMTPCon();
  							con.setErrorMessage("Receiver " + receiver + " does not have email address");
  							processMessage("SMTPError",	con);
  						}
  					}
  				}
  			}  // agent address
  			else
  			if(perf.equals("error"))  {
  				String inReplyTo = kqml.getValue("in-reply-to");
  				if(inReplyTo != null)  {
  					_replyQueue.remove(inReplyTo);
  				}

					processMessage("RouterError",kqml);
  			}
  		}

		} catch (ConnectionException ce)  {
			throw new ConnectionException(ce.toString());
		}

  }

	/**
   * You should implement this method. By default, messages from SMTPThread and
   * FTPThread will be managed here but you can extends whatever you want. Refer to
   * IPRCActionExample.
   * @param command Command to be executed
   * @param obj Reference object depends on command
   */
  public void processMessage(String command,Object obj)  {
  	if(command.startsWith("SMTP"))  {
  		SMTPCon con = (SMTPCon)obj;
  		if(con == null)  {
  			System.out.println("Can not process message::Null SMTPCon passed");
  		}

  		String error = con.getErrorMessage();
  		if((error == null) || (error.equals("")))  {
  			System.out.println("SMTP success");
  		}
  		else  {
  			System.out.println("SMTPError " + error);
  		}
  	}
  	else
  	if(command.startsWith("FTP"))  {
  		FTPCon con = (FTPCon)obj;
  		if(con == null)  {
  			System.out.println("Can not process message::Null FTPCon passed");
  		}

  		String error = con.getErrorMessage();
  		if((error == null) || (error.equals("")))  {
  			System.out.println("FTP success");
  		}
  		else  {
  			System.out.println("FTPError " + error);
  		}
  	}
  	else
  	// router error message
  	if(command.startsWith("Router"))  {
  		KQMLmessage kqml = (KQMLmessage)obj;
  		System.out.println("Router Error :: " + kqml.getReadString());
  	}
  }


  /**
    * Should implement this for applets. Callback from the FTPThread
    * Assume that the OutputStream will be given and wrap it with appropriate
    * IO streams such as PrintWriter or DataInputStream, etc.
    * @param fileName File name you specified in FTPCon(passed as a formal parameter
    * in createDataTransferConnection).
    * @param out OutputStream derived from a proxy socket
    * @exception Abstract.ConnectionException
    */
  public abstract void FTPStore(String fileName,/*OutputStream out*/ObjectOutputStream out) throws ConnectionException;

  /**
    * Should implement this for applets. Callback from the FTPThread
    * Assume that the OutputStream will be given and wrap it with appropriate
    * IO streams such as PrintWriter or DataInputStream, etc.
    * @param fileName File name you specified in FTPCon(passed as a formal parameter
    * in createDataTransferConnection).
    * @param out OutputStream derived from a proxy socket
    * @exception Abstract.ConnectionException
    */
  public abstract void FTPAppend(String fileName,/*OutputStream out*/ObjectOutputStream out) throws ConnectionException;

  /**
    * Should implement this for applets. Callback from the FTPThread
    * Assume that the InputStream will be given and wrap it with appropriate
    * IO streams such as PrintWriter or DataInputStream, etc.
    * @param fileName File name you specified in FTPCon(passed as a formal parameter
    * in createDataTransferConnection).
    * @param in InputStream derived from a proxy socket
    * @exception Abstract.ConnectionException
    */
  public abstract void FTPRetrieve(String fileName,/*InputStream in*/ObjectInputStream in) throws ConnectionException;
}



