package ProtocolLayer;

import Abstract.*;
import RouterLayer.Router.*;

import java.net.*;
import java.util.*;
import java.io.*;

/**
* IP Router server thread. 
* Two data members, incoming message box directiry and reservation table are added.
* The run() method will accept the connection from the IPRouter clients and create
* IPRouterRecvThread.
*/

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class IPRouterServerThread extends RouterServerThread  {
	
  /**
	* Default Constructor
	*/
	public IPRouterServerThread()  {
	}
	
	/**
	* Constructor
	* @param myaddress Router server address
	* @param connections ConnectionTable, which will be passed to the RouterRecvThread
	* @param security RouterSecurity
	* @param messageboxpath Incoming message box directory
	* @param reservedbook Message reservation table
	* @param offlinelist The off line agent name list.
	* @param disconnectlist The disconnected agents name list.
	* @exception IOException if socket error occurs
	* @exception ConnectionException if connection related error occurs
	* @exception UnknownHostException if host address is not found
	*/
	public IPRouterServerThread(Address myaddress,
														ConnectionTable connections,
														Security security,
														String messageboxpath,
														Hashtable reservedBook,
														OffLineAgentTable offlinelist,
														Vector disconnectlist) 
	throws IOException,ConnectionException,UnknownHostException {
		super(myaddress,
				  connections,
				  security,
				  messageboxpath,
				  reservedBook,
				  offlinelist,
				  disconnectlist);
  }

	/**
	* Run method will create RouterRecvThread by accepting connection.
	* endWith character and maximum idle time will be set to the RouterRecvThread
	*/
	public void run()  {
    try {
      while (true) {
				Socket client_socket = _listenSocket.accept();
				createReceiverThread(client_socket);	
			}	
    }  catch (IOException e) {
    }	
  } 
 
 	/**
 	* Create RouterReceiverThread and invoke start() method
 	* @param socket Socket, created by the ServerSocket
 	*/
  protected synchronized void createReceiverThread(Socket socket)  {
  	try  {
			IPRouterRecvThread c = new IPRouterRecvThread(socket,
									Thread.NORM_PRIORITY,_myaddress,_connections,null,_security,_messageBoxPath,_reservedBook,_offLineList,_disconnectTable);
															
			// do not forget to set end with character if it is not '\004'					
			c.setEndWith(_endWith);	
System.out.println("IPRecvThread created");							
			if(_durationTime != -1)  {
				c.setDurationTime(_durationTime);
			}	
				
			c.start();
		} catch (ConnectionException cc)  {
		}	
  } 	
 	

}				  	