package ProtocolLayer;

import Abstract.*;
import BaseLayer.*;

import java.net.*;
import java.lang.*;

/**
  * Security performed for SMTP.
  */
/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class SMTPSecurity extends Security   {
	
	/**
	* Default constructor. By default, no password checking is assumed
	*/
  public SMTPSecurity()  {
		super();
	}
	
	/**
	* SMTP is only for client side security check. No implementation. 
	* This will be override if new register protocol is added.
	* @param c ReceiverThread will be started
	*/
	public void processServerLogin(ReceiverThread c) {}; 
	
  /**
  * Opposite to the processServerLogin. Connection should be added to the
  * connection table, by caller.
  * @param c ReceiverThread, which will connect to the other server
  * @param serveraddress Other agent address where I want to connect
  * @exception ConnectionException if error occurs including SMTP
  */
  public void processClientLogin(ReceiverThread c,Address serveraddress) 
  	throws ConnectionException  {
  	if(serveraddress == null)  {
  		System.out.println("Null email server address is passed");
  		throw new ConnectionException("Null email server address is passed");
  	}
  	
  	String retstr = c.getOneMessage();
  	if(!retstr.startsWith("220"))  {
  		System.out.println("Can not connect to email server " + serveraddress.getHost());
  		throw new ConnectionException("Can not connect to email server " + serveraddress.getHost());
		}
		  		
  	String opencmd = "HELO " + serveraddress.getHost();
  	c.writeMsg(opencmd);
  	retstr = c.getOneMessage();
  	if(!retstr.startsWith("250"))  {
  		System.out.println("Can not connect to email server " + serveraddress.getHost());
  		throw new ConnectionException("Can not connect to email server " + serveraddress.getHost());
  	}	
  }	

	/**
	* Send disconnect message to the server. The ReceiverThread is initiated
	* by me.
	* @param c ReceiverThread, which will stop.
	* @param myid My name, may be passed to the server to disconnect gracefully
	*/
  public void processClientLogout(ReceiverThread c) {
  	try  {
  		c.writeMsg("QUIT");
  	}  catch (ConnectionException e)  {}	
  } 
  
  /**
  * No implementation for SMTP
  * @param obj Object which dependent upon the security implementation
  * @return true if the agent is valid else, retur false
  */
  public boolean isValidAgent(Object o)  {
  	return true;
  }	
  
}		
			
	
				