package RouterLayer.AgentClient.Example.NBanner;

import KQMLLayer.*;
import Abstract.*;
import RouterLayer.AgentClient.*;

import java.awt.*;
import java.util.*;
import java.applet.*;
import java.net.*;
import java.lang.*;

/**
* This is a simple example of the JATLite applet. 
* First, you will need to implement RouterClientAction like BannerAction class, to notify
* a message(actually invoke the processMessage(KQMLmail) method of BannerApplet object) to
* the BannerApplet. Refer to BannerAction.java. The received message will be parsed and
* performed according to the message content.
* See the init() method. It will show that how the BannerAction object created.
* See the action(Event,Object) method. Several methods of the RouterClientAction will be
* invoked.
*/

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class NBannerFrame extends Frame   {

  NBannerAction _agentAction;
  
	NBannerApplet _parent;
	  
  TextArea _in;
  TextArea _out;
  BannerDisplayCanvas _canvas;
  
  Address _myAddress;
  Address _routerAddress;
  Address _registrarAddress;
  
  public NBannerFrame(Address myaddress,
  										Address routeraddress,
  										Address registraraddress,
  										String controlID)  {
  	super(myaddress.getID());
  	_myAddress = myaddress;
  	_routerAddress = routeraddress;
  	_registrarAddress = registraraddress;
  	
    // construct the BannerAction object
    _agentAction = new NBannerAction(_myAddress,_routerAddress,_registrarAddress);
    
    // set applet to be notified the received message from the BannerAction
    _agentAction.setAgentFrame(this);

  	initGUI();
  	
  }
  
  public void setApplet(NBannerApplet p)  {
  	
  	_parent = p;
  }
  
  public Address getMyAddress()  {
  	return _myAddress;
  }	
  	
  /**
   */
  public void initGUI() {
    _in = new TextArea(10,40);
    _in.setText("Received Message\r\n");

    _out = new TextArea(10,40);
	  _out.setText("Sending area\r\n");
    
    setLayout(new GridLayout(3,1));
    add(_in);
    add(_out);
		_canvas = new BannerDisplayCanvas(this);
		add(_canvas);
		    
    resize(400,320);
  }

	public void register()  throws ConnectionException {
		try  {
      _in.appendText("Register to router as " + _myAddress.getID() + "...\r\n");
      _agentAction.register();
      _in.appendText("Register success\r\n");
    }  catch (Exception e)  {
    	_in.appendText("Register fail " + e.toString());
    	throw new ConnectionException(e.toString());
    }	  
	}
	
	public void unregister() throws ConnectionException {
    try  {
      _in.appendText("Unregister from router..\r\n");
      _agentAction.unregister();
      _in.appendText("Unregister success\r\n");
		}  catch (Exception e)  {
			_in.appendText("Unregister fail " + e.toString());
    	throw new ConnectionException(e.toString());
			
		}
	}
		
	public void connect() throws ConnectionException  {
    try  {
      _in.appendText("Connecting ...\r\n");
      _agentAction.connect();
      _in.appendText("Connect success\r\n");
      _agentAction.start();
      
    	_canvas.start();
      
		}  catch (Exception e)  {
			_in.appendText("Connect fail " + e.toString());
    	throw new ConnectionException(e.toString());
			
		}
		
  }	
  
	public void disconnect() throws ConnectionException  {
    try  {
      _in.appendText("Disconnecting ...\r\n");
      _agentAction.disconnect();
      _in.appendText("Disconnect success\r\n");
		}  catch (Exception e)  {
			_in.appendText("Disconnect fail " + e.toString());
    	throw new ConnectionException(e.toString());
			
		}
  }	
  

	public void processMessage(KQMLmail mail)  {
		_canvas.processMessage(mail);
	}
		  
  
  public void sendResponse(String receiver, String content)  {
    String sendmsg = "(tell :sender "; 
    sendmsg = sendmsg + _myAddress.getID() + " :receiver " + receiver + " :content ";
    sendmsg = sendmsg + content + ")";
    
    try  {
    
	    _agentAction.sendMessage(sendmsg);
    
  	  _out.appendText("===================================================\r\n");
//    	_parent.increaseSendingMessageNumber("NBanner");
    
      _out.appendText((new KQMLmessage(sendmsg)).getReadString());
    }  catch (Exception e) {}	
  }	   
  
  public void appendInputText(String str) {
  	_in.appendText(str);
  }
  
  public void appendOutputText(String str)  {
  	_out.appendText(str);
  }
  
  public void sendMessage(String str)   {
  	try  {
  		_agentAction.sendMessage(str);
  	}  catch (Exception e)  {
  		MessageBox b = new MessageBox("Message can not be sent! Connection broken",this);
  	}		
  }
  			
  public boolean handleEvent(Event evt)   {  
 
    if (evt.id == Event.WINDOW_DESTROY)   {
			stop();
	  }
    
    return super.handleEvent(evt);
  }  
  	
  /**
   */  
  public void stop()  {
    if(_canvas != null)  {
      _canvas.stop();
    }
    
    if(_agentAction != null)  {
      try {
	_agentAction.unregister();
	_agentAction.endAction();
      } catch (Exception e)  {
      }		
    }		
    
    if(_parent != null)   {
    	_parent.notifyClose("NBanner");
    }
    	
    dispose();
  }   
    
  
}

class BannerDisplayCanvas extends Canvas implements Runnable  {
  /**
  * Runnable thread fetcher
  */  
  Thread _fetcher;

  /**
   * Displaying text
   */
  String _display;
  
  /**
   * Displaying text font
   */
  Font _font;
  
  /**
   * Displaying speed
   */
  int _speed;
  
  /**
   * Double buffered image
   */
  Image _image;
  
  /**
   * Image width
   */
  int _imagex;
  
  int _currentX;
  /**
   * Displayign text color
   */
  Color _textColor;
  
  /**
   * Displaying background
   */
  Color _bgColor;
	
  boolean _suspended = false;
  NBannerFrame _parent;
	
	public BannerDisplayCanvas(NBannerFrame parent)   {
 	  _fetcher = new Thread(this);
    // speed unit 100.
    _speed = 100;
      
    // default displaying text
    _display = "JATLite Banner Example";
  	
    _font = new java.awt.Font("TimesRoman", Font.PLAIN, 24);
    _bgColor = getBackground();
    _textColor = Color.blue;
    _parent = parent;
    resize(400,100);
	}
			
	public void start()  {
		makeImage();
		
		if(_fetcher != null)  {
			_fetcher.start();
		}
	}
						
  protected synchronized void makeImage()  {
    Graphics myg = getGraphics();
    myg.setFont(_font);
    FontMetrics fm = myg.getFontMetrics();
    
    Dimension d = size();	
    _imagex = fm.stringWidth(_display)  + d.width;
    
    int imagey = _font.getSize() + 10;
    _image = createImage(_imagex,imagey);
    
    Graphics g = _image.getGraphics();
    g.setColor(_bgColor);
    
    g.fillRect(0,0,_imagex,imagey);
    g.setColor(_textColor);
    g.setFont(_font);
    g.drawString(_display,0,imagey - 10);
  }    
	
  /**
   * run method
   */
  public void run() {
    while(true)  {
      if(_image != null) {
				int unitmove = 5;
				Dimension d = size();
				int nofcount = _imagex / unitmove + unitmove;
	
				for(int i=0; i < nofcount; i++)  {
					_currentX = d.width - i * unitmove;
					repaint();
 	  			try {
	    			_fetcher.sleep(_speed);
	  			} catch (InterruptedException e)  {}
				}
      }	
      else  {
      	try  {
      		_fetcher.sleep(1000);
      	}  catch (Exception e)  {}	
      }	
    }
  }
  
  public void paint(Graphics g)   {
  	if(_image != null)  {
  		int y = 1 * size().height/3;
		  g.drawImage(_image,_currentX,y,this);
		}  			
  	
  }
	
  /**
   * If the message queue of the RouterClientAction is not empty, the RouterClientAction will
   * invoke this method. By parsing KQMLmail and KQMLmessage contained in the KQMLmail, 
   * proper action will be performed
   * @param mail KQMLmail, received from the RouterClientAction
   */    
    public void processMessage(KQMLmail mail)  {
      // use KQMLmail getKQMLmessage() to get KQMLmessage object
      KQMLmessage kqml = mail.getKQMLmessage();
      _parent.appendInputText("====================================================\r\n");
//	    _parent.increaseReceivingMessageNumber("NBanner");
      
      _parent.appendInputText(kqml.getReadString());
      String receiver = kqml.getValue("sender");
      
      try  {
      	String perf = kqml.getValue("performative");
      	if(perf.equalsIgnoreCase("ask"))  {
	  String replyWith = kqml.getValue("reply-with");
	  String retstr = "(reply :sender ";
	  retstr = retstr + _parent.getMyAddress().getID() + " :receiver " + receiver;
	  retstr = retstr + " :content (font-list ";

	  String[] flist = getToolkit().getFontList();
	  for(int i=0; i < flist.length; i++)  {
	  	retstr += flist[i] + " ";
	  }
	  	
	  retstr = retstr + ")";
	  if(replyWith != null)  {
	    retstr = retstr + " :in-reply-to " + replyWith;
	  }
	  retstr += ")";	
	  _parent.sendMessage(retstr);
	  _parent.appendOutputText("===================================================\r\n");
//    _parent.increaseSendingMessageNumber("NBanner");
	  
	  try  {
	    _parent.appendOutputText((new KQMLmessage(retstr)).getReadString());
	  }  catch (Exception e) {}	
	  
      	}
      	else
	  if(perf.equalsIgnoreCase("tell"))  {	
	    // this is a typical way to find out the field value
	    String content = kqml.getValue("content");
	    if(content != null)  {
	      if(content.startsWith("("))  {	    
		Vector v = new takeOffList(content);
		if(v.size() == 0)  {
		  return;
		}
		
		String cperf = (String)v.elementAt(0);
		if(cperf.equals("change-font")) {
		  if(v.size() == 2)  {
		    _font = new Font((String)v.elementAt(1), Font.PLAIN, 24);
		  }
		}
		else
		  if(cperf.equals("change-color"))  {
		    if(v.size() == 3)  {
		      String back = (String)v.elementAt(1);
		      String text = (String)v.elementAt(2);
		      
		      Vector vv = new takeOffList(back);
		      String r = (String)vv.elementAt(0);
		      String g = (String)vv.elementAt(1);
		      String b = (String)vv.elementAt(2);
		      
		      Color backColor = new Color((Integer.valueOf(r)).intValue(),
						  (Integer.valueOf(g)).intValue(),
						  (Integer.valueOf(b)).intValue());
		      _bgColor = backColor;					 																	 
		      vv = new takeOffList(text);
		      r = (String)vv.elementAt(0);
		      g = (String)vv.elementAt(1);
		      b = (String)vv.elementAt(2);
		      
		      Color textColor = new Color((Integer.valueOf(r)).intValue(),
						  (Integer.valueOf(g)).intValue(),
						  (Integer.valueOf(b)).intValue());
		      _textColor = textColor;	
		      
		    }																	 
		    
		  }
		  else
		    if(cperf.equals("change-text"))    {
		      if(v.size() == 2)  {
			String text = (String)v.elementAt(1);
			_display = text.substring(1,text.length() -1);
		      }
		    }
		
		makeImage();
		_parent.sendResponse(receiver,cperf);
	      }
	      else  {
						if(content.startsWith("increase"))  {
		  				_speed /= 2;
						}
						else
						if(content.startsWith("decrease"))  {
		   				_speed *= 2;
						}
						else
						if(content.startsWith("play"))  {
		  				_suspended = false;
		  				_fetcher.resume();
						}
						else
						if(content.startsWith("pause"))  {
		  				_suspended = true;
		  				_fetcher.suspend();
						}
						_parent.sendResponse(receiver,content);
	      } // content starts with (
	    }		// content not null	
	  }	
      }  catch (Exception e)  {
	
	// Message box
      }
    }								
	
	public void stop()  {
		if(_fetcher != null)  {
			_fetcher.stop();
			_fetcher = null;
		}
	}		
}	