package RouterLayer.AgentClient.Example.RCApplet;

import Abstract.*;
import BaseLayer.*;
import KQMLLayer.*;
import RouterLayer.Router.*;
import RouterLayer.AgentClient.*;

import java.net.*;
import java.lang.*;
import java.io.*;
import java.util.*;

/**
* A RouterClient example. This example implements the Act(Object) and processMessage(String,String[]),
* those are declared as abstract in the RouterClientAction class. The Act(Object) method will pass
* the received message to the RCApplet object to display the message. 
*/

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class RCActionExample extends RouterClientAction  {

	// since this example use GUI frame
	RCFrame _agentFrame;

	// set GUI	
	public void setRCFrame(RCFrame f)  {
		_agentFrame = f;
		((RouterSecurityWithGUI)_security).setRCFrame(f);
	}

	public RCActionExample()  {
		super();
	}

	/**
	* Normally invoked by a stand-alone agent
	*/
	public RCActionExample(Address myaddress,
						   Address routeraddress,
						   Address registraraddress,	
						   int durationTime)  {
		super(myaddress,routeraddress,registraraddress,durationTime);
		
		// override default security with RouterSecurityWithGUI
		_security = new RouterSecurityWithGUI(_addresses,null);
	
		// createServerThread, if it is not an applet
		if(myaddress.getPort() != -1)  {
			createServerThread(myaddress.getID(),Thread.NORM_PRIORITY);
		}	
	}

	
	public RCActionExample(Address myaddress,
						   Address routeraddress,
						   Address registraraddress,	
						   int durationTime,
						   ThreadGroup t)  {
		super(myaddress,routeraddress,registraraddress,durationTime,t);
		
		// override default security with RouterSecurityWithGUI
		_security = new RouterSecurityWithGUI(_addresses,null);
	
		// createServerThread, if it is not an applet
		if(myaddress.getPort() != -1)  {
			createServerThread(myaddress.getID(),Thread.NORM_PRIORITY);
		}	
	}

	/**
	* Override the register method of the RouterClientAction to support GUI
	* @param registrar The Router Registrar address
	* @param myaddress My address to be registered
	* @exception ConnectionException for any error occurs during processing this service
	*/	
	public synchronized void register(Address registrar,Address myaddress)  
	throws ConnectionException  {
		try  {
			_addresses.addAddress(myaddress);
			_addresses.addAddress(registrar);
			
			// This part has been changed to support GUI
			RegistrarSecurityWithGUI rsw = 
				new RegistrarSecurityWithGUI((RouterSecurity)_security);
			rsw.setRCFrame(_agentFrame);	
				
			RegistrarRecvThread rth = new RegistrarRecvThread(registrar,
																	Thread.NORM_PRIORITY,myaddress,null,null,rsw,getThreadGroup());
			rth.setEndWith(_endWith);															
			rth.setDurationTime(10);
			rth.start();
		} catch (Exception e)  {
			throw new ConnectionException(e.toString());
		}		
	}	
	
		
 	/**
 	* Example to display the message to the _agentApplet.
 	*/
 	public boolean Act(Object o)  {
 		String stampmsg = (String)o;

 		try  {
	 		//KQMLmail wrap the KQMLmessage
 			KQMLmail mail = new KQMLmail(stampmsg,_mailQueue.size());
 			
 			// add the mail to the _mailQueue.
 			_mailQueue.addElement(mail);
 			
 			// agentApplet will update the list panel
			_agentFrame.processMessage(mail);
 		} catch (Exception e)  {
 			return false;
 		}

 		return true;
  }

	/**
	* This class does not use the processMessage
	*/
  public void processMessage(String command,Object obj)  {
  }

	/**
	* Send delete-message message to the Router and update the GUI
	*/
  public void sendDeleteMessage()  {
  	super.sendDeleteMessage();
 		_agentFrame.updateListPanel();
  }

}