/**
 * @(#)Tab.java	2.2 11/25/96 Rodney Ryan (rer@interport.net)
 *
 * Copyright (c) 1996 Rodney Ryan, All Rights Reserved.
 *
 */

package RouterLayer.AgentClient.Example.RCApplet.extawt;

import java.awt.*;

/**
 * A class to display a tab with an inset name.
 *
 * @author 	Rodney Ryan (rer@interport.net)
 * @version 	2.2 11/25/96
 *
 */
public class Tab extends Rectangle
{

    /**
    * The unselected Font.
    */
    public static Font  normalFont;

    /**
    * The selected Font.
    */
    public static Font  selectFont;

    /**
    * The foreground Color
    */
    public static Color foreground;

    /**
    * The background Color
    */
    public static Color background;

    /**
    * The agreed height of all the Tabs.
    */
    private static int tabHeight = 0;

    /**
    * The name of the Tab.
    */
    private String  name;

    /**
    * Indicates whether the Tab is enabled or not.
    */
    private boolean enabled = true;

    /**
    * Indicates whether the Tab is selected or not.
    */
    private boolean selected = false;

    /**
    * Constructs a new Tab.
    * @param name the name of the Tab
    */
    public Tab(String name)
    {
        super();
        this.name = name;
    }

    /**
    * Returns the name of the Tab.
    */
    public String getName()
    {
        return name;
    }

    /**
    * Indicates whether the Tab is enabled or not.
    */
    public boolean enabled()
    {
        return enabled;
    }

    /**
    * Enables the Tab.
    */
    public void enable()
    {
        enabled = true;
    }

    /**
    * Disables the Tab.
    */
    public void disable()
    {
        enabled = false;
    }

    /**
    * Indicates whether the Tab is selected or not.
    */
    public boolean selected()
    {
        return selected;
    }

    /**
    * Selects the Tab.
    */
    public void select()
    {
        selected = true;
    }

    /**
    * Deselects the Tab.
    */
    public void deselect()
    {
        selected = false;
    }

    /**
    * Returns true if the given point is inside the area of the Tab or not.
    * @param x the x coordinate of the point
    * @param y the y coordinate of the point
    */
    public boolean inside(int x,
                          int y)
    {
        if (enabled) return super.inside(x, y);
            else return false;
    }

    /**
    * Paints the Tab.
    * @param g the Graphics
    */
    public void paint(Graphics g)
    {
        if (enabled) {
            drawTab(g);
            drawName(g);
        }
    }

    /**
    * Draws the Tab.
    * @param g the Graphics
    */
    public void drawTab(Graphics g)
    {
        int grow = (selected) ? 2 : 0;

        g.setColor(background);
        if (!selected) {
            g.fillRect(x, y + 2, width, tabHeight - 2);
            g.drawLine(x + 1, y + 1, width + x - 2, y + 1);
        } else {
          g.fillRect(x - 1, y, width + 2, tabHeight);
          g.drawLine(x, y - 1, x + width, y - 1);
        }
        // left side, rounded corner and top side lit
        g.setColor(Color.white);
        g.drawLine(x - grow, y + tabHeight - 2, x - grow, y + 2 - grow);
        g.drawLine(x - grow, y + 2 - grow, x + 2 - grow, y - grow);
        g.drawLine(x + 2 - grow, y - grow, x + width - 2 + grow, y - grow);
        if (!selected) {
          // right side shadowed
          g.setColor(Color.gray);
          g.drawLine(x + width - 1, y, x + width - 1, y + tabHeight - 2);
          // shadow for right side
          g.setColor(Color.black);
          g.drawLine(x + width, y + 2, x + width, y + tabHeight - 2);
        } else {
          // right side shadowed
          g.setColor(Color.gray);
          g.drawLine(x + width - 1 + grow, y - grow, x + width - 1 + grow, y + tabHeight - 2);
          // shadow for right side
          g.setColor(Color.black);
          g.drawLine(x + width + grow, y + 2 - grow, x + width + grow, y + tabHeight - 2);
          // open line between tab and contents
          g.setColor(background);
          g.drawLine(x - grow + 1, y + tabHeight, x + width, y + tabHeight);
        }
    }

    /**
    * Draws the name of the Tab.
    * @param g the Graphics
    */
    public void drawName(Graphics g)
    {
        int         captionX;
        int         captionY;
        FontMetrics fm;

        if (selected) {
          g.setFont(selectFont);
          fm = g.getFontMetrics(selectFont);
        } else {
          g.setFont(normalFont);
          fm = g.getFontMetrics(normalFont);
        }
        captionX = x + ((width - fm.stringWidth(name)) / 2);
        captionY = y + (tabHeight / 2) - fm.getDescent() + (fm.getHeight() / 2);
        g.setColor(foreground);
        g.drawString(name, captionX, captionY);
    }

    /**
    * Sets the foreground Color.
    * @param foreground the new foreground color
    */
    public static void setForeground(Color foreground)
    {
        Tab.foreground = foreground;
    }

    /**
    * Sets the background Color.
    * @param background the new background color
    */
    public static void setBackground(Color background)
    {
        Tab.background = background;
    }

    /**
    * Sets the normal Font
    * @param normalFont the normal Font
    */
    public static void setNormalFont(Font normalFont)
    {
        Tab.normalFont = normalFont;
    }

    /**
    * Sets the selected Font
    * @param selectFont the selected Font
    */
    public static void setSelectFont(Font selectFont)
    {
        Tab.selectFont = selectFont;
    }

    /**
    * Sets the Tab height
    * @param tabHeight the new Tab height
    */
    public static void setTabHeight(int tabHeight)
    {
        Tab.tabHeight = tabHeight;
    }

    /**
    * Returns the Tab height
    */
    public static int getTabHeight()
    {
        return Tab.tabHeight;
    }
}