/**
 * @(#)TabbedFolder.java	2.2 11/25/96 Rodney Ryan (rer@interport.net)
 *
 * Copyright (c) 1996 Rodney Ryan, All Rights Reserved.
 *
 */

package RouterLayer.AgentClient.Example.RCApplet.extawt;

import java.awt.*;

/**
 * TabFolder holds Components identified by a name in the structure of a
 * "tab folder".  When a tab is selected, the corresponding Component is
 * shown.
 *
 * TabFolder contains a TabStrip to hold the Tabs, a TabPanel to hold the
 * related Components, and a CardLayout to show the selected Component.
 *
 * @author 	Rodney Ryan (rer@interport.net)
 * @version 	2.2 11/25/96
 *
 */
public class TabFolder extends Panel
{

    /**
    * The default Font for unselected tabs.
    */
    protected static Font  defaultNormal = new Font("Helvetica", Font.PLAIN, 11);

    /**
    * The default Font for selected tabs.
    */
    protected static Font  defaultSelect = new Font("Helvetica", Font.BOLD, 12);

    /**
    * The tabStrip variable.
    */
    protected TabStrip   tabStrip;

    /**
    * The tabPanel variable.
    */
    protected TabPanel   tabPanel;

    /**
    * The cardLayout variable.
    */
    protected CardLayout cardLayout;

    /**
    * Constructs a new TabFolder.
    * @param firstTab the name of the first tab
    * @param firstComponent the component corresponding to the first tab
    * @param normalFont the Font for unselected tabs
    * @param selectFont the Font for selected tabs
    */
    public TabFolder(String    firstTab,
                     Component firstComponent,
                     Font      normalFont,
                     Font      selectFont)
    {
        super();
        Tab.setNormalFont(normalFont);
        Tab.setSelectFont(selectFont);
        tabStrip = new TabStrip(firstTab);
        tabPanel = new TabPanel();
        cardLayout = new CardLayout();
        tabPanel.setLayout(cardLayout);
        tabPanel.add(firstTab, firstComponent);
        setLayout(new BorderLayout());
        add("Center", tabPanel);
        add("North", tabStrip);
    }

    /**
    * Constructs a new TabFolder.
    * @param firstTab the name of the first tab
    * @param firstComponent the Component corresponding to the first tab
    */
    public TabFolder(String    firstTab,
                     Component firstComponent)
    {
        this(firstTab, firstComponent, defaultNormal, defaultSelect);
    }

    /**
    * Ensures the tabPanel is repainted correctly.
    */
    public void validate()
    {
        super.validate();
        tabPanel.repaint();
    }

    /**
    * Adds a tab and corresponding Component.
    * @param tabName the name of the tab
    * @param component the corresponding component
    */
    public void addTab(String    tabName,
                       Component component)
    {
        tabStrip.addTab(tabName);
        tabPanel.add(tabName, component);
    }

    /**
    * Handles the given event.
    * @param e the Event to be handled
    */
    public boolean handleEvent(Event e)
    {
        if (e.id == TabStrip.TAB_SELECTED) {
            cardLayout.show(tabPanel, (String)e.arg);
            tabPanel.repaint();
            return true;
        } else return super.handleEvent(e);
    }
}