/**
 * @(#)TabStrip.java	2.2 11/25/96 Rodney Ryan (rer@interport.net)
 *
 * Copyright (c) 1996 Rodney Ryan, All Rights Reserved.
 *
 */

package RouterLayer.AgentClient.Example.RCApplet.extawt;

import java.awt.*;

/**
 * TabStrip contains TabCanvas and the two scroll buttons.
 *
 * @author 	Rodney Ryan (rer@interport.net)
 * @version 	2.2 11/25/96
 *
 */
public class TabStrip extends Panel
{
    /**
    * The event id for scrolling one tab back.
    */
    public static final int TABS_BACK    = 5000;

    /**
    * The event id for scrolling one tab forward.
    */
    public static final int TABS_FORWARD = 5001;

    /**
    * The event id for selecting a new tab.
    */
    public static final int TAB_SELECTED = 5003;

    /**
    * The TabCanvas variable.
    */
    protected TabCanvas     tabCanvas;

    /**
    * The back button variable.
    */
    protected JavaButton    backButton;

    /**
    * The forward button variable.
    */
    protected JavaButton    forwardButton;

    /**
    * Constructs a new TabStrip with the indicated first tab name.
    * @param firstTabName the name of the first tab
    */
    public TabStrip(String firstTabName)
    {
        super();
        tabCanvas = new TabCanvas(firstTabName);
        backButton = new JavaButton(new ArrowButtonFace(ArrowButtonFace.WEST), TABS_BACK);
        forwardButton = new JavaButton(new ArrowButtonFace(ArrowButtonFace.EAST), TABS_FORWARD);
        layoutComponents();
    }

    /**
    * Lays out the TabStrip and the two scroll buttons.
    */
    protected void layoutComponents()
    {
        GridBagLayout gridBag = new GridBagLayout();
        setLayout(gridBag);
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.BOTH;
        c.weightx = 1;
        gridBag.setConstraints(tabCanvas, c);
        add(tabCanvas);
        c.weightx = 0;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.SOUTH;
        gridBag.setConstraints(backButton, c);
        add(backButton);
        gridBag.setConstraints(forwardButton, c);
        add(forwardButton);
    }

    /**
    * Adds a new tab.
    * @param name the name of the new tab
    */
    public void addTab(String name)
    {
        tabCanvas.addTab(name);
        tabCanvas.forcePaint();
    }

    /**
    * Forces the tabCanvas instance to fully redraw.
    */
    public void forcePaint()
    {
        tabCanvas.forcePaint();
    }

    /**
    * Returns true if the two scroll buttons are both presently visible.
    */
    protected boolean scrollButtonsVisible()
    {
        return ((backButton.isVisible()) && (forwardButton.isVisible()));
    }

    /**
    * Returns true if there is no need for the scroll buttons, i.e. both the
    * first and last tab are visible.
    */
    protected boolean scrollButtonsNotUseful()
    {
        return ((tabCanvas.firstTabVisible()) && (tabCanvas.lastTabVisible()));
    }

    /**
    * Checks if the scroll buttons are needed.  If they are, disables the forward
    * button if the last tab is visible and disables the back button if the last
    * tab is visible.  If they are not needed, hides both buttons.
    */
    protected void checkArrowButtons()
    {
        if (scrollButtonsNotUseful()) {
            if (scrollButtonsVisible()) {
                backButton.hide();
                forwardButton.hide();
                layout();
            }
        } else {
            if (!scrollButtonsVisible()) {
                backButton.show();
                forwardButton.show();
                layout();
            }
        }
        if ((tabCanvas.atFirstTab()) && (!backButton.disabled())) {
            backButton.disable();
            backButton.repaint();
        } else if ((!tabCanvas.atFirstTab()) && (backButton.disabled())) {
            backButton.enable();
            backButton.repaint();
        }
        if ((tabCanvas.lastTabVisible()) && (!forwardButton.disabled())) {
            forwardButton.disable();
            forwardButton.repaint();
        } else if ((!tabCanvas.lastTabVisible()) && (forwardButton.disabled())) {
            forwardButton.enable();
            forwardButton.repaint();
        }
    }

    /**
    * Handles the given event.
    * @param e the Event to be handled
    */
    public boolean handleEvent(Event e)
    {
        if (e.id == TABS_BACK) {
            tabCanvas.moveBack();
            checkArrowButtons();
            return true;
        } else if (e.id == TABS_FORWARD) {
            tabCanvas.moveForward();
            checkArrowButtons();
            return true;
        } else if (e.id == TabCanvas.TAB_SELECTED) {
            tabCanvas.forcePaint();
            postEvent(new Event(getParent(), TAB_SELECTED, (String)e.arg));
            return true;
        } else if (e.id == TabCanvas.POSITIONING_CHANGED) {
            checkArrowButtons();
            return true;
        } else return super.handleEvent(e);
    }
}