package RouterLayer.Router;

import Abstract.*;
import java.util.*;

/**
* Table which contains the off line agent list. 'Off-line' agent is the agent,
* which does not send 'disconnect-agent' message to the router, but to which
* the router can not send message.(normally accidental disconnection from the 
* router client side). If the agent is an applet, it will not be added to the
* off line agent list, since the router can not initiate the connection to the
* applet. 
*/

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class OffLineAgentTable extends Hashtable  {
	
	/**
	* Maximum trial times to try to initiate the connection to the agent
	*/
	int _maxTrialTimes;
	
	/**
	* Connection time check period in mimutes
	*/
	int _checkPeriod;	
	
	/**
	* Constructor
	* @param maxtrial Maximum trial times to send message
	* @param checkperiod Check period in minutes
	*/
	public OffLineAgentTable(int maxtrial, int checkperiod)  {
		_maxTrialTimes = maxtrial;
		_checkPeriod = checkperiod;
	}
	
	/**
	* Agent off line is only applied to the stand alone agent. If the agent is an applet,
	* the agent will not be added to the off line list
	* @param address Add off-line agent to the table
	*/
	public void addAgent(Address address)  {
		try  {
			String agentid = address.getID();
			if(address.getPort() != -1)  {
				Reservation res = 
					new Reservation(agentid,address,Long.toString(System.currentTimeMillis()),_maxTrialTimes);
				put(agentid,res);																			
			}
		} catch (Exception e)  {}
	}
}	
				
		