package RouterLayer.Router;

import java.net.*;
import java.io.*;
import java.util.*;

import Abstract.*;
import BaseLayer.*;

/**
* Registrar server thread.
* The Router has two server socket, one for registration and one for message passing 
* The RegistrarServerThread is accepting the connection and generate RouterRecvThread
* to get registration information.
* @see RouterLayer.Router.RegistrarRecvThread
*/

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class RegistrarServerThread extends BServerThread {

	/**
	* The Router address to be informed to the agent
	*/
	Address _routerAddress;
	
	/**
	* Default constructor
	*/
	public RegistrarServerThread()  {
		super();
	}
		 
  /**
  * Constructor using Address object
  * @param routeraddress Address object which contains the router address
  * @param registraraddress Address object which contains the registrar address
  * @param connections connection table
  * @param mesg the message queue. Nornmally null
  * @param security RegistrarSecurity
  * @exception Abstract.ConnectionException if connection error occurs
  * @exception java.net.UnknownHostExceptoin if host unknown
  * @exception java.io.IOException if io error occurs.
  */
  public RegistrarServerThread(Address routeraddress,
  											 Address registraraddress,
  											 ConnectionTable connections,
  										   MessageBuffer mesg, // will be null
  										   Security security)
	throws ConnectionException,UnknownHostException,IOException	   {
		super(registraraddress,connections,mesg,security);
		_routerAddress = routeraddress;
	}	

		
	/**
	* Run method override
	* Accept the connection for registration and create RegistrarRecvThread
	*/
  public void run()  {
    
    try {
      while (true) {
				Socket client_socket = _listenSocket.accept();

				RegistrarRecvThread c = new RegistrarRecvThread(client_socket,
									Thread.NORM_PRIORITY,_routerAddress,_connections, null,_security);
									
				// do not forget to set end with character if it is not '\n'					
				c.setEndWith(_endWith);					
				if(_durationTime != -1)  {
					c.setDurationTime(_durationTime);
				}	
				
				c.start();
			}
    }  catch (IOException e) {
    }  catch (ConnectionException ee)  {
    }	
  } 
  
}
    					