package RouterLayer.Router;

import java.lang.*;
import Abstract.*;

/**
* Reservation class to reserve message to the Router. The Reservation object
* will be created by RouterRecvThread and it will be added to the _reservedBook
* data member of RouterAction. While looping, the RouterAction will check 
* that the Reservation time. If the time is less than the current time,
* the Router will send message to the reserved receiver.
*/

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 
public class Reservation  {

	/**
	* Reserved time
	*/
	String _time;
	
	/**
	* Intended receiver
	*/
	String _receiver;
	
	/**
	* Current times to try to send. At first, it is 0 and increased by 1
	*/
	int _tryTimes;
	
	/**
	* Maximum trial times to send the message. If the trial times are greater than
	* maxTimes, the reservation will be canceled
	*/
	int _maxTimes;
	
	/**
	* Intended receiving location. Null if the agent wants to receive at the most
	* recent location
	*/
	Address _address;	
	
	/**
	* Constructor
	* @param receiver Intended receiver. May be different from the reserver
	* @param address Intended receiving location. Can be passed as null if the location
	*  to receive is not different from the most recent connection location
	* @param time Reservation time stamp
	* @param maxTimes Maximum trial times
	*/
	public Reservation(String receiver,
										 Address address,
										 String time,
										 int maxTimes)  {
		_time = time;
		_receiver = receiver;
		_tryTimes = 0;
		_address = address;
		_maxTimes = maxTimes;
	}
	
	/**
	* Constructor
	* @param receiver Intended receiver. May be different from the reserver
	* @param address Intended receiving location. Can be passed as null if the location
	*  to receive is not different from the most recent connection location
	* @param time Reservation time stamp
	*/
	public Reservation(String receiver,
										 Address address,
										 String time)  {
		_time = time;
		_receiver = receiver;
		_tryTimes = 0;
		_address = address;
	}
	
	/**
	* Increase trial times by one
	*/
	public void incTimes()  {
		_tryTimes++;
	}
	
	/**
	* Get trial times
	* @return Current trial times
	*/
	public int getTryTimes()   {
		return _tryTimes;
	}				
	
	/**
	* Get reserved address
	* @return Reserved location address
	*/
	public Address getAddress()  {
		return _address;
	}
		
	/**
	* Get intended receiver
	* @return Intended receiver
	*/	
	public String getReceiver()   {
		return _receiver;
	}		
	
	/**
	* Get reservation time as string
	* @return Time stamp
	*/
	public String getTimeString()  {
		return _time;
	}
	
	/**
	* Get reservation time as long
	*/
	public long getTime()  {
		return (Long.valueOf(_time)).longValue();
	}	
	
	public void setTime(long time)  {
		_time = Long.toString(time);
	}
		
		
	public int getTrialTimes()  {
		return _tryTimes;
	}
			
	/**
	* Return true of the trial times is greater than the maximum trial times
	* @return true if the trial times is greater than the maximum trial times
	*/	
	public boolean overMaxTimes()   {
		if(_tryTimes > _maxTimes)  return true;
		return false;
	}							 	
}										 