package RouterLayer.util;   
   
/**
* Quick Sort algorithm, extracted from the JDK demo example
*/   
/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/
public class QSort  {

	/**
	* Default constructor
	*/
	public QSort()  {
	}
	   
   /** This is a generic version of C.A.R Hoare's Quick Sort 
    * algorithm.  This will handle arrays that are already
    * sorted, and arrays with duplicate keys.<BR>
    *
    * If you think of a one dimensional array as going from
    * the lowest index on the left to the highest index on the right
    * then the parameters to this function are lowest index or
    * left and highest index or right.  The first time you call
    * this function it will be with the parameters 0, a.length - 1.
    *
    * @param a       an integer array
    * @param lo0     left boundary of array partition
    * @param hi0     right boundary of array partition
    */
   public void sort(int a[], int lo0, int hi0)  {
      int lo = lo0;
      int hi = hi0;
      int mid;

      if ( hi0 > lo0)   {

         /* Arbitrarily establishing partition element as the midpoint of
          * the array.
          */
         mid = a[ ( lo0 + hi0 ) / 2 ];

         // loop through the array until indices cross
         while( lo <= hi )       {
            /* find the first element that is greater than or equal to 
             * the partition element starting from the left Index.
             */
	     			while( ( lo < hi0 ) && ( a[lo] < mid ))
		 					++lo;

            /* find an element that is smaller than or equal to 
             * the partition element starting from the right Index.
             */
	     			while( ( hi > lo0 ) && ( a[hi] > mid ))
		 					--hi;

            // if the indexes have not crossed, swap
            if( lo <= hi )   {
               swap(a, lo, hi);
               ++lo;
               --hi;
            }
         }

         /* If the right index has not reached the left side of array
          * must now sort the left partition.
          */
         if( lo0 < hi )
            sort( a, lo0, hi );

         /* If the left index has not reached the right side of array
          * must now sort the right partition.
          */
         if( lo < hi0 )
            sort( a, lo, hi0 );
      }
   }

   private void swap(int a[], int i, int j)   {
      int T;
      T = a[i]; 
      a[i] = a[j];
      a[j] = T;

   }
   
   /**
   * Sort integer array the from first element to the last element
   * @param a[] Integer array to be sorted
   * @exception Array related exception will be thrown
   */
   public void sort(int a[]) throws Exception   {
      sort(a, 0, a.length - 1);
   }

 }  