package Setup;

import java.awt.*;
import java.net.*;
import java.util.*;

import Abstract.*;

import ProtocolLayer.*;
import RouterLayer.AgentClient.Example.RCApplet.extawt.*;


/**
* Sub component of the Setup class. 
*/

/*
* Copyright (c) 1996,1997,1998 <A HREF="http://cdr.stanford.edu/">Center for Design Research</A>, Stanford University.
* All rights reserved.<p>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You can obtain the GNU General Public License at
* http://www.gnu.org/copyleft/gpl.html or by writing to the Free
* Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
* 
* This software is bound by the terms and conditions listed in the
* attached <a href="../LICENSE">LICENSE file</A>.
*  @author <A HREF="http://cdr.stanford.edu/html/people/jeon-bio.html>Heecheol Jeon</A>
*  @version Java(tm) 1.1
*/ 

public class SetupRouterPanel extends Panel  {
	//GUI variables
	private Font normFont = new Font("Dialog",Font.PLAIN,10),
				 boldFont = new Font("Dialog",Font.BOLD,10),
				 ultraFont = new Font("Times", Font.PLAIN,18),
				 ultraBoldFont = new Font("Times", Font.BOLD,18),
				 bigFont = new Font("Times",Font.PLAIN,14),
				 bigBoldFont = new Font("Times",Font.BOLD,14);
	Color bkgColor = new Color(210,210,210);

	/**
	* router name
	*/	
	TextField _rname;
	
	/**
	* router host inetaddress
	*/
	TextField _rhost;
	
	/**
	* router port
	*/
	TextField _rport;
		
	/**
	* registrar name
	*/	
	TextField _regname;
	
	
	/**
	* registrar port
	*/
	TextField _regport;
	
	/**
	* Maximum idle time for the received connections
	*/
	TextField _durationTime;

	SetupFrame _frame;
	/**
	* Default constructor
	*/
	public SetupRouterPanel(SetupFrame f)  	{
		setLayout(new BorderLayout());
		_frame = f;
		add("Center", GUIComponents());
	}
	
		
	/**
	* GUI component panel construction
	* @return GUI components panel
	*/	
	protected Panel GUIComponents()  {
		Panel wholePanel = newPackerPanel();
		wholePanel.setBackground(bkgColor);
		
		Panel rNamePanel = newPackerPanel();
		Label rname = new Label("Router Name:", Label.RIGHT);
		_rname = new TextField("Router",20);
		rNamePanel.add("label;side=right;fill=both", _rname);
		rNamePanel.add("label;side=right;padx=5;fill=both", rname);
		wholePanel.add("Panel;side=top;pady=3;expand=true;fill=both", rNamePanel);
		
		Panel rHostPanel = newPackerPanel();
		Label rhost = new Label("Host",Label.RIGHT);
		
		String host = "";
		try  {
			host = InetAddress.getLocalHost().getHostName();
		} catch (UnknownHostException e)  {
			SetupMessageBox b = new SetupMessageBox("Host Unknown. Check your machine is connected to network and Run again",_frame);
			System.exit(0);
		}		
		
		_rhost = new TextField(host,20);
		rHostPanel.add("label;side=right;fill=both", _rhost);        
		rHostPanel.add("label;side=right;padx=5;fill=both", rhost);
		wholePanel.add("Panel;side=top;pady=3;expand=true;fill=both", rHostPanel);
		
		Panel rPortPanel = newPackerPanel();
		Label rport = new Label("Router Port:",Label.RIGHT);
		_rport = new TextField("4444",20);
		rPortPanel.add("label;side=right;fill=both", _rport);        
		rPortPanel.add("label;side=right;padx=5;fill=both", rport);
		wholePanel.add("Panel;side=top;pady=3;expand=true;fill=both", rPortPanel);
		
		Panel regNamePanel = newPackerPanel();
		Label regname = new Label("Registrar Name:", Label.RIGHT);
		_regname = new TextField("RouterRegistrar",20);
		regNamePanel.add("label;side=right;fill=both", _regname);
		regNamePanel.add("label;side=right;padx=5;fill=both", regname);
		wholePanel.add("Panel;side=top;pady=3;expand=true;fill=both", regNamePanel);
		
		Panel regPortPanel = newPackerPanel();
		Label regport = new Label("Port:",Label.RIGHT);
		_regport = new TextField("4445",20);
		regPortPanel.add("label;side=right;fill=both", _regport);        
		regPortPanel.add("label;side=right;padx=5;fill=both", regport);
		wholePanel.add("Panel;side=top;pady=3;expand=true;fill=both", regPortPanel);
		
		Panel idlePanel = newPackerPanel();
		Label idle = new Label("Max. Idle Time:",Label.RIGHT);
		_durationTime = new TextField("1000",20);
		idlePanel.add("label;side=right;fill=both", _durationTime);        
		idlePanel.add("label;side=right;padx=5;fill=both", idle);
		wholePanel.add("Panel;side=top;pady=3;expand=true;fill=both", idlePanel);
		
		
		return wholePanel;
	}
	
	/**
	* Get router address
	* @return Router address
	*/			
	public Address getRouterAddress()  {
		String rname = _rname.getText();
		String rhost = _rhost.getText();
		String rport = _rport.getText();
		
		Address addr = new Address(rname,rhost,(Integer.valueOf(rport)).intValue(),"MessageRouter","(MessageRouter)");
		return addr;
	}
	
	/**
	* Get registrar address
	* @return Registrar address
	*/
	public Address getRegistrarAddress()  {
		String rname = _regname.getText();
		String rhost = _rhost.getText();
		String rport = _regport.getText();
		
		Address addr = new Address(rname,rhost,(Integer.valueOf(rport)).intValue(),"MessageRouter","(MessageRouterRegistrar)");
		return addr;
	}
	
	public int getDurationTime()  {
		String dstr = _durationTime.getText();
		if(!dstr.equals(""))  {
			return Integer.valueOf(dstr).intValue();
		}
		
		return -1;
	}
			
	public Insets insets()   	{
    	return new Insets(10, 10, 10, 10);
  	}
  	
 	private Panel newPackerPanel()	{
		Panel p = new Panel();
		p.setLayout(new RouterLayer.AgentClient.Example.RCApplet.extawt.PackerLayout());
		return p;
	}																			
}	
				