package webcad_01_0_1;

/**
H de ser melhorado e revisto

 */

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class ButtonBar
    extends Canvas
    implements MouseListener, MouseMotionListener {

  ButtonBarApplet _parent;
  Vector _buttons;
  int _insetX = 2, _insetY = 5;
  ImgButton _currentButton = null;
  ImgButton _eraseButton = null;

  Font _tipFont = new Font("Dialog", Font.PLAIN, 10);
  int _currentX = 4;
  int _currentY = 4;
  boolean _mouseEntered = false;

  // Image button coordination
  public ButtonBar(ButtonBarApplet parent, int width, int height) {
    _parent = parent;
    _buttons = new Vector();
    addMouseListener( (MouseListener)this);
    addMouseMotionListener( (MouseMotionListener)this);

    _currentX += _insetX;
    _currentY += _insetY;
    setBackground(Color.white);

    setSize(width + _insetX * 2, height + _insetY * 2);
//		_dbuffer = getGraphics().create(0,0,width,height);
  }

  public void addButton(String buttonId, Image imgd, Image imga, Image imgc,
                        Image imgi) {
    ImgButton b = new ImgButton(buttonId, imgd, imga, imgc, imgi, this);
    int imgWidth = imgd.getWidth(this);
    b.setBounds(new Rectangle(_currentX, _currentY, imgWidth, imgd.getHeight(this)));
    _currentX = _currentX + _insetX + imgWidth;
    _buttons.addElement(b);
    _currentButton = b;

    repaint();
  }

  public void addButton(String buttonId, Image[] img) {
    ImgButton b = new ImgButton(buttonId, img, this);
    int imgWidth = img[0].getWidth(this);
    b.setBounds(new Rectangle(_currentX, _currentY, imgWidth, img[0].getHeight(this)));
    _currentX = _currentX + _insetX + imgWidth;
    _buttons.addElement(b);
    _currentButton = b;

    repaint();
  }

  public void addSpace() {
    _currentX += _insetX;
  }

  public ImgButton getButtonByName(String name) {
    ImgButton retbtn = null;
    Enumeration e = _buttons.elements();
    while (e.hasMoreElements()) {
      ImgButton b = (ImgButton) e.nextElement();
      if (b.getName().equals(name)) {
        return b;
      }
    }
    return retbtn;
  }

  public void setPersistentButton(String buttonName, int mode) {
    ImgButton b = getButtonByName(buttonName);
    if (b != null) {
      b.setPersistentMode(mode);
    }
  }

  public void getButtonByPosition(Point p) {
    if (!_mouseEntered) {
      return;
    }

    Enumeration e = _buttons.elements();
    while (e.hasMoreElements()) {
      ImgButton b = (ImgButton) e.nextElement();
      if (b.contains(p)) {
        _currentButton = b;
        return;
      }
    }

    _currentButton = null;
  }

  public void mouseDragged(MouseEvent e) {
  }

  public void mouseMoved(MouseEvent e) {
    // move on the same button
    if (!inactivateButton(e))return;
    // return to default

    getButtonByPosition(e.getPoint());
    if (_currentButton != null) {
      _currentButton.setState(1);
      repaint();
    }
  }

  public void mouseReleased(MouseEvent e) {
    // move on the same button
    inactivateButton(e);
    getButtonByPosition(e.getPoint());
    if (_currentButton != null) {
      _currentButton.setState(1);
      repaint();

      // process with name and event
      _parent.processButtonReleased(_currentButton.getName(), e);
    }

  }

  public void mousePressed(MouseEvent e) {
    // move on the same button
    inactivateButton(e);
    getButtonByPosition(e.getPoint());
    if ( (_currentButton != null) && (!_currentButton.isDisabled())) {
      _currentButton.setState(2);
      repaint();

      // process with name and event
      _parent.processButtonPressed(_currentButton.getName(), e);
    }

  }

  public void mouseClicked(MouseEvent e) {
  }

  public void mouseEntered(MouseEvent e) {
    _mouseEntered = true;
    getButtonByPosition(e.getPoint());
    if (_currentButton != null) {
      _currentButton.setState(1);
      repaint();
    }
  }

  public void mouseExited(MouseEvent e) {
    _mouseEntered = false;
    // move on the same button

    if (_currentButton != null) {
      _currentButton.setState(0);
      _eraseButton = _currentButton;
    }

    repaint();
  }

  // if act on the same button, return false. Otherwise return true
  protected boolean inactivateButton(MouseEvent e) {
    if (_currentButton != null) {
      if (_currentButton.contains(e.getPoint())) {
        return false;
      }
      else {
        _eraseButton = _currentButton;
        _eraseButton.setState(0);
      }
    }

    return true;
  }

  public void paint(Graphics g) {
    int x = size().width;
    int y = size().height;

    g.setColor(Color.black);
    g.drawLine(3, 3, x - 6, 3);
    g.drawLine(0, y - 1, x, y - 1);
    g.drawLine(3, 3, 3, y - 6);
    g.drawLine(x - 1, 0, x - 1, y);

    g.setColor(Color.gray);
    g.drawLine(2, 2, x - 4, 2);
    g.drawLine(1, y - 2, x - 2, y - 2);
    g.drawLine(2, 2, 2, y - 4);
    g.drawLine(x - 2, 1, x - 2, y - 2);

    g.setColor(Color.lightGray);
    g.drawLine(0, 0, x, 0);
    g.drawLine(3, y - 4, x - 6, y - 4);
    g.drawLine(0, 0, 0, y);
    g.drawLine(x - 4, 3, x - 4, y - 6);

    g.setColor(Color.white);
    g.drawLine(1, 1, x - 2, 1);
    g.drawLine(2, y - 3, x - 4, y - 3);
    g.drawLine(1, 1, 1, y - 2);
    g.drawLine(x - 3, 2, x - 3, y - 4);

    Enumeration ee = _buttons.elements();
    while (ee.hasMoreElements()) {
      ImgButton b = (ImgButton) ee.nextElement();
      g.drawImage(b.getImage(), b.getX(), b.getY(), this);
    }
  }

  public void update(Graphics g) {

    FontMetrics _fMetrics = getFontMetrics(_tipFont);
    int height = _fMetrics.getHeight() + 2;

    if (_eraseButton != null) {
      int width = _fMetrics.stringWidth(_eraseButton.getName()) + 4;

      int startx = _eraseButton.getX() + 2 * _eraseButton.getWidth() / 3;
      int starty = _eraseButton.getY() + 3 * _eraseButton.getHeight() / 4;

      g.clearRect(startx, starty, width + 1, height + 1);

      Enumeration ee = _buttons.elements();
      while (ee.hasMoreElements()) {
        ImgButton b = (ImgButton) ee.nextElement();
        int x = b.getX();
        if ( (x <= (startx + width + 1)) && (x >= startx)) {
          g.drawImage(b.getImage(), b.getX(), b.getY(), this);
        }
      }

      g.drawImage(_eraseButton.getImage(), _eraseButton.getX(),
                  _eraseButton.getY(), this);
      _eraseButton = null;
    }

    if (_currentButton != null) {

      int x = _currentButton.getX();
      int y = _currentButton.getY();
      g.drawImage(_currentButton.getImage(), x, y, this);

      String id = _currentButton.getName();
      if ( ( (_currentButton.getState() == 1) ||
            (_currentButton.getState() == 2)) &&
          (id != null)) {

        int width = _fMetrics.stringWidth(id) + 4;

        int startx = _currentButton.getX() + 2 * _currentButton.getWidth() / 3;
        int starty = _currentButton.getY() + 3 * _currentButton.getHeight() / 4;

        Color old = g.getColor();

        g.setColor(Color.yellow);
        _currentButton.setTipRectangle(startx, starty, width, height);
        g.fillRect(startx, starty, width, height);
        g.setColor(Color.black);
        g.drawRect(startx, starty, width, height);
        g.drawString(id, startx + 2, starty + height - 4);

        g.setColor(old);
      }
    }
  }
}
