package util;

import java.util.Vector;
import features.*;
import pontos.*;

public class NormalizadorDeFeatures {
  public NormalizadorDeFeatures() {
  }

  /**
   * Normalizacao do vetor _featureOriginal
   *
   * 1 - faz a normalizacao do vetor _featureOriginal
   * 2 - inverte ele
   * 3 - faz a normalizacao novamente dele
   * 4 - inverte ele novamente
   *
   * @param _featureOriginal Vector
   * @return Vector
   *
   */
  public static Vector getFeaturesNormalizadas(Vector _featureOriginal) {
    Vector _featuresNormalizadas = new Vector();
    /*Vector tmp = normalizadorParcial(_featureOriginal);
         tmp = inversorDeFeatures(tmp);
         tmp = normalizadorParcial(tmp);
         _featuresNormalizadas = inversorDeFeatures(tmp);*/

    Vector _featureNormalizadaSpline = CopiadorDeFeatures.getFeaturesCopiado(_featureOriginal);
    _featureNormalizadaSpline = normalizadorDeSpline(_featureNormalizadaSpline);

    boolean interior = false;
    Vector tmpExterior = new Vector(), tmpInterior = new Vector();
    for (int i = 0; i < _featureNormalizadaSpline.size(); i++) {
      Feature fTmp = (Feature) _featureNormalizadaSpline.elementAt(i);
      if (fTmp.Tipo == Feature.IDSTRAIGHT || fTmp.Tipo == Feature.IDTAPERED ||
          fTmp.Tipo == Feature.IDCONVEX || fTmp.Tipo == Feature.IDCONCAVE ||
          fTmp.Tipo == Feature.IDCONTOURED) {
        if (interior == false) {
          interior = true;
        }
      }
      if (!interior)
        tmpExterior.add(_featureNormalizadaSpline.elementAt(i));
      else
        tmpInterior.add(_featureNormalizadaSpline.elementAt(i));
    }


    tmpExterior = getFeaturesNormalizadasParcial(tmpExterior);
    tmpExterior = InversorDeFeatures.getFeaturesInvertido(tmpExterior);
    tmpExterior = getFeaturesNormalizadasParcial(tmpExterior);
    tmpExterior = InversorDeFeatures.getFeaturesInvertido(tmpExterior);

    double maxDiametroInterior = 0.0;
    for (int i = 0; i < tmpInterior.size(); i++) {
      Feature fTmp = (Feature) tmpInterior.elementAt(i);
      if (fTmp.D1 > maxDiametroInterior)
        maxDiametroInterior = fTmp.D1;
      if (fTmp.D2 > maxDiametroInterior)
        maxDiametroInterior = fTmp.D2;
    }

    if (interior) {
      for (int i = 0; i < tmpInterior.size(); i++) {
        Feature fTmp = (Feature) tmpInterior.elementAt(i);
        fTmp.D1 = maxDiametroInterior - fTmp.D1;
        fTmp.D2 = maxDiametroInterior - fTmp.D2;
      }

      tmpInterior = getFeaturesNormalizadasParcial(tmpInterior);

      tmpInterior = InversorDeFeatures.getFeaturesInvertido(tmpInterior);
      tmpInterior = getFeaturesNormalizadasParcial(tmpInterior);
      tmpInterior = InversorDeFeatures.getFeaturesInvertido(tmpInterior);

      for (int i = 0; i < tmpInterior.size(); i++) {
        Feature fTmp = (Feature) tmpInterior.elementAt(i);
        fTmp.D1 = maxDiametroInterior - fTmp.D1;
        fTmp.D2 = maxDiametroInterior - fTmp.D2;
        if (fTmp.Tipo == Feature.GROOVE_COMPLEXO) { //Arrumar os diametros das features do groove complexo
          GrooveComplexo gTmp = (GrooveComplexo) fTmp;
          for (int j = 0; j < gTmp.features.size(); j++) {
            Feature fTmp2 = (Feature) gTmp.features.elementAt(j);
            fTmp2.D1 = maxDiametroInterior - fTmp2.D1;
            fTmp2.D2 = maxDiametroInterior - fTmp2.D2;
          }
        }
      }

    }

    for (int i = 0; i < tmpExterior.size(); i++)
      _featuresNormalizadas.add(tmpExterior.elementAt(i));

    for (int i = 0; i < tmpInterior.size(); i++)
      _featuresNormalizadas.add(tmpInterior.elementAt(i));

    for (int i = 0; i < _featuresNormalizadas.size(); i++) {
      Feature fTmp = (Feature) _featuresNormalizadas.elementAt(i);
      fTmp.Ordem = i + 1;
      if (fTmp.Tipo == Feature.GROOVE_COMPLEXO) { //Arrumar os diametros das features do groove complexo
        GrooveComplexo gTmp = (GrooveComplexo) fTmp;
        for (int j = 0; j < gTmp.features.size(); j++) {
          Feature fTmp2 = (Feature) gTmp.features.elementAt(j);
          fTmp2.Ordem = j + 1;
        }
      }
    }

    MyMath.imprimeVetorDeFeatures(_featuresNormalizadas, "\t\t\t\t");

    return _featuresNormalizadas;
  }

  /**
   * Substitui os mergulhos que nao sao grooves simples
   * por grooves_complexos
   *
   * @param _featureOriginal Vector
   * @return Vector
   *
   */
  private static Vector getFeaturesNormalizadasParcial(Vector _featureOriginal) {
    Vector _featuresNormalizadas = new Vector();
    int size = _featureOriginal.size();

    Feature tmp;
    Vector grooveVector = null;
    double D1 = 0.0;
    double D2 = 0.0;
    double Largura = 0.0;
    int gTmp = -1;

    GrooveComplexo grooveTmp = null;
    GrooveComplexo grooveTmp2 = null;
    for (int i = 0; i < size; i++) {
      tmp = (Feature) _featureOriginal.elementAt(i);
      if (tmp.D2 > tmp.D1) {
        //System.out.println("tmp.D2 > tmp.D1");
        if (grooveVector == null) { //o perfil ainda e crescente
          //System.out.println("grooveVector == null");
          _featuresNormalizadas.add(tmp);
        }
        else { //o perfil ja desceu
          //System.out.println("grooveVector != null");
          grooveTmp = (GrooveComplexo) grooveVector.elementAt(gTmp);

          //1403
          if (tmp.D2 > grooveTmp.D1) { //Fechou o grove atual e pode ter fechado outro groove
            //System.out.println("tmp.D2 > grooveTmp.D1");
            //muda os valores do grooveTmp (L1 e D2)
            //muda a ordem do grooveTmp
            //adiciona o tmp ao grooveTmp
            //manda agruparGrooves
            /*grooveTmp.L1 += tmp.L1;
                           grooveTmp.D2 = tmp.D2;
                           grooveTmp.features.add(tmp);
                           Vector f = agrupaGrooves(grooveTmp.features);
                           grooveTmp.features=f;
             grooveTmp.Ordem = ((Feature)f.firstElement()).Ordem;*/

            if (gTmp == 0) { //nao existem outro groove para fechar -> fecha somente o groove atual
              //System.out.println("gTmp == 0");
              //Finaliza o groove atual e adiciona ele ao vetor de features normalizadas
              grooveTmp.L1 += tmp.L1;
              grooveTmp.D2 = tmp.D2;
              grooveTmp.features.add(tmp);
              Vector agrupado = agrupaGrooves(grooveTmp.features);
              grooveTmp.features = agrupado;
              grooveTmp.Ordem = ( (Feature) agrupado.firstElement()).Ordem;

              _featuresNormalizadas.add(grooveTmp); //adiciona ao vetor de features normalizadas
              grooveVector = null; //limpa os dados sobre grooves
              grooveTmp = null;
              grooveTmp2 = null;
              gTmp = -1;
            }
            else { //pode ter fechado outro grooveComplexo
              //System.out.println("gTmp != 0");
              grooveTmp.L1 += tmp.L1;
              grooveTmp.D2 = tmp.D2;
              grooveTmp.features.add(tmp);
              /*Vector agrupado = agrupaGrooves(grooveTmp.features);
                               grooveTmp.features = agrupado;
               grooveTmp.Ordem = ((Feature) agrupado.firstElement()).Ordem;*/

              boolean a = true;
              do {
                grooveTmp2 = (GrooveComplexo) grooveVector.elementAt(gTmp - 1);
                if (grooveTmp.D2 > grooveTmp2.D1) { //finalizou o groove anterior e pode ter finalizado outro
                  //System.out.println("grooveTmp.D2 > grooveTmp2.D1");
                  //Finaliza o grooveTmp2
                  //Continua no "doWhile" se necessario
                  grooveTmp2.L1 += grooveTmp.L1;
                  grooveTmp2.D2 = grooveTmp.D2;
                  grooveTmp2.features.add(grooveTmp);
                  Vector agrupado = agrupaGrooves(grooveTmp2.features);
                  grooveTmp2.features = agrupado;
                  grooveTmp2.Ordem = ( (Feature) grooveTmp2.features.
                                      firstElement()).Ordem;

                  grooveVector.removeElementAt(gTmp); //remove grooveTmp
                  gTmp--;
                  if (gTmp == 0) {
                    //Adiciona o grooveTmp2 ao vetor de features Normalizadas
                    _featuresNormalizadas.add(grooveTmp2);
                    grooveVector = null; //limpa os dados sobre grooves
                    grooveTmp = null;
                    grooveTmp2 = null;
                    gTmp = -1;
                    a = false; //sai do "doWhile"
                  }
                  else { //continua no "doWhile"
                    grooveTmp = grooveTmp2;
                    grooveTmp2 = null;
                  }
                }
                else if (grooveTmp.D2 == grooveTmp2.D1) { //finalizou apenas o groove anterior
                  //System.out.println("grooveTmp.D2 == grooveTmp2.D1");
                  //Finaliza grooveTmp2 e sai do "doWhile"
                  grooveTmp2.L1 += grooveTmp.L1;
                  grooveTmp2.D2 = grooveTmp.D2;
                  grooveTmp2.features.add(grooveTmp);
                  Vector agrupado = agrupaGrooves(grooveTmp2.features);
                  grooveTmp2.features = agrupado;
                  grooveTmp2.Ordem = ( (Feature) grooveTmp2.features.
                                      firstElement()).Ordem;

                  grooveVector.removeElementAt(gTmp); //remove grooveTmp
                  gTmp--; //atualiza gTmp
                  if (gTmp == 0) { //vetor de groovesComplexos so possui um elemento (grooveTmp2)
                    //System.out.println("->gTmp == 0");
                    //Adiciona o grooveTmp2 ao vetor de features Normalizadas
                    _featuresNormalizadas.add(grooveTmp2);
                    grooveVector = null; //limpa os dados sobre grooves
                    grooveTmp = null;
                    grooveTmp2 = null;
                    gTmp = -1;
                    a = false; //sai do "doWhile"
                  }
                  else { //vetor de groovesComplexos possui mais de um elemento
                    //System.out.println("->gTmp != 0");
                    //Adiciona o grooveTmp2 ao grooveAnterior a este
                    //sai do "doWhile"
                    grooveTmp = grooveTmp2;
                    grooveTmp2 = (GrooveComplexo) grooveVector.elementAt(gTmp -
                        1);

                    grooveTmp2.L1 += grooveTmp.L1;
                    grooveTmp2.features.add(grooveTmp);
                    agrupado = agrupaGrooves(grooveTmp2.features);
                    grooveTmp2.features = agrupado;

                    grooveVector.removeElementAt(gTmp);
                    grooveTmp = null;
                    grooveTmp2 = null;
                    gTmp--;
                    a = false;
                  }
                }
                else if (grooveTmp.D2 < grooveTmp2.D1) { //nao finalizou o groove anterior
                  //System.out.println("grooveTmp.D2 < grooveTmp2.D1");
                  //Adiciona o grooveTmp ao vetor de features do grooveTmp2
                  //Deleta o grooveTmp do grooveVector
                  //atualiza dados
                  //sai do "doWhile"
                  grooveTmp2.L1 += grooveTmp.L1;
                  grooveTmp2.features.add(grooveTmp);
                  Vector agrupado = agrupaGrooves(grooveTmp2.features);
                  grooveTmp2.features = agrupado;

                  grooveVector.removeElementAt(gTmp);
                  grooveTmp = null;
                  grooveTmp2 = null;
                  gTmp--;
                  a = false; //sai do "doWhile"
                }
              }
              while (a == true);
            }

            //Checar se grooveTmp.D2 >= grooveTmp2.D1
            //Sim -> faz a mesma estratégia para o grooveTmp2:
            //Passa as features do grooveTmp para o grooveTmp2
            //Atualiza os dados do grooveTmp2 (L1 += grooveTmp.L1, D2 = grooveTmp.D2)
            //gTmp--, remove grooveTmp do grooveVector, atualiza grooveTmp e faz grooveTmp2 = null
            //se gTmp == 0 sai do "for"
            //Nao -> finaliza o grooveTmp (gTmp--, adicionar ao groove anterior e remove-lo da lista de groovesComplexos)
            //sai do "for"

          }
          else if (tmp.D2 == grooveTmp.D1) { //Fechou apenar o groove atual
            //System.out.println("tmp.D2 == grooveTmp.D1");
            //Finaliza o grooveComplexo atual
            //calculus.alert("Fechou por igual" + grooveTmp.L1);
            grooveTmp.L1 += tmp.L1;
            //calculus.alert("Fechou por igual" + grooveTmp.L1);
            grooveTmp.D2 = tmp.D2;
            grooveTmp.features.add(tmp);
            Vector f = agrupaGrooves(grooveTmp.features);
            grooveTmp.features = f;
            grooveTmp.Ordem = ( (Feature) f.firstElement()).Ordem;

            if (gTmp == 0) { //adiciona o grooveComplexo ao vetor das features normalizadas
              gTmp--;
              _featuresNormalizadas.add(grooveTmp);
              grooveVector = null;
            }
            else { //adiciona o grooveComplexo ao vetor de features do groove anterior
              gTmp--;
              ( (GrooveComplexo) grooveVector.elementAt(gTmp)).features.add(
                  grooveTmp);
              ( (GrooveComplexo) grooveVector.elementAt(gTmp)).L1 += grooveTmp.
                  L1;
              grooveVector.removeElementAt(gTmp + 1);
              /**
               * @todo remover o elemento deve causar problema para o
               * vetor de features do groove anterior, verificar depois!!!
               */
            }
          }
          else if (tmp.D2 < grooveTmp.D1) { //Ainda naum fechou o groove
            //System.out.println("tmp.D2 < grooveTmp.D1");
            grooveTmp.L1 += tmp.L1;
            grooveTmp.features.add(tmp);
            grooveVector.setElementAt(grooveTmp, gTmp);
            /*@todo ver se isso funciona*/
          }
          //1403
        }
      }
      else if (tmp.D2 == tmp.D1) {
        if (i != size - 1) {
          //System.out.println("tmp.D2 == tmp.D1");
          if (grooveVector == null) { //o perfil ainda e crescente
            _featuresNormalizadas.add(tmp);
          }
          else { //o perfil e constante dentro de um groove
            grooveTmp = (GrooveComplexo) grooveVector.elementAt(gTmp);
            grooveTmp.L1 += tmp.L1;
            grooveTmp.features.add(tmp);
            grooveVector.setElementAt(grooveTmp, gTmp);
            /*@todo ver se isso funciona*/
          }
        }
        else {
          if (grooveVector == null) _featuresNormalizadas.add(tmp);
          else { //deixa os grooves que ele criou em aberto e vai adicionando seus elementos aos grooves anteriores e por ultimo ao featuresNormalizadas
            int sizeJ = grooveVector.size();

            //## Insere no ultimo groove a ultima feature
            grooveTmp = (GrooveComplexo) grooveVector.lastElement();
            grooveTmp.features.add(tmp);
            grooveVector.setElementAt(grooveTmp, sizeJ - 1);

            for (int j = sizeJ - 1; j >= 0; j--) {
              grooveTmp = (GrooveComplexo) grooveVector.elementAt(j);
              if (j != 0) { //esta trabalhando com um groove qualquer -> adiciona seus dados ao groove anterior (indice menor)
                grooveTmp2 = (GrooveComplexo) grooveVector.elementAt(j - 1);
                int sizeK = grooveTmp.features.size();
                for (int k = 0; k < sizeK; k++) {
                  grooveTmp2.features.add(grooveTmp.features.elementAt(k));
                }
                grooveVector.setElementAt(grooveTmp2, j - 1);
                grooveVector.removeElementAt(j);
              }
              else { //esta trabalhando com o groove de indice Zero -> adiciona seus dados no featuresNormalizadas
                int sizeK = grooveTmp.features.size();
                for (int k = 0; k < sizeK; k++) {
                  _featuresNormalizadas.add(grooveTmp.features.elementAt(k));
                }
              }
            }
          }
        }
      }
      else if (tmp.D2 < tmp.D1) {
        //System.out.println("tmp.D2 < tmp.D1 - DESCEU");
        if (i != size - 1) { //se nao for a ultima feature
          if (grooveVector == null) grooveVector = new Vector(); //caso seja a primeira descida

          //cria um novo grooveComplexo com os dados da feature atual
          grooveTmp = new GrooveComplexo();
          grooveTmp.features.add(tmp);
          grooveTmp.D1 = tmp.D1;
          grooveTmp.D2 = tmp.D2;
          grooveTmp.L1 = tmp.L1;
//00
          //        grooveTmp.Ordem = tmp.Ordem;
//00
          //adiciona o novo grooveComplexo ao vetor de grooves
          grooveVector.add(grooveTmp);
          //atualiza o indice do groove atual
          gTmp++;
        }
        else {
          if (grooveVector == null) _featuresNormalizadas.add(tmp);
          else { //deixa os grooves que ele criou em aberto e vai adicionando seus elementos aos grooves anteriores e por ultimo ao featuresNormalizadas
            int sizeJ = grooveVector.size();

            //## Insere no ultimo groove a ultima feature
            grooveTmp = (GrooveComplexo) grooveVector.lastElement();
            grooveTmp.features.add(tmp);
            grooveVector.setElementAt(grooveTmp, sizeJ - 1);

            for (int j = sizeJ - 1; j >= 0; j--) {
              grooveTmp = (GrooveComplexo) grooveVector.elementAt(j);
              if (j != 0) { //esta trabalhando com um groove qualquer -> adiciona seus dados ao groove anterior (indice menor)
                grooveTmp2 = (GrooveComplexo) grooveVector.elementAt(j - 1);
                int sizeK = grooveTmp.features.size();
                for (int k = 0; k < sizeK; k++) {
                  grooveTmp2.features.add(grooveTmp.features.elementAt(k));
                }
                grooveVector.setElementAt(grooveTmp2, j - 1);
                grooveVector.removeElementAt(j);
              }
              else { //esta trabalhando com o groove de indice Zero -> adiciona seus dados no featuresNormalizadas
                int sizeK = grooveTmp.features.size();
                for (int k = 0; k < sizeK; k++) {
                  _featuresNormalizadas.add(grooveTmp.features.elementAt(k));
                }
              }
            }
          }
        }
      }

      /*System.out.println("Depois de i= " + i);
               System.out.println("_featuresNormalizadas");
               calculus.imprimeTamanho(_featuresNormalizadas);

               if(grooveVector != null){
        int sizeX = grooveVector.size();
        calculus.alert("" + sizeX);
        for(int x = 0; x < sizeX; x++){
          System.out.println("Groove numero: " + x);
       calculus.imprimeTamanho(((GrooveComplexo)grooveVector.elementAt(x)).features);
        }
               }
               System.out.println("");*/

    }

    return _featuresNormalizadas;
  }

  /**
   * agrupaGrooves Serve para agrupar as features de um groove interno dentro de
   * um mais externo
   *
   * @param entrada Vector
   * @return Vector
   */
  private static Vector agrupaGrooves(Vector entrada) {
    int size = entrada.size();
    Vector saida = new Vector(size);
    for (int i = 0; i < size; i++) {
      Feature tmp = (Feature) entrada.elementAt(i);
      if (tmp.Tipo == Feature.GROOVE_COMPLEXO) {
        Vector tmpJ = ( (GrooveComplexo) tmp).features;
        int sizeJ = tmpJ.size();
        for (int j = 0; j < sizeJ; j++) {
          saida.add(tmpJ.elementAt(j));
        }
      }
      else {
        saida.add(tmp);
      }
    }

    return MyMath.ordenaVectorDeFeatures(saida);
  }

  private static Vector normalizadorDeSpline(Vector entrada){
    Feature fTmp;
    Vector saida = entrada;
    Vector pontos;
    double DMax, DMin;
    Feature fNull_1, fNull_2;

    int size = entrada.size();
    for(int i = 0; i < size; i++){
      fTmp = (Feature)entrada.elementAt(i);
      if(fTmp.Tipo == Feature.ODCONTOURED){
        pontos = getPontosSpline(fTmp);
        int pSize = pontos.size();
        for(int j = 0; j < pSize; j++){
          Ponto2D p = (Ponto2D)pontos.elementAt(j);
          System.out.println("FINAL: z: " + p.z + " x: " + p.x);
        }
        DMax = getDiametroMax(pontos);

        fNull_2 = new Feature();
        fNull_2.D1 = DMax;
        fNull_2.D2 = fTmp.D2;
        saida.insertElementAt(fNull_2, i+1);

        System.out.println("Maior Diametro: " + DMax);
        ODSplineVirtual tmp = new ODSplineVirtual((ODContoured)fTmp, pontos, DMax);
        saida.insertElementAt(tmp, i+1);

        fNull_1 = new Feature();
        fNull_1.D1 = fTmp.D1;
        fNull_1.D2 = DMax;
        saida.insertElementAt(fNull_1, i+1);

        saida.removeElementAt(i);
      }
      if(fTmp.Tipo == Feature.IDCONTOURED){
        pontos = getPontosSpline(fTmp);
        DMin = getDiametroMin(pontos);

        fNull_2 = new Feature();
        fNull_2.D1 = DMin;
        fNull_2.D2 = fTmp.D2;
        saida.insertElementAt(fNull_2, i+1);

        System.out.println("Menor Diametro: " + DMin);
        IDSplineVirtual tmp = new IDSplineVirtual((IDContoured)fTmp, pontos, DMin);
        saida.insertElementAt(tmp, i+1);

        fNull_1 = new Feature();
        fNull_1.D1 = fTmp.D1;
        fNull_1.D2 = DMin;
        saida.insertElementAt(fNull_1, i+1);

        saida.removeElementAt(i);
      }

    }

    size = saida.size();
    for(int i = 0; i < size; i++){
      fTmp = (Feature)saida.elementAt(i);
      fTmp.Ordem = i + 1;
    }

    return saida;
  }

  private static Vector getPontosSpline(Feature fTmp){
    Vector pontos = null;
    int precisao = 10;
    Vector pontosIncrementais, pontosTmp;
    Ponto2D inicioTmp, fimTmp, ultimoControle;

    if(fTmp.Tipo == Feature.ODCONTOURED){
      System.out.println("ODCONTOURED");
      ODContoured odcTmp = (ODContoured)fTmp;

      ControlPoint1Spline t = (ControlPoint1Spline)odcTmp.pontos.lastElement();
      ultimoControle = new Ponto2D(-1*t.y, t.x);

      for(int i = 0; i < odcTmp.pontos.size(); i++){
        ControlPoint1Spline tmp = (ControlPoint1Spline)odcTmp.pontos.elementAt(i);
        System.out.println("Ponto de Controle -> z=" + tmp.x + " x=" + tmp.y);
      }

      pontosTmp = MyMath.CalculaODContouredCSpline(odcTmp.pontos, precisao);
      pontosTmp.add(ultimoControle);
      for(int j = 0; j < pontosTmp.size(); j++){
        Ponto2D pTmp = (Ponto2D)pontosTmp.elementAt(j);
        pTmp.x = 2*pTmp.x;
        System.out.println("Z: " + pTmp.z + " X: " + pTmp.x);
      }

      pontosIncrementais = new Vector();
      inicioTmp = (Ponto2D)pontosTmp.elementAt(0);
      for(int j = 0; j < pontosTmp.size(); j++){
        fimTmp = (Ponto2D)pontosTmp.elementAt(j);

        Ponto2D novo = new Ponto2D();
        novo.x = fimTmp.x;
        novo.z = fimTmp.z - inicioTmp.z;
        System.out.println("\tZ: " + novo.z + " X: " + novo.x);

        pontosIncrementais.add(novo);

        inicioTmp = fimTmp;
      }

      pontos = pontosIncrementais;
    }
    if(fTmp.Tipo == Feature.IDCONTOURED){
      pontos = new Vector();
      IDContoured idcTmp = (IDContoured)fTmp;

      ControlPoint1Spline t = (ControlPoint1Spline)idcTmp.pontos.lastElement();
      ultimoControle = new Ponto2D(-1*t.y, t.x);

      pontosTmp = MyMath.CalculaODContouredCSpline(idcTmp.pontos, precisao);
      for(int j = 0; j < pontosTmp.size(); j++){
        Ponto2D pTmp = (Ponto2D)pontosTmp.elementAt(j);
        pTmp.x = 2*pTmp.x;
        System.out.println("Z: " + pTmp.z + " X: " + pTmp.x);
      }

      pontosIncrementais = new Vector();
      inicioTmp = (Ponto2D)pontosTmp.elementAt(0);
      for(int j = 0; j < pontosTmp.size(); j++){
        fimTmp = (Ponto2D)pontosTmp.elementAt(j);

        Ponto2D novo = new Ponto2D();
        novo.x = fimTmp.x;
        novo.z = fimTmp.z - inicioTmp.z;
        System.out.println("\tZ: " + novo.z + " X: " + novo.x);

        pontosIncrementais.add(novo);

        inicioTmp = fimTmp;
      }

      pontosIncrementais.add(ultimoControle);

      pontos = pontosIncrementais;
    }

    return pontos;
  }

  private static double getDiametroMax(Vector pontos){
    double diametro = 0;
    Ponto2D tmp;

    int size = pontos.size();
    for(int i = 0; i < size; i++){
      tmp = (Ponto2D)pontos.elementAt(i);
      if(tmp.x > diametro)
        diametro = tmp.x;
    }

    return diametro;
  }

  private static double getDiametroMin(Vector pontos){
    double diametro = 99999;
    Ponto2D tmp;

    int size = pontos.size();
    for(int i = 0; i < size; i++){
      tmp = (Ponto2D)pontos.elementAt(i);
      if(tmp.x < diametro)
        diametro = tmp.x;
    }

    return diametro;
  }

}
