package webcapp_01_0_1;

import java.util.Vector;
import features.*;
import pontos.Ponto2D;
import util.*;
import featureseixoc.*;

/**
 *
 * <p>Title: CAPP</p>
 * <p>Description: CAPP para o projeto Webmachining</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: GrACo</p>
 * @author not attributable
 * @version 1.0
 */
public class DecomposicaoOrientadaAGeometria {
  public InfoFeature infoFeature;
  public InfoDecomposicaoOrientadaASetup infoDOS;
  public InfoDecomposicaoOrientadaAGeometria infoDOG;

  public Blank blankNecessario, blankUtilizado;
  public Vector fuDoSetupN;

  public Blank blankSetup1 = null, blankSetup2 = null;
  public boolean interiorSetup1 = false, interiorSetup2 = false;
  public boolean furoSetup1 = false, furoSetup2 = false;


  /**
   *
   * @param infoFeature InfoFeature
   * @param infoDOS InfoDecomposicaoOrientadaASetup
   */
  public DecomposicaoOrientadaAGeometria(InfoFeature infoFeature,
                                         InfoDecomposicaoOrientadaASetup
                                         infoDOS) {
    this.infoFeature = infoFeature;
    this.infoDOS = infoDOS;

    System.out.println("===>Iniciando Decomposicao Orientada A Geometria<===");

    //Determinar os blanks
    this.blankNecessario = getBlankNecessario(this.infoFeature._feature);
    this.blankUtilizado = getBlankUtilizado(this.blankNecessario);

    //Analisando a Peca
    analiseInicial();

    //Determinar as features de Usinagem
    this.fuDoSetupN = getFeaturesDeUsinagem();

    //Determinando o info decomposicao orientada a geometria
    this.infoDOG = getInfoDecomposicaoOrientadaAGeometria();

    //Imprimindo
    imprime();

    System.out.println("===>Finalizando Decomposicao Orientada A Geometria<===");
  }

  /**
   * Determina o minimo blank necessario para que a peca possa ser feita
   * Da Famlia de Tubos (Tubo vazado e blank macio)
   * @param vector Vector
   * @return Blank
   */
  public Blank getBlankNecessario(Vector _feature) {
    Blank blankNecessarioTmp = new Blank();

    double L1 = 0.0, D1 = 0.0, D2 = 0.0, sobreMaterial = 0.0;

    int size = _feature.size();
    Feature tmp;
    for (int i = 0; i < size; i++) {
      tmp = (Feature) _feature.elementAt(i);

      if (i == 0)
        L1 += getSobreMaterialTotal(tmp);

      if (this.infoFeature.temInterior) {
        if (i < infoFeature.indiceFeaturesInternas) {
          L1 += tmp.L1;
          if (i == this.infoFeature.indiceFeaturesInternas - 1)
            L1 += getSobreMaterialTotal(tmp);
        }
        else {
          if (i == this.infoFeature.indiceFeaturesInternas)
            D1 = D2;
          double D1Tmp = tmp.D2 - getSobreMaterialTotal(tmp);
          if (D1Tmp < D1)
            D1 = D1Tmp;
        }
      }
      else {
        L1 += tmp.L1;
        if (i == size - 1)
          L1 += getSobreMaterialTotal(tmp);
      }

      double D2Tmp = tmp.D2 + getSobreMaterialTotal(tmp);
      if (D2Tmp > D2)
        D2 = D2Tmp;

        /*if(tmp.Ordem < infoFeature.indiceFeaturesInternas){
          L1 += tmp.L1;

          double D2Tmp = tmp.D2 + getSobreMaterialTotal(tmp);
          if (D2Tmp > D2)
            D2 = D2Tmp;
               }
         if(infoFeature.temInterior && tmp.Ordem >= infoFeature.indiceFeaturesInternas){
          if(i == this.infoFeature.indiceFeaturesInternas)
            D1 = D2;
          double D1Tmp = tmp.D2 - getSobreMaterialTotal(tmp);
          if(D1Tmp < D1)
            D1 = D1Tmp;
               }*/
    }

    if (D1 < 0)
      D1 = 0;

    blankNecessarioTmp.L1 = L1;
    blankNecessarioTmp.D1 = D1;
    blankNecessarioTmp.D2 = D2;

    return blankNecessarioTmp;
  }

  /**
   * Determina o blank que sera utilizado na fabricacao da peca
   * @todo Tem que implementar a procura no banco de dados
   *
   * @param blankNecessario Blank
   * @return Blank
   */
  public Blank getBlankUtilizado(Blank blankNecessario) {
    Blank blankUtilizadoTmp = blankNecessario;

    return blankUtilizadoTmp;
  }





  /**
   * Analisa a presenca de furos, de features internas e determina o tipo de
   * peca que esta sendo tratado.
   */
  public void analiseInicial() {
    //Determina se os setups tem parte interna.
    if (this.infoFeature.temInterior) { //Peca tem interior
      /**
       * Precisa verificar se o primeiro setup tem parte interna. Para
       * isso basta verificar se a ultima feature do vetor _feature tem
       * D2 != 0
       */
      if ( ( (Feature)this.infoFeature._feature.lastElement()).D2 != 0) { //existe parte interna no primeiro setup
        this.interiorSetup1 = true;
      }
      /**
       * Verifica existe parte interna no segundo setup. Para isso deve-se
       * ver se o diametro 1 (D1) da primeira feature interna e diferente
       * de zero.
       */
      if ( ( (Feature)this.infoFeature._feature.elementAt(this.infoFeature.
          indiceFeaturesInternas)).D1 != 0) { //setup dois tem parte interna
        this.interiorSetup2 = true;
      }
    }

    /**
     * A determinacao da nacessidade de furos deve considerar o "blankSolicitado",
     * caso ele seja um "prefuro".
     * Deve avaliar tambem a possibilidade da broca nao conseguir fazer o furo
     * completo de uma peca. Nesse caso devera ser criado furo nos dois setups
     * e verificar se alguma das decisoes afeta ou e afetada pelo posicinamento
     * da superficie delimitadora interna.
     *
     * Por enquanto a funcao considera o blankNecessario como sendo o blank
     * escolhido pelo usuario durante a modelagem.
     */

    //Verificando a necessidade de furacao.
    if (this.blankUtilizado.D1 == 0) { //blank macico, podendo ou nao existir parte interna
      if (this.blankNecessario.D1 != 0) { //furo na peca inteira
        /**
         * Nessa parte deve ser feita a verificacao sobre tamanho do furo possivel
         * com uma broca escolhida e o tamanho da peca.
         * Se a broca pudesse fazer o furo na peca inteira entao o blankSetup1 e
         * o blankSetup2 serao iguais, cujo valor de D1 sera igual ao valor do
         * diametro da broca escolhida e tb sem valores de comprimento, pois nao
         * sera necessario, ja que so sera feito um furo com o comprimento da
         * peca inteira.
         *
         * Caso a broca nao possa ser utilizada para fazer o furo inteiro serao
         * criados dois outros blanks "blankSetup1" e "blankSetup2", com o
         * comprimento do furo que devera ser realizado e o diametro da broca
         * escolhida.
         *
         */
        this.furoSetup1 = true;
        this.blankSetup1 = new Blank();
        this.blankSetup1.D1 = this.blankNecessario.D1;//nesse local sera colocado o diamentro da broca
        this.blankSetup1.D2 = this.blankUtilizado.D2;
        this.blankSetup2 = this.blankSetup1;
      }
      else { //sem furo ou com um furo em um dos lados
        /**
         * Necessario verificar se existe parte interna em algum dos setups para
         * saber se furos sao necessarios.
         */
        if (this.interiorSetup1) { //furo no setup 1
          this.furoSetup1 = true;
          /**
           * Encontrar o menor diametro diferente de zero.
           */
          //Nesse caso existe obrigatoriamente uma feature IDStraight com diametro igual a zero
          int inicio = 0;
          int fim = this.infoFeature._feature.size() -
                this.infoDOS.infoSuperficieDelimitadoraInterna.indiceDaFeature;

          double diametroTmp = ((Feature)this.infoFeature._featureInvertido.elementAt(inicio)).D1;
          double comprimentoTmp = 0.0;
          for(int i = inicio; i <= fim; i++){
            Feature fTmp = (Feature)this.infoFeature._featureInvertido.elementAt(i);
            if((fTmp.D1 - getSobreMaterialTotal(fTmp)) < diametroTmp && fTmp.D1 != 0) diametroTmp = fTmp.D1 - getSobreMaterialTotal(fTmp);
            if((fTmp.D2 - getSobreMaterialTotal(fTmp)) < diametroTmp && fTmp.D2 != 0) diametroTmp = fTmp.D2 - getSobreMaterialTotal(fTmp);
            if(fTmp.D2 == 0){
              /**
               * Caso a ultima feature de um furo seja um groove complexo deve-se
               * fazer outro tratamento, adicionando o comprimento dele ate sua
               * ultima feature e removendo o sobrematerial da ultima feature
               * do groove complexo.
               */
              if(fTmp.Tipo == Feature.GROOVE_COMPLEXO){
                GrooveComplexo gTmp = (GrooveComplexo)fTmp;
                int size = gTmp.features.size();
                for(int j = 0; j < size; j++){
                  Feature fTmp2 = (Feature)gTmp.features.elementAt(j);
                  if(j < size - 1)
                    comprimentoTmp += fTmp2.L1;
                  else
                    comprimentoTmp -= getSobreMaterialTotal(fTmp2);
                }
              }
              else
                comprimentoTmp -= getSobreMaterialTotal(fTmp);
              break;
            }
            comprimentoTmp += fTmp.L1;
          }

          //Cria um blank para o setup 1 com os dados calculados acima
          this.blankSetup1 = new Blank();
          this.blankSetup1.L1 = comprimentoTmp;
          this.blankSetup1.D1 = diametroTmp;
          this.blankSetup1.D2 = this.blankUtilizado.D2;

          //Aponta o blankSetup2 para o blankUtilizado.
          this.blankSetup2 = this.blankUtilizado;
        }
        else if (this.interiorSetup2) { //furo no setup 2
          this.furoSetup2 = true;
          /**
           * Encontra o menor diametro diferente de zero.
           */
          int inicio = this.infoFeature.indiceFeaturesInternas;
          int fim;
          if(this.interiorSetup1){//caso o primeiro setup tenha interior
            fim = this.infoDOS.infoSuperficieDelimitadoraInterna.
                  indiceDaFeature;
          }
          else{//caso o primeiro setup tenha interior
            fim = this.infoDOS.infoSuperficieDelimitadoraInterna.
                  indiceDaFeature - 1;
          }

          double diametroTmp = ((Feature)this.infoFeature._feature.elementAt(inicio)).D1;
          double comprimentoTmp = 0.0;
          System.out.println(inicio);
          for(int i = inicio; i <= fim; i++){//VERIFICAR O <=
            Feature fTmp = (Feature)this.infoFeature._feature.elementAt(i);
            if((fTmp.D1 - getSobreMaterialTotal(fTmp)) < diametroTmp && fTmp.D1 != 0) diametroTmp = fTmp.D1 - getSobreMaterialTotal(fTmp);
            if((fTmp.D2 - getSobreMaterialTotal(fTmp)) < diametroTmp && fTmp.D2 != 0) diametroTmp = fTmp.D2 - getSobreMaterialTotal(fTmp);
            if(fTmp.D2 == 0){
              comprimentoTmp -= getSobreMaterialTotal(fTmp);
              break;
            }
           comprimentoTmp += fTmp.L1;
          }

          //Cria um blank para o setup 2 com os dados calculados acima
          this.blankSetup2 = new Blank();
          this.blankSetup2.L1 = comprimentoTmp;
          this.blankSetup2.D1 = diametroTmp;
          this.blankSetup2.D2 = this.blankUtilizado.D2;

          //Aponta o blankSetup1 para o blankUtilizado, caso ele ainda nao tenha sido definido
          if(this.blankSetup1 == null)
            this.blankSetup1 = this.blankUtilizado;
        }
        else{//Nao existe interior
          this.blankSetup1 = this.blankUtilizado;
          this.blankSetup2 = this.blankUtilizado;
        }
      }
    }
    else { //blank vazado
      /**
       * Nesse caso naum e necessario fazer nada. So precisa fazer o blank de
       * cada um dos setups apontar para o blankUtilizado.
       */
      System.out.println("BLANK VAZADO");
      this.blankSetup1 = this.blankUtilizado;
      this.blankSetup2 = this.blankUtilizado;
      if(this.blankSetup1.equals(this.blankSetup2)){
        System.out.println("Blanks iguais");
      }
    }
  }



  /**
   * Determina o sobre material total para a feature de entrada
   * @todo Criar o metodo para encontrar o sobre material total
   *
   * @param f Feature
   * @return double
   */
  public double getSobreMaterialTotal(Feature f) {
    return 2;
  }

  /**
   * Cria o vetor de vetores de features de usinagem
   * @todo Tem que arrumar esta funcao e tambem arrumar a origem das features de
   * usinagem em todos os metodos, por causa das features de usinagem internas
   *
   * @param infoFeature InfoFeature
   * @return Vector
   */
  public Vector getFeaturesDeUsinagem() {
    Vector fuDoSetupNTmp = new Vector();

    Vector fuTmp;

    for (int i = 0; i < 2; i++) {
      fuTmp = new Vector();
      if (i == 0) { //primeiro setup
        System.out.println(
            "===>Encontrando Features de Usinagem do Primeiro Setup<===");

        //Parte Externa

        //Encontrando a Fu de Furo
        if(this.furoSetup1){//deve criar um FURO no primeiro setup
          if(!this.furoSetup2){
            if(this.blankSetup1 == this.blankSetup2){//Faz um furo na peca inteira
              double comprimento = 0.0;
              for(int j = 0; j < this.infoFeature.indiceFeaturesInternas; j++){
                Feature fTmp = (Feature)this.infoFeature._feature.elementAt(i);
                comprimento += fTmp.L1;
              }

              fuTmp.add(getFuFuracao(this.blankSetup1.D1, comprimento));
            }
            else{//Furo somente na parte do primeiro setup
              fuTmp.add(getFuFuracao(this.blankSetup1.D1, this.blankSetup1.L1));
            }
          }
          else{//Furo somente na parte do primeiro setup
            fuTmp.add(getFuFuracao(this.blankSetup1.D1, this.blankSetup1.L1));
          }
        }


        fuTmp.add(getFuFaceamento( (Feature)this.infoFeature._feature.
                                  firstElement(), this.blankSetup1));
        fuTmp.add(getFuBloco(this.infoFeature._feature, 0, 1,
                             this.infoDOS.infoSuperficieDelimitadoraExterna.
                             indiceDaFeature, this.blankSetup1));

        Vector fuEspeciaisExternas = getFuEspeciais(this.infoFeature._feature,
            0,
            this.infoDOS.
            infoSuperficieDelimitadoraExterna.
            indiceDaFeature);
        if (fuEspeciaisExternas != null) {
          int sizeExt = fuEspeciaisExternas.size();
          for (int j = 0; j < sizeExt; j++) {
            fuTmp.add(fuEspeciaisExternas.elementAt(j));
          }
        }

        //Features de usinagem do eixo C
        Vector fuEixoC = getFuEixoC(this.infoFeature._feature,
                                    this.infoFeature._featureEixoC, 0,
                                    this.infoDOS.
                                    infoSuperficieDelimitadoraExterna.
                                    indiceDaFeature);
        if (fuEixoC != null) {
          int sizeEixoC = fuEixoC.size();
          for(int j = 0; j < sizeEixoC; j++){
            fuTmp.add(fuEixoC.elementAt(j));
          }
        }

        //Parte Interna
        if (this.interiorSetup1) { //verifica se tem interior no primeiro setup
          Vector interiorTmp = new Vector();

          /**
           * Para calcular o final e necessario ver se os setups tem blanks diferentes,
           * pois caso isso seja verdade a ultima feature do perfil interno devera ser
           * descartaja, uma vez que ela ja foi resolvida pelo furo.
           *
           * != -> retira a ultima feature
           * == -> faz como deveria ser feito
           */
          int fim;
          if(this.blankSetup1 != this.blankSetup2)
            fim = this.infoFeature._feature.size() -
              this.infoDOS.infoSuperficieDelimitadoraInterna.indiceDaFeature - 1;
          else
            fim = this.infoFeature._feature.size() -
              this.infoDOS.infoSuperficieDelimitadoraInterna.indiceDaFeature;

          for (int j = 0; j < fim; j++) { //so serve para encontar o maior diametro interno
            interiorTmp.add(this.infoFeature._featureInvertido.elementAt(j));
          }

          double maxDiametroInterno = EspelhoDeFeatures.encontrarMaiorDiametroInterno(
              interiorTmp);
          interiorTmp = EspelhoDeFeatures.espelharVerticalmenteVetor(interiorTmp,
              maxDiametroInterno);

          this.blankSetup1 = EspelhoDeFeatures.espelharBlank(this.blankSetup1, maxDiametroInterno);
          double D1Tmp = this.blankSetup1.D1;
          this.blankSetup1.D1 = this.blankSetup1.D2;
          this.blankSetup1.D2 = D1Tmp;

          //Encontra as features de usinagem "normais"
          Vector fuInternasTmp = new Vector();
          fuInternasTmp.add(getFuBloco(this.infoFeature._featureInvertido, 0,
                                       0, fim, this.blankSetup1));

          //Encontra as features de usinagem especiais
          Vector fuEspeciaisInternas = getFuEspeciais(this.infoFeature.
              _featureInvertido, 0, fim);
          if (fuEspeciaisInternas != null) {
            int sizeInt = fuEspeciaisInternas.size();
            for (int j = 0; j < sizeInt; j++) {
              fuInternasTmp.add(fuEspeciaisInternas.elementAt(j));
            }
          }

          interiorTmp = EspelhoDeFeatures.espelharVerticalmenteVetor(interiorTmp,
              maxDiametroInterno);

          this.blankSetup1 = EspelhoDeFeatures.espelharBlank(this.blankSetup1,
                                                  maxDiametroInterno);
          D1Tmp = this.blankSetup1.D1;
          this.blankSetup1.D1 = this.blankSetup1.D2;
          this.blankSetup1.D2 = D1Tmp;
          /**
           * Percore as features internas buscando features nulas, que deveram
           * ser espelhadas, ja que elas naum pertemcem ao vetor original de
           * features
           */
          int sizeFuTmp = fuInternasTmp.size();
          for (int k = 0; k < sizeFuTmp; k++) {
            FeatureDeUsinagem tmpK = (FeatureDeUsinagem) fuInternasTmp.
                elementAt(k);

            int sizeL = tmpK._featuresDaFU.size();
            for (int l = 0; l < sizeL; l++) {
              Feature tmpL = (Feature) tmpK._featuresDaFU.elementAt(l);
              if (tmpL.Tipo == 0) {
                tmpL.D1 = maxDiametroInterno - tmpL.D1;
                tmpL.D2 = maxDiametroInterno - tmpL.D2;
              }
            }

            /**
             * Tem que inverter o diametro da origem
             */
            tmpK.origem.x = maxDiametroInterno - tmpK.origem.x;

            fuTmp.add(tmpK);
          }

          //Features de usinagem do eixo C
          fuEixoC = getFuEixoC(this.infoFeature._featureInvertido,
                               this.infoFeature._featureEixoCInvertido,
                               0, fim);
          if (fuEixoC != null) {
            int sizeEixoC = fuEixoC.size();
            for (int j = 0; j < sizeEixoC; j++) {
              fuTmp.add(fuEixoC.elementAt(j));
            }
          }


        }
      }
      else { //segundo setup
        System.out.println(
            "===>Encontrando Features de Usinagem do Segundo Setup<===");

        if (!this.interiorSetup2) { //Nao tem Interior no segundo setup
          int inicio = 0;

          if(this.interiorSetup1){
            inicio = this.infoFeature._feature.size() - this.infoFeature.indiceFeaturesInternas;
          }

          fuTmp.add(getFuFaceamento( (Feature)this.infoFeature.
                                    _featureInvertido.elementAt(inicio), this.blankSetup2));
          int fim = this.infoFeature._feature.size() -
              this.infoDOS.infoSuperficieDelimitadoraExterna.
              indiceDaFeature;

          System.out.println("SEGUNDO SETUP EXTERIOR -> inicio: " + inicio + " fim: " + fim);

          fuTmp.add(getFuBloco(this.infoFeature._featureInvertido, inicio, inicio + 1, fim, this.blankSetup2));

          Vector fuEspeciaisExternas = getFuEspeciais(this.infoFeature.
              _featureInvertido, inicio, fim);
          if (fuEspeciaisExternas != null) {
            int sizeExt = fuEspeciaisExternas.size();
            for (int j = 0; j < sizeExt; j++) {
              fuTmp.add(fuEspeciaisExternas.elementAt(j));
            }
          }

          //Features de usinagem do eixo C
          Vector fuEixoC = getFuEixoC(this.infoFeature._featureInvertido, this.infoFeature._featureEixoCInvertido,
                                      inicio, fim);
          if (fuEixoC != null) {
            int sizeEixoC = fuEixoC.size();
            for (int j = 0; j < sizeEixoC; j++) {
              fuTmp.add(fuEixoC.elementAt(j));
            }
          }

        }
        else { //Tem Interior
          //Parte Externa

          //Cria o furo
          if(this.furoSetup2)
            fuTmp.add(getFuFuracao(this.blankSetup2.D1, this.blankSetup2.L1));

          //Cria a Fu de Faceamento
          int inicioExterior = this.infoFeature._feature.size() -
              this.infoFeature.indiceFeaturesInternas;
          fuTmp.add(getFuFaceamento( (Feature)this.infoFeature.
                                    _featureInvertido.elementAt(inicioExterior), this.blankSetup2));

         //Cria a Fu de Torneamento Externa
          int fimExterior = this.infoFeature._feature.size() -
              this.infoDOS.infoSuperficieDelimitadoraExterna.indiceDaFeature;

          System.out.println("SEGUNDO SETUP EXTERIOR -> inicio: " + inicioExterior + " fim: " + fimExterior);

          fuTmp.add(getFuBloco(this.infoFeature._featureInvertido,
                               inicioExterior, inicioExterior + 1, fimExterior, blankSetup2));

         //Encontra as Fu Especiais Externas
          Vector fuEspeciaisExternas = getFuEspeciais(this.infoFeature.
              _featureInvertido, inicioExterior, fimExterior);
          if (fuEspeciaisExternas != null) {
            int sizeExt = fuEspeciaisExternas.size();
            for (int j = 0; j < sizeExt; j++) {
              fuTmp.add(fuEspeciaisExternas.elementAt(j));
            }
          }

          //Features de usinagem do eixo C
          Vector fuEixoC = getFuEixoC(this.infoFeature._featureInvertido,
                                      this.infoFeature._featureEixoCInvertido,
                                      inicioExterior, fimExterior);
          if (fuEixoC != null) {
            int sizeEixoC = fuEixoC.size();
            for(int j = 0; j < sizeEixoC; j++){
              fuTmp.add(fuEixoC.elementAt(j));
            }
          }

          //Parte Interna
          int inicioInterior = this.infoFeature.indiceFeaturesInternas;
          int fimInterior;

          /**
           * Tem que verificar se a ultima feature da parte interna do segundo
           * setup possui diametro inicial diferentes de zero.
           * D1 != 0 -> so existe interior no segundo setup
           * D1 == 0 -> existe interior no primeiro setup tb
           * Para cada um dos casos o fim do perfil sera diferente.
           */
          /**
           * Para calcular o final e necessario ver se os setups tem blanks diferentes,
           * pois caso isso seja verdade a ultima feature do perfil interno devera ser
           * descartaja, uma vez que ela ja foi resolvida pelo furo.
           *
           * != -> retira a ultima feature
           * == -> faz como deveria ser feito
           */
          if ( ( (Feature)this.infoFeature._feature.elementAt(this.infoDOS.
              infoSuperficieDelimitadoraInterna.indiceDaFeature - 1)).D1 != 0){
            if(!MyMath.comparaDiametrosDeBlanks(this.blankSetup1, this.blankSetup2)){
              System.out.println("D1 != 0 e blanks diferentes");
              fimInterior = this.infoDOS.infoSuperficieDelimitadoraInterna.
                  indiceDaFeature - 1;
            }
            else{
              System.out.println("D1 != 0 e blanks iguais");
              fimInterior = this.infoDOS.infoSuperficieDelimitadoraInterna.
                  indiceDaFeature;
            }
          }
          else{
            if(!MyMath.comparaDiametrosDeBlanks(this.blankSetup1, this.blankSetup2)){
              System.out.println("D1 == 0 e blanks diferentes");
              fimInterior = this.infoDOS.infoSuperficieDelimitadoraInterna.
                  indiceDaFeature - 1 - 1;//um "-1" e pq existe parte interna no primeiro setup, outra e pq o furo resolveu o a ultima feature
            }
            else{
              System.out.println("D1 == 0 e blanks iguais");
              fimInterior = this.infoDOS.infoSuperficieDelimitadoraInterna.
                  indiceDaFeature - 1;
            }
          }

          System.out.println("SEGUNDO SETUP INTERIOR -> inicio: " + inicioInterior + " fim: " + fimInterior);

          Vector interiorTmp = new Vector();
          for (int j = inicioInterior; j < fimInterior; j++) {
            interiorTmp.add(this.infoFeature._feature.elementAt(j));
          }

          double maxDiametroInterno = EspelhoDeFeatures.encontrarMaiorDiametroInterno(
              interiorTmp);
          interiorTmp = EspelhoDeFeatures.espelharVerticalmenteVetor(interiorTmp,
              maxDiametroInterno);

          this.blankSetup2 = EspelhoDeFeatures.espelharBlank(this.blankSetup2,
                                                  maxDiametroInterno);
          double D1Tmp = this.blankSetup2.D1;
          this.blankSetup2.D1 = this.blankSetup2.D2;
          this.blankSetup2.D2 = D1Tmp;


          //Cria a Fu de Torneamento Interna
          Vector fuInternasTmp = new Vector();
          fuInternasTmp.add(getFuBloco(this.infoFeature._feature,
                                       inicioInterior, inicioInterior,
                                       fimInterior, this.blankSetup2));

          //Encontra as Fu Especiais Internas
          Vector fuEspeciaisInternas = getFuEspeciais(this.infoFeature.
              _feature, inicioInterior, fimInterior);
          if (fuEspeciaisInternas != null) {
            int sizeInt = fuEspeciaisInternas.size();
            for (int j = 0; j < sizeInt; j++) {
              fuInternasTmp.add(fuEspeciaisInternas.elementAt(j));
            }
          }

          interiorTmp = EspelhoDeFeatures.espelharVerticalmenteVetor(interiorTmp,
              maxDiametroInterno);

          this.blankSetup2 = EspelhoDeFeatures.espelharBlank(this.blankSetup2,
                                                  maxDiametroInterno);
          D1Tmp = this.blankSetup2.D1;
          this.blankSetup2.D1 = this.blankSetup2.D2;
          this.blankSetup2.D2 = D1Tmp;


          /**
           * Tem que inverter todas as features nulas que estao presentes
           * nas features de usinagem.
           * Depois adiciona as features de usinagem que foram encontradas
           * no interior ao vetor de features de usinagem do setup.
           */
          int sizeFuTmp = fuInternasTmp.size();
          for (int k = 0; k < sizeFuTmp; k++) {
            FeatureDeUsinagem tmpK = (FeatureDeUsinagem) fuInternasTmp.
                elementAt(k);

            int sizeL = tmpK._featuresDaFU.size();
            for (int l = 0; l < sizeL; l++) {
              Feature tmpL = (Feature) tmpK._featuresDaFU.elementAt(l);
              if (tmpL.Tipo == 0) {
                tmpL.D1 = maxDiametroInterno - tmpL.D1;
                tmpL.D2 = maxDiametroInterno - tmpL.D2;
              }
            }

            /**
             * A origem tambem tem que ser invertida.
             */
            tmpK.origem.x = maxDiametroInterno - tmpK.origem.x;

            fuTmp.add(tmpK);
          }

          //Features de usinagem do eixo C
          fuEixoC = getFuEixoC(this.infoFeature._feature,
                               this.infoFeature._featureEixoC, inicioInterior,
                               fimInterior);
          if (fuEixoC != null) {
            int sizeEixoC = fuEixoC.size();
            for (int j = 0; j < sizeEixoC; j++) {
              fuTmp.add(fuEixoC.elementAt(j));
            }
          }

        }
      }

      //Coloca as Features de Usinagem de Eixo C por ultimo no vetor
      Vector fuConcentricas = new Vector();
      Vector fuEixoC = new Vector();
      for (int j = 0; j < fuTmp.size(); j++) {
        FeatureDeUsinagem fuTmp2 = (FeatureDeUsinagem) fuTmp.elementAt(
            j);
        if (fuTmp2.tipo == FeatureDeUsinagem.PADRAO_DE_FURO ||
            fuTmp2.tipo == FeatureDeUsinagem.RASGO_DE_CHAVETA)
          fuEixoC.add(fuTmp2);
        else
          fuConcentricas.add(fuTmp2);
      }

      Vector fuFinal = new Vector();
      for (int j = 0; j < fuConcentricas.size(); j++) {
        fuFinal.add(fuConcentricas.elementAt(j));
      }
      for (int k = 0; k < fuEixoC.size(); k++) {
        fuFinal.add(fuEixoC.elementAt(k));
      }

      fuTmp = fuFinal;

      fuDoSetupNTmp.add(fuTmp);
    }

    return fuDoSetupNTmp;
  }

  /**
   * Cria uma feature de usinagem do tipo furo a partir de um diamentro e de
   * um comprimento de entrada.
   *
   * @param diametro double
   * @param comprimento double
   * @return FeatureDeUsinagem
   */
  public FeatureDeUsinagem getFuFuracao(double diametro, double comprimento){
    FeatureDeUsinagem furo = new FeatureDeUsinagem();

    Feature fNull_1 = new Feature();
    fNull_1.L1 = comprimento;
    fNull_1.D1 = diametro;
    fNull_1.D2 = fNull_1.D1;

    Feature fNull_2 = new Feature();
    fNull_2.L1 = 0;
    fNull_2.D1 = diametro;
    fNull_2.D2 = 0;

    Vector _featuresDaFU = new Vector();
    _featuresDaFU.add(fNull_1);
    _featuresDaFU.add(fNull_2);

    furo.setFeaturesDaFU(_featuresDaFU);
    furo.setOrigem(new Ponto2D(0.0, 0.0));

    if(diametro < 20)
      furo.setTipo(FeatureDeUsinagem.FURO_ACIONADO);
    else
      furo.setTipo(FeatureDeUsinagem.FURO);

    return furo;
  }

  /**
   * Cria uma feature de usinagem do tipo faceamento, considerando o fato
   * da pea possuir ou nao parte interna.
   *
   * @param face Feature
   * @return FeatureDeUsinagem
   */
  public FeatureDeUsinagem getFuFaceamento(Feature face, Blank blankDoSetup) {
    FeatureDeUsinagem faceamento;
    Vector _featuresDaFU = new Vector();
    Ponto2D origem = new Ponto2D(0.0, 0.0);

    if (! (face.Tipo == Feature.FACECONCAVE || face.Tipo == Feature.FACECONVEX ||
           face.Tipo == Feature.FACESTRAIGHT ||
           face.Tipo == Feature.FACETAPERED)) {
      MyMath.alert("Verifique as features de face sao:\nFACECONCAVE\nFACECONVEX\nFACESTRAIGHT\nFACETAPERED");
    }

    if (face.D1 != 0) { //implica a existencia de parte interna, nem que seja somente um furo
      origem.x = blankDoSetup.D1;

      Feature fNull_1 = new Feature();
      fNull_1.D1 = blankDoSetup.D1;
      fNull_1.D2 = face.D1;

      _featuresDaFU.add(fNull_1);
    }

    _featuresDaFU.add(face);

    Feature fNull_2 = new Feature();
    fNull_2.D1 = face.D2;
    fNull_2.D2 = blankDoSetup.D2;
    _featuresDaFU.add(fNull_2);

    Feature fNull_3 = new Feature();
    fNull_3.D1 = blankDoSetup.D2;
    fNull_3.D2 = fNull_3.D1;
    fNull_3.L1 = - (face.L1 + getSobreMaterialTotal(face)); //TALVES DE ERRO QUANDO FOR UM FACETAPERED...
    _featuresDaFU.add(fNull_3);

    faceamento = new FeatureDeUsinagem(_featuresDaFU, origem);
    faceamento.setTipo(FeatureDeUsinagem.FACE);

    return faceamento;
  }

  /**
   * Cria uma feature de usinagem do tipo torneamento externo. Caso se deseje uma
   * do tipo torneamento interno, esta alteracao devera ser feita fora desta funcao!
   *
   *
   * @param _feature Vector
   * @param inicioPerfil int
   * @param inicio int
   * @param fim int
   * @return FeatureDeUsinagem
   */
  public FeatureDeUsinagem getFuBloco(Vector _feature, int inicioPerfil,
                                      int inicio, int fim, Blank blankSetup) {
    FeatureDeUsinagem bloco;

    Vector _featuresDaFU = new Vector();
    Feature tmp = null;
    double x = 0.0, z = 0.0;

    for (int i = inicioPerfil; i < fim; i++) {
      tmp = (Feature) _feature.elementAt(i);
      if (i < inicio) z += tmp.L1; //encontra o Z
      if (i == inicio) x = tmp.D1; //encontra o X
      if (i >= inicio) { //percorre a parte da FeatureDeUsinagem
        if (tmp.Tipo == Feature.ODCONTOURED || tmp.Tipo == Feature.IDCONTOURED ||
            tmp.Tipo == Feature.FACECONTOURED) {
          MyMath.alert(
              "Encontrou uma SPLINE.\nFalta fazer o reconhecimento da FU");
        }
        if (tmp.D2 >= tmp.D1) {
          if (! (isGroove(tmp) || isGrooveComplexo(tmp))) { //caso a feature nao seja um groove
            _featuresDaFU.add(tmp);
          }
          else { //adiciona uma feature nula caso seja um groove
            Feature fNull = new Feature();
            fNull.D1 = tmp.D1;
            fNull.D2 = tmp.D2;
            fNull.L1 = tmp.L1;
            _featuresDaFU.add(fNull);
          }
        }
      }
    }

    System.out.println("      " + tmp.D2);
    //Adicionando a feature nula
    Feature fNull_final = new Feature();
    fNull_final.D1 = tmp.D2;
    fNull_final.D2 = blankSetup.D2;
    fNull_final.L1 = 0.0;
    _featuresDaFU.add(fNull_final);

    //Criando a FeatureDeUsinagem
    bloco = new FeatureDeUsinagem(_featuresDaFU, new Ponto2D(x, z));

    if(MyMath.isExterno(bloco))
      bloco.setTipo(FeatureDeUsinagem.BLOCO_EXTERNO);
    else
      bloco.setTipo(FeatureDeUsinagem.BLOCO_INTERNO);

    return bloco;
  }

  /**
   * Encontra todas as features especiais presentes no projeto, retornando um vetor
   * que devera ser trabalhado na funcao que esta num nivel superior ao desta.
   *
   *
   * @param _feature Vector
   * @param inicio int
   * @param fim int
   * @return Vector
   */
  public Vector getFuEspeciais(Vector _feature, int inicio, int fim) {
    Vector featuresEspeciais = null;

    Feature tmp;
    FeatureDeUsinagem fuTmp;
    double z = 0.0;
    for (int i = 0; i < fim; i++) { //procura grooves simples e grooves complexos
      tmp = (Feature) _feature.elementAt(i);
      if (i >= inicio) {
        if (isGrooveComplexo(tmp) || isGroove(tmp)) {
          fuTmp = new FeatureDeUsinagem();

          Vector _featuresDaFU = new Vector();
          _featuresDaFU.add(tmp);
          fuTmp.setFeaturesDaFU(_featuresDaFU);

          Ponto2D origem = new Ponto2D(tmp.D1, z);
          fuTmp.setOrigem(origem);

          if (tmp.Tipo == Feature.GROOVE_COMPLEXO){
            if(MyMath.isExterno(fuTmp))
              fuTmp.setTipo(FeatureDeUsinagem.GROOVECOMPLEXO_EXTERNO);
            else
              fuTmp.setTipo(FeatureDeUsinagem.GROOVECOMPLEXO_INTERNO);
          }
          else{
            if(MyMath.isExterno(fuTmp))
              fuTmp.setTipo(FeatureDeUsinagem.GROOVE_EXTERNO);
            else
              fuTmp.setTipo(FeatureDeUsinagem.GROOVE_INTERNO);
          }

          if (featuresEspeciais == null)
            featuresEspeciais = new Vector();
          featuresEspeciais.add(fuTmp);
        }
        z += tmp.L1;
      }
    }

    z = 0.0;
    for (int i = 0; i < fim; i++) { //procura roscas
      tmp = (Feature) _feature.elementAt(i);
      if (i >= inicio) {
        if (temRosca(tmp)) {
          fuTmp = new FeatureDeUsinagem();

          Vector _featuresDaFU = new Vector();
          _featuresDaFU.add(tmp);
          fuTmp.setFeaturesDaFU(_featuresDaFU);

          Ponto2D origem = new Ponto2D(tmp.D1, z);
          fuTmp.setOrigem(origem);

          if(MyMath.isExterno(fuTmp))
            fuTmp.setTipo(FeatureDeUsinagem.ROSCA_EXTERNA);
          else
            fuTmp.setTipo(FeatureDeUsinagem.ROSCA_INTERNA);

          if (featuresEspeciais == null)
            featuresEspeciais = new Vector();
          featuresEspeciais.add(fuTmp);
        }
        z += tmp.L1;
      }
    }

    z = 0.0;
    for (int i = 0; i < fim; i++) { //procura recartilhado
      tmp = (Feature) _feature.elementAt(i);
      if (i >= inicio) {
        if (temRecartilhado(tmp)) {
          fuTmp = new FeatureDeUsinagem();
          fuTmp.setTipo(FeatureDeUsinagem.RECARTILHADO);
          Vector _featuresDaFU = new Vector();
          _featuresDaFU.add(tmp);
          fuTmp.setFeaturesDaFU(_featuresDaFU);

          Ponto2D origem = new Ponto2D(tmp.D1, z);
          fuTmp.setOrigem(origem);

          if (featuresEspeciais == null)
            featuresEspeciais = new Vector();
          featuresEspeciais.add(fuTmp);
        }
        z += tmp.L1;
      }
    }

    z = 0.0;
    for(int i = 0; i < fim; i++){//procura por splines
      tmp = (Feature) _feature.elementAt(i);
      if(i >= inicio){
        if(tmp.Tipo == Feature.ODSPLINE_VIRTUAL){
          fuTmp = new FeatureDeUsinagem();
          fuTmp.setTipo(FeatureDeUsinagem.SPLINE_EXTERNA);
          Vector _featuresDaFU = new Vector();
          _featuresDaFU.add(tmp);
          fuTmp.setFeaturesDaFU(_featuresDaFU);

          ODSplineVirtual osvTmp = (ODSplineVirtual)tmp;

          Ponto2D origem = new Ponto2D(osvTmp.D1Original, z);
          fuTmp.setOrigem(origem);

          if (featuresEspeciais == null)
            featuresEspeciais = new Vector();
          featuresEspeciais.add(fuTmp);
        }
        else if(tmp.Tipo == Feature.IDSPLINE_VIRTUAL){
          fuTmp = new FeatureDeUsinagem();
          fuTmp.setTipo(FeatureDeUsinagem.SPLINE_INTERNA);
          Vector _featuresDaFU = new Vector();
          _featuresDaFU.add(tmp);
          fuTmp.setFeaturesDaFU(_featuresDaFU);

          IDSplineVirtual isvTmp = (IDSplineVirtual)tmp;

          Ponto2D origem = new Ponto2D(isvTmp.D1Original, z);
          fuTmp.setOrigem(origem);

          if (featuresEspeciais == null)
            featuresEspeciais = new Vector();
          featuresEspeciais.add(fuTmp);
        }
        z += tmp.L1;
      }
    }

    return featuresEspeciais;
  }

  /**
   *
   * @param inicio int
   * @param fim int
   * @return Vector
   */
  public Vector getFuEixoC(Vector feature, Vector eixoC, int inicio, int fim){
    Vector fuEixoC = null;
    double z = 0.0;
    double x = 0.0;

    if(eixoC != null){
      int size = eixoC.size();
      System.out.println("\n\nExistem " + size + " features no EIXO C:");
      System.out.println("Inicio: " + inicio);
      System.out.println("Fim: " + fim);
      for(int i = 0; i < size; i++){
        featureEixoC tmp = (featureEixoC)eixoC.elementAt(i);
        if(tmp.indiceMae-1 >= inicio && tmp.indiceMae-1 < fim){
          if(fuEixoC == null) fuEixoC = new Vector();
          FeatureDeUsinagem fuTmp;
          Vector _featuresDaFU;
          switch (tmp.tipo){
            case featureEixoC.KEYWAY:
              System.out.println("\tKeyWay " + tmp.indiceMae);
              KeyWay kwTmp = (KeyWay)tmp;
              fuTmp = new FeatureDeUsinagem();
              fuTmp.setTipo(FeatureDeUsinagem.RASGO_DE_CHAVETA);
              _featuresDaFU = new Vector();
              _featuresDaFU.add(kwTmp);
              fuTmp.setFeaturesDaFU(_featuresDaFU);

              z = 0.0;
              for (int j = inicio; j < fim; j++) {
                if (j < tmp.indiceMae -1)
                  z += ( (Feature) feature.elementAt(j)).L1;
              }
              x = ((Feature)feature.elementAt(tmp.indiceMae-1)).D2;// - kwTmp.d1;
              fuTmp.setOrigem(new Ponto2D(x, z));

              if (fuEixoC == null)
                fuEixoC = new Vector();
              fuEixoC.add(fuTmp);
              break;
            case featureEixoC.HOLE_PATTERN:
              System.out.println("\tHole_Pattern " + tmp.indiceMae);
              HolePattern hpTmp = (HolePattern)tmp;
              fuTmp = new FeatureDeUsinagem();
              fuTmp.setTipo(FeatureDeUsinagem.PADRAO_DE_FURO);
              _featuresDaFU = new Vector();
              _featuresDaFU.add(hpTmp);
              fuTmp.setFeaturesDaFU(_featuresDaFU);

              z = 0.0;
              for(int j = inicio; j < fim; j++){
                if(j < tmp.indiceMae)
                  z += ((Feature)feature.elementAt(j)).L1;
              }
              MyMath.alert("" + hpTmp.diametro);
              x = hpTmp.diametro;
              fuTmp.setOrigem(new Ponto2D(x, z));

              if (fuEixoC == null)
                fuEixoC = new Vector();
              fuEixoC.add(fuTmp);
              break;
            default:
              MyMath.alert("Tentando criar uma feature de usinagem\npara uma feature do eixoC nao\nconhecida.");
              break;
          }
        }
      }
    }
    System.out.println("\n\n");

    return fuEixoC;
  }

  /**
   * Verifica se uma dada feature e um groove.
   *
   * @param f Feature
   * @return boolean
   */
  private boolean isGroove(Feature f) {
    if (f.Tipo == Feature.GROOVE_FACE_RADIAL ||
        f.Tipo == Feature.GROOVE_FACE_SQUARE ||
        f.Tipo == Feature.GROOVE_FACE_TAPER
        || f.Tipo == Feature.GROOVE_I_HELICAL ||
        f.Tipo == Feature.GROOVE_I_RADIAL || f.Tipo == Feature.GROOVE_I_RELIEF
        || f.Tipo == Feature.GROOVE_I_SQUARE ||
        f.Tipo == Feature.GROOVE_I_TAPER || f.Tipo == Feature.GROOVEHELICAL
        || f.Tipo == Feature.GROOVERADIAL || f.Tipo == Feature.GROOVERELIEF ||
        f.Tipo == Feature.GROOVESQUARE
        || f.Tipo == Feature.GROOVETAPER) {
      return true;
    }

    return false;
  }

  /**
   * Verifica se uma determinada feature e um groove complexo. Este teste esta
   * separado do teste "isGroove", pois as features de usinagem e consequentemente
   * os workingsteps criados para cada um dos casos sao diferentes.
   *
   * @param f Feature
   * @return boolean
   */
  private boolean isGrooveComplexo(Feature f) {
    return (f.Tipo == Feature.GROOVE_COMPLEXO);
  }

  /**
   * Verifica a existencia de uma rosca numa dada feature. Sendo que so existem
   * roscas em ODStraigh, ODTapered e as respectivas features internas.
   *
   * @param f Feature
   * @return boolean
   */
  private boolean temRosca(Feature f) {
    if (f.Tipo == Feature.ODSTRAIGHT) {
      ODStraight tmp = (ODStraight) f;
      if (tmp.rosca != null) return true;
      else return false;
      //return tmp.TemRosca;
    }
    else if (f.Tipo == Feature.ODTAPERED) {
      ODTapered tmp = (ODTapered) f;
      return tmp.TemRosca;
    }
    else if (f.Tipo == Feature.IDSTRAIGHT) {
      IDStraight tmp = (IDStraight) f;
      return tmp.TemRosca;
    }
    else if (f.Tipo == Feature.IDTAPERED) {
      IDTapered tmp = (IDTapered) f;
      return tmp.TemRosca;
    }
    else {
      return false;
    }
  }

  /**
   * Verifica a existencia de recartilhado numa dada feature. Sendo que o
   * recartilhado so existe em ODStraight e ODTapered.
   *
   * @param f Feature
   * @return boolean
   */
  private boolean temRecartilhado(Feature f) {
    if (f.Tipo == Feature.ODSTRAIGHT) {
      ODStraight tmp = (ODStraight) f;
      return tmp.TemRecartilhado;
    }
    else if (f.Tipo == Feature.ODTAPERED) {
      ODTapered tmp = (ODTapered) f;
      return tmp.TemRecartilhado;
    }
    else {
      return false;
    }
  }

  /**
   * Cria a InfoDecomposicaoOrientadaAGeometria
   *
   * @return InfoDecomposicaoOrientadaAGeometria
   */
  public InfoDecomposicaoOrientadaAGeometria
      getInfoDecomposicaoOrientadaAGeometria() {
    if (this.infoDOG == null) {
      infoDOG = new InfoDecomposicaoOrientadaAGeometria();
      infoDOG.setFuDoSetupN(this.fuDoSetupN);
      infoDOG.blankUtilizado = this.blankUtilizado;
    }

    return this.infoDOG;
  }

  /**
   * Imprime os dados referentes a InfoDecomposicaoOrientadaAGeometria
   */
  public void imprime() {
    Vector tmp;
    boolean concentrica = true;

    System.out.println("\n##############################################");
    System.out.println("DECOMPOSICAO ORIENTADA A GEOMETRIA:");
    System.out.println("\n\tBlank Necessario:");
    System.out.println("\t\tD1: " + this.blankNecessario.D1 + "\n\t\tD2: " +
                       this.blankNecessario.D2 + "\n\t\tL1: " +
                       this.blankNecessario.L1);
    System.out.println("\n\tBlank Utilizado (Existente no estoque):");
    System.out.println("\t\tD1: " + this.blankUtilizado.D1 + "\n\t\tD2: " +
                       this.blankUtilizado.D2 + "\n\t\tL1: " +
                       this.blankUtilizado.L1);
    System.out.println("\n\tBlank Setup1:");
    System.out.println("\t\tD1: " + this.blankSetup1.D1 + "\n\t\tD2: " +
                       this.blankSetup1.D2 + "\n\t\tL1: " +
                       this.blankSetup1.L1);
    System.out.println("\n\tBlank Setup2:");
    System.out.println("\t\tD1: " + this.blankSetup2.D1 + "\n\t\tD2: " +
                       this.blankSetup2.D2 + "\n\t\tL1: " +
                       this.blankSetup2.L1);


    int sizeI = this.fuDoSetupN.size();
    for (int i = 0; i < sizeI; i++) {
      System.out.println("\n\tSETUP numero: " + (i + 1));
      tmp = (Vector)this.fuDoSetupN.elementAt(i);
      int sizeJ = tmp.size();
      for (int j = 0; j < sizeJ; j++) {
        System.out.println("\t\tFEATURE DE USINAGEM numero: " + (j + 1));
        FeatureDeUsinagem tmpJ = (FeatureDeUsinagem) tmp.elementAt(j);
        System.out.print("\t\t\tTipo: ");
        switch (tmpJ.tipo){
          case FeatureDeUsinagem.BLOCO_EXTERNO:
            System.out.println("Bloco Externo");
            concentrica = true;
            break;
          case FeatureDeUsinagem.BLOCO_INTERNO:
            System.out.println("Bloco Interno");
            concentrica = true;
            break;
          case FeatureDeUsinagem.FACE:
            System.out.println("Face");
            concentrica = true;
            break;
          case FeatureDeUsinagem.FURO:
            System.out.println("Furo");
            concentrica = true;
            break;
          case FeatureDeUsinagem.GROOVE_EXTERNO:
            System.out.println("Groove Externo");
            concentrica = true;
            break;
          case FeatureDeUsinagem.GROOVE_INTERNO:
            System.out.println("Groove Interno");
            concentrica = true;
            break;
          case FeatureDeUsinagem.GROOVECOMPLEXO_EXTERNO:
            System.out.println("GrooveComplexo Externo");
            concentrica = true;
            break;
          case FeatureDeUsinagem.GROOVECOMPLEXO_INTERNO:
            System.out.println("GrooveComplexo Interno");
            concentrica = true;
            break;
          case FeatureDeUsinagem.RECARTILHADO:
            System.out.println("Recartilhado");
            concentrica = true;
            break;
          case FeatureDeUsinagem.ROSCA_EXTERNA:
            System.out.println("Rosca Externa");
            concentrica = true;
            break;
          case FeatureDeUsinagem.ROSCA_INTERNA:
            System.out.println("Rosca Interna");
            concentrica = true;
            break;
          case FeatureDeUsinagem.SPLINE_EXTERNA:
            System.out.println("Spline Externa");
            concentrica = true;
            break;
          case FeatureDeUsinagem.SPLINE_INTERNA:
            System.out.println("Spline Interna");
            concentrica = true;
            break;
          case FeatureDeUsinagem.RASGO_DE_CHAVETA:
            System.out.println("Rasgo de Chaveta");
            concentrica = false;
            break;
          case FeatureDeUsinagem.PADRAO_DE_FURO:
            System.out.println("Padrao de Furos");
            concentrica = false;
            break;
          default:
            System.out.println("Nao Tratada");
            break;
        }
        System.out.println("\t\t\tOrigem: Z= " + tmpJ.origem.z + " X= " +
                           tmpJ.origem.x);
        Vector FUj = tmpJ._featuresDaFU;
        System.out.println("\t\t\tFeatures:");
        if(concentrica){
          int sizeK = FUj.size();
          for (int k = 0; k < sizeK; k++) {
            Feature tmpK = (Feature) FUj.elementAt(k);
            System.out.println("\t\t\t\tTipo: " + tmpK.Tipo);
            if (tmpK.Tipo == Feature.GROOVE_COMPLEXO) {
              GrooveComplexo gCTmp = (GrooveComplexo) tmpK;
              System.out.println("\t\t\t\t\tFeatures incluidas:");
              MyMath.imprimeVetorDeFeatures(gCTmp.features, "\t\t\t\t");
            }

            System.out.println("\t\t\t\t\tL1: " + tmpK.L1);
            System.out.println("\t\t\t\t\tD1: " + tmpK.D1);
            System.out.println("\t\t\t\t\tD2: " + tmpK.D2);
          }
        }
        else{
          int sizeK = FUj.size();

        }

      }
    }

    System.out.println("##############################################\n");
  }

}
