package webcapp_01_0_1;

import java.util.Vector;
import features.*;
import util.*;


public class WebCAPP {
  public static final int PECA_CURTA = 1;
  public static final int PECA_INTERMEDIARIA = 2;
  public static final int EIXO = 3;

  public Vector _featureOriginal; //vetor de feature criado no WebCAD

  public InfoFeature infoFeature;

  public DecomposicaoOrientadaASetup decomposicaoOrientadaASetup;
  public InfoDecomposicaoOrientadaASetup infoDOS;

  public DecomposicaoOrientadaAGeometria decomposicaoOrientadaAGeometria;
  public InfoDecomposicaoOrientadaAGeometria infoDOG;

  public DecomposicaoOrientadaAOperacao decomposicaoOrientadaAOperacao;
  public InfoDecomposicaoOrientadaAOperacao infoDOO;

  private double LPeca;
  private String tipoDePeca;
  /**
   * Magno 20050921 Foi tornado global para ser acessado do mtodo geraArquivoXML
   */
  private CodigoG geracaoDeCodigoG;

  /**
   *
   * @param _featureOriginal Vector
   * @param _featureEixoCOriginal Vector
   */
  public WebCAPP(Vector _featureOriginal, Vector _featureEixoCOriginal) {
    this.infoFeature = new InfoFeature(_featureOriginal, _featureEixoCOriginal);

    init();
  }

  public void init() {

    double RelacaoLD = relacaoLpecaDpeca(this.infoFeature._feature);

    switch (funcaoEstabilidade(RelacaoLD)) {
      case EIXO:
        tipoDePeca = "Eixo";
        break;
      case PECA_CURTA:
        tipoDePeca = "Peca Curta";
        break;
      case PECA_INTERMEDIARIA:
        tipoDePeca = "Peca intermediaria";
        break;
    }
    System.out.println("Tipo de peca: " + tipoDePeca);

    //Decomposicao Orientada a Setup
    decomposicaoOrientadaASetup = new DecomposicaoOrientadaASetup(this.infoFeature);
    this.infoDOS = decomposicaoOrientadaASetup.getInfoDecomposicaoOrientadaASetup();

    //Decomposicao Orientada a Geometria
    decomposicaoOrientadaAGeometria = new DecomposicaoOrientadaAGeometria(this.infoFeature, this.infoDOS);
    this.infoDOG = decomposicaoOrientadaAGeometria.getInfoDecomposicaoOrientadaAGeometria();

    //Decomposicao Orientada a Operacao
    decomposicaoOrientadaAOperacao = new DecomposicaoOrientadaAOperacao(this.infoFeature, this.infoDOG);
    this.infoDOO = decomposicaoOrientadaAOperacao.getInfoDecomposicaoOrientadaAOperacao();

    //Gera o codigo G do projeto
    geracaoDeCodigoG = new CodigoG(this.infoDOO);
    System.out.println(geracaoDeCodigoG.getCodigoGTxT(geracaoDeCodigoG.codigoG));

  }


  /**
   *
   *
   * @param _feature Vector
   * @return double
   */
  private double relacaoLpecaDpeca(Vector _feature) {
    int size = _feature.size();
    Feature tmp;
    double DPeca = 0;
    double LPeca = 0;
    for (int i = 0; i < size; i++) {
      tmp = (Feature) _feature.elementAt(i);
      if (DPeca < tmp.D2) {
        DPeca = tmp.D2;
      }
      LPeca += tmp.L1;
    }
    this.LPeca = LPeca;
    return (LPeca / DPeca);
  }

  /**
   *
   * @param RelacaoLD double
   * @return int
   */
  private int funcaoEstabilidade(double RelacaoLD) {
    //Funcao Estabilidade
    if (RelacaoLD > 4) {
      //Eixo
      return EIXO;
    }
    else if (RelacaoLD < 2) {
      //Peca Curta
      return PECA_CURTA;
    }
    else {
      //(2<=RelacaoLD<=4)
      return PECA_INTERMEDIARIA;
    }
  }
  /**
   * Mtodo para gerar o arquivo XML de sada.
   * @param nomeDoArquivoXML String
   */
  public void geraArquivoXML(String nomeDoArquivoXML){
    String texto= XMLcomunicator.XMLDecomposicaoOrientadaAOperacao(infoDOO,XMLcomunicator.codigoGTXTparaXML(geracaoDeCodigoG.getCodigoGTxT(geracaoDeCodigoG.codigoG)));
    String filepath="capp"+nomeDoArquivoXML+".xml";

    XMLcomunicator.geraArquivo(filepath,texto);

  }

}
